///////////////////////////////////////////////////////////////////////////////	
//    Ashtech receiver tools
//    Copyright (C) 1999  Magellan Coropration/Ashtech Precision Products 
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
///////////////////////////////////////////////////////////////////////////////


#ifndef __TRANS_HPP
#include "trans.hpp"
#endif

void swap_nbytes( char *head, short count )
{{
  char *tail = head + count - 1;
  char tmp_byte;

  while ( (unsigned long) head < (unsigned long) tail )
  { tmp_byte = *head;
    *head = *tail;
    *tail = tmp_byte;
    head ++;
    tail --;
  }
}}


//	===================================================================
//	NAME:		swap_bytes
//
//	PURPOSE:	To Swap The Most Significant Byte With The Least
//				Significant Byte (To Convert From The Intel Format To
//				The Motorola Format And Visa-Versa)
//
//	RETURNS:	unsigned short 	- The Resulting Integer After The Swap
//
//	HISTORY:
//	===================================================================
unsigned short swap_bytes (unsigned short bytes)
{{
  return ( (bytes << 8) + (bytes >> 8));
}}

void Trans::extract_ulong (short start_bit, short end_bit, long *result)
{{
  short         start_byte;
  short         end_byte;
  short         first_bit;
  short         last_bit;
  short         i;
  unsigned char temp_char;

  start_byte = (start_bit-1) / 8;
  end_byte   = (end_bit-1) / 8;
  *result = 0;

  first_bit = start_bit - (start_byte) * 8;
  if (end_byte == start_byte)
  { last_bit = end_bit - (end_byte) * 8;
    *result=(unsigned char)(bf.ibf[start_byte]) >> (8-last_bit);
    for (i = 8; i > (end_bit - start_bit) + 1; i--)
      *result &= ~bit[i-1];
  }
  else
  { /* Extract Bits from first byte */
    temp_char = bf.ibf[start_byte];
    for (i = 1; i < first_bit; i++) temp_char &= ~bit[8-i];
    *result = temp_char;

    /* Extract Bits from middle bytes */
    for (i = start_byte+1; i < end_byte; i++)
    { *result <<= 8;
      *result += bf.ibf[i];
    }

    /* Extract Bits from last byte */
    last_bit = end_bit - (end_byte * 8);
    *result <<= (last_bit);
    *result +=(unsigned char)bf.ibf[end_byte] >> (8 - last_bit);
  }
}}

long Trans::extract_long (short *start_bit, short total_bits, short mode)
{{
  short         start_byte;
  short         first_bit;
  long          result;
  short         end_bit;
  unsigned long power;
  short         i;

  end_bit = *start_bit + total_bits - 1;
  extract_ulong (*start_bit, end_bit, &result);
  if (mode == F_SIGNED)
  { start_byte = (*start_bit-1)/8;
    first_bit = *start_bit - (start_byte) * 8;
    if (bf.ibf[start_byte] & bit[8 - first_bit])
    { power = 2147483648L;
      for (i = 32; i > total_bits ; i--)
      { result |= power;
				power >>= 1;
      }
    }
  }
  *start_bit += total_bits;
  return (result);
}}

//	===================================================================
//	NAME:		pack_word_bits
//
//	PURPOSE:	To Pack The Bits To Be Right Justified 30-Bit Words
//				Stored In 32-Bit Units. (They Come From The Receiver In
//				Two 16-Bit Units With 15 Right Justified Bits In Each
//
//	RETURNS:	void
//
//	HISTORY:
//	===================================================================
void pack_word_bits(unsigned char *page,short word_num)
{{
  unsigned char raw_word[4];

  raw_word[0] = page[word_num * 4];
  raw_word[1] = page[word_num * 4 + 1];
  raw_word[2] = page[word_num * 4 + 2];
  raw_word[3] = page[word_num * 4 + 3];

  raw_word[2] = raw_word[2] & 0x7F;
  raw_word[2] = raw_word[2] | ((raw_word[1] & 0x01) << 7);
  raw_word[1] = raw_word[1] >> 1;
  raw_word[1] = raw_word[1] & 0x7F;
  raw_word[1] = raw_word[1] | ((raw_word[0] & 0x01) << 7);
  raw_word[0] = raw_word[0] >> 1;

  page[word_num * 4] = raw_word[0];
  page[word_num * 4 + 1] = raw_word[1];
  page[word_num * 4 + 2] = raw_word[2];
  page[word_num * 4 + 3] = raw_word[3];
}}


//===================================================================
// NAME:		gps_to_mjd
//
// PURPOSE:	To Convert GPS TIme To Modified Julian Time
//
// RETURNS:	void
//
// HISTORY:
//===================================================================
void gps_to_mjd( long gpswk, double secofwk, long *pmjd, double *pfmjd)
{{
   long jan680=44244L;
   double days_per_second= 0.00001157407407407407;
   long dayofwk;

   dayofwk= (long) (secofwk*days_per_second);
   *pfmjd = (secofwk - dayofwk*86400L)*days_per_second;
   *pmjd  = jan680 + gpswk*7 + dayofwk;
}}

// ===================================================================
// NAME:		fmjd_to_hms
//
// PURPOSE:	To Convert A Fractional Day To Hours, Minutes, And
//				Seconds
//
// RETURNS:	void
//
// HISTORY:
// ===================================================================
void fmjd_to_hms( double fmjd, long *phour, long *pmin, double *psec)
{{
  *phour = (long) (fmjd*24L);
  *pmin  = (long) (fmjd*1440L  - *phour*60L);
  *psec  = fmjd*86400L - *phour*3600L - *pmin*60L;
}}

// ===================================================================
// NAME:		gps_to_ydhms
//
// PURPOSE:	To COnvert GPS Time To Year, Day-Of-Year, Hour, Min,
//				And Second
//
// RETURNS:	void
//
// HISTORY:
// ===================================================================
void gps_to_ydhms(long 	 gpswk,    // GPS week
									double secofwk,  // within a GPS week
									long 	 *pyear,
									long 	 *pyday,
									long 	 *phour,
									long	 *pminute,
									double *psecond)
// used for name files only !
{{
  const long mjd_jan1_1901 = 15385;

  long	day_diff;
  long	remain;
  long	mjday;
  long	elections;
  long	kyear;
  double	fmjd;

  gps_to_mjd( gpswk, secofwk, &mjday, &fmjd);
  day_diff = mjday - mjd_jan1_1901;
  elections = day_diff/1461;
  *pyear = 1901 + 4*elections;
  remain = day_diff - elections*1461;     //  For remain = 1460 we
  kyear = remain/365 - remain/1460;       //  have remain/365 is 4.
  *pyear += kyear;                        //  -remain/1460 fixes this.
  *pyday = remain - 365*kyear + 1;
  fmjd_to_hms( fmjd, phour, pminute, psecond);
}}

// ===================================================================
// NAME:		month_and_day
//
// PURPOSE:	To Convert The Day-Of-Year To Month and Day
//
// RETURNS:	void
//
// HISTORY:
// ===================================================================
void month_and_day( long year, long yday, long *pmonth, long *pmday)
{{
  const long month_day[2][13] = {
	    {0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334, 365},
	    {0, 31, 60, 91, 121, 152, 182, 213, 244, 274, 305, 335, 366}
			       };
  long leap, guess, more;

  leap = (year%4 == 0);  /* leap = true = 1 for a leap year.*/
  if ((year%100 == 0 )&&(year != 2000L)) leap = 0; // true no leap year !!!
  guess = (long) (yday*0.032);    /* 0.032 was derived empirically. */
  more = (  ( yday - month_day[(short)leap][(short)guess+1] )  > 0  );
		/* more computes to 0 or 1 depending on veracity. */
  *pmonth = guess + more + 1;
  *pmday = yday - month_day[(short)leap][(short)(guess+more)];
}}

// ===================================================================
// NAME:		gps_to_ymdhms
//
// PURPOSE:	To Convert GPS TIme To Year, Month, Day, Hour, Minute,
//				And Second
//
// RETURNS:	void
//
// HISTORY:
// ===================================================================
void gps_to_ymdhms(long gpswk,
								   double secofwk,
								   long *pyear,
								   long *pmonth,
									 long *pmday,
									 long *phour,
									 long *pminute,
									 double *psecond)
{{
  long yday;

  gps_to_ydhms( gpswk, secofwk, pyear, &yday, phour, pminute, psecond);
  month_and_day( *pyear, yday, pmonth, pmday);
}}

// ===================================================================
// NAME:		day_of_year
//
// PURPOSE:	converts month and day_of_month to day_of_year.
//
// RETURNS:	long
//
// HISTORY:
// ===================================================================
long day_of_year (long year, long month, long mday)
{{
  static long month_day[2][12] = {
	    {0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334},
	    {0, 31, 60, 91, 121, 152, 182, 213, 244, 274, 305, 335}
				};
  long   leap;

  leap = (year%4 == 0);  /* leap = true = 1 for a leap year.*/
  if ((year%100 == 0 )&&(year != 2000L)) leap = 0; // true no leap year !

  return (month_day[(short)leap][(short)(month-1)] + mday );
}}

// ===================================================================
// NAME:		ydhms_to_mjd
//
// PURPOSE:	converts a (year, day, hour,
//          minute, second) quintuple to a
//          (mjday, fmjd) couple.  Speed and
//          clarity were the main goals...
//
// RETURNS:	void
//
// HISTORY:
// ===================================================================
void ydhms_to_mjd (long year, long yday, long hour, long minute,
								   double second, long *pmjd, double *pfmjd)
{{
  static long   mjd_jan1_1901   = 15385L;
  static double days_per_second = 0.00001157407407407407;
  long          years_into_election, elections;

  years_into_election = (year-1L)%4L;
  elections           = (year - 1901L)/4L;
  *pfmjd = (hour*3600 + minute*60 + second)*days_per_second;
  *pmjd  = mjd_jan1_1901 + elections*1461L + years_into_election*365L + yday - 1L;
}}

// ===================================================================
// NAME:		mjd_to_gps
//
// PURPOSE:	converts a (mjd, fmjd) couple to
//          a (gpsweek, seconds_of_week)
//          couple.  Speed was the primary
//          consideration...
//
// RETURNS:	void
//
// HISTORY:
// ===================================================================
void mjd_to_gps (long mjd, double fmjd, long *pgpswk, double *psecofwk)
{{
  static long jan680=44244L;
  long        idays, dayofwk;

  idays     = mjd - jan680;
  *pgpswk   = idays/7;
  dayofwk   = idays - (*pgpswk)*7;
  *psecofwk = (dayofwk + fmjd)*86400L;
}}

// ===================================================================
// NAME:		ydhms_to_gps 
//
// PURPOSE:	converts a (year, yday, hour,
//          minute, second) quintuple to a
//          (gpswk, secofwk) couple.
// RETURNS:	void
//
// HISTORY:
// ===================================================================
void ydhms_to_gps (long year, long yday, long hour, long minute,
								   double second, long *pgpswk, double *psecofwk)
{{
  long   mjday;
  double fmjd;

  ydhms_to_mjd (year, yday, hour, minute, second, &mjday, &fmjd);
  mjd_to_gps   (mjday, fmjd, pgpswk, psecofwk);
}}
