///////////////////////////////////////////////////////////////////////////////	
//    Ashtech receiver tools
//    Copyright (C) 1999  Magellan Coropration/Ashtech Precision Products 
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
///////////////////////////////////////////////////////////////////////////////


#include <math.h>

#ifndef __TRANS_HPP
#include "trans.hpp"
#endif

double elevation(double locx,double locy,double locz,
		 double svx, double svy, double svz)
{{
  double a,b,c,s,r;

  a=sqrt(locx*locx+locy*locy+locz*locz);
  b=sqrt((locx-svx)*(locx-svx)+(locy-svy)*(locy-svy)+
	 (locz-svz)*(locz-svz));
  c=sqrt(svx*svx+svy*svy+svz*svz);
  s=(a+b+c)/2.0;
  r=sqrt(((s-a)*(s-b)*(s-c))/s);
  return(2.0*atan2(r,s-c)-bPI/2.0);
}}

double azim(double locx,double locy,double locz,
	    double svx, double svy, double svz)
{{
  double r2,d,e,f,g,j,a,b,c,cosazm,sinazm;

  r2=locx*locx+locy*locy+locz*locz;
  d=(r2-(locx*svx+locy*svy+locz*svz))/r2;
  e=r2/locz-locz;
  f=d*locx+svx-locx;
  g=d*locy+svy-locy;
  j=d*locz+svz-locz;
  a=e*j-locx*f-locy*g;
  b=sqrt(locx*locx+locy*locy+e*e);
  c=sqrt(f*f+g*g+j*j);
  cosazm=a/(b*c);
  sinazm=sqrt(1-cosazm*cosazm);
  if((locx*svy-locy*svx)<0 ) sinazm=-sinazm;
  if(locz<0)
  { sinazm=-sinazm;
    cosazm=-cosazm;
  }
  a=180*atan2(sinazm, cosazm)/bPI;
  if(a<0) a+=360.0;
  return(a);
}}

void Trans::broadcast_xyz(unsigned char j,double t,double satp[3])
{{
  static double n0, omega0i, omegai, omegadoti, deltani;
  static double i0i, idoti,m0i, n, tk, mk, ekp1,cosek, sinek;
  static double sinvk, cosvk, sin2vk, cos2vk, sinomega;
  static double cosomega, sin2omega, cos2omega, sin2phik, cos2phik;
  static double deltauk,deltark, deltaik, sinphik, cosphik, sinduk;
  static double cosduk, sinuk, cosuk,rk, ik, xkprime, ykprime;
  static double omegak, sinomk, cosomk, sinik, cosik;

  unsigned char i;

  omegai=   enav[j].omega*bPI;
  omega0i=  enav[j].omega0*bPI;
  omegadoti=enav[j].omegadot*bPI;
  deltani=  enav[j].deltan*bPI;
  idoti=    enav[j].idot*bPI;
  i0i=      enav[j].i0*bPI;
  m0i=      enav[j].m0*bPI;
  n0=1000.0*sqrt( bGM84 )/(enav[j].roota*enav[j].roota*enav[j].roota);
  n=n0+deltani;

  tk=t-enav[j].toe;
  if(tk> 302400.0) tk -= 604800.0;
  if(tk<-302400.0) tk += 604800.0;
  mk=m0i+n*tk;

  ekp1=mk;
  for(i=0;i<7;i++) ekp1=mk+enav[j].e*sin(ekp1);

  cosek=cos(ekp1);
  sinek=sin(ekp1);

  sinvk=(sqrt(1.0-enav[j].e*enav[j].e)*sinek)/(1.0-enav[j].e*cosek);
  cosvk=(cosek-enav[j].e)/(1.0-enav[j].e*cosek);
  sin2vk=2*sinvk*cosvk;
  cos2vk=cosvk*cosvk-sinvk*sinvk;
  sinomega=sin(omegai);
  cosomega=cos(omegai);
  sin2omega=2*sinomega*cosomega;
  cos2omega=cosomega*cosomega-sinomega*sinomega;
  sin2phik=sin2vk*cos2omega+cos2vk*sin2omega;
  cos2phik=cos2vk*cos2omega-sin2vk*sin2omega;
  deltauk=enav[j].cuc*cos2phik+enav[j].cus*sin2phik;
  deltark=enav[j].crc*cos2phik+enav[j].crs*sin2phik;
  deltaik=enav[j].cic*cos2phik+enav[j].cis*sin2phik;
  sinphik=sinvk*cosomega+cosvk*sinomega;
  cosphik=cosvk*cosomega-sinvk*sinomega;
  sinduk=sin(deltauk);
  cosduk=cos(deltauk);
  sinuk=sinphik*cosduk+cosphik*sinduk;
  cosuk=cosphik*cosduk-sinphik*sinduk;
  rk=enav[j].roota*enav[j].roota*(1.0-enav[j].e*cosek)+deltark;
  ik=i0i+deltaik+idoti*tk;
  xkprime=rk*cosuk;
  ykprime=rk*sinuk;
  omegak=omega0i+(omegadoti-bOMEGAE84)*tk-bOMEGAE84*enav[j].toe;
  sinomk=sin(omegak);
  cosomk=cos(omegak);
  sinik=sin(ik);
  cosik=cos(ik);
  satp[0]=xkprime*cosomk-ykprime*cosik*sinomk;
  satp[1]=xkprime*sinomk+ykprime*cosik*cosomk;
  satp[2]=ykprime*sinik;
}}

void Trans::compute_azel(void)
{{
  double rcvt,elev,az;
  double dRadius;
  double satp[3];
  char	 j;

  // Ensure good position computed by receiver.
  dRadius=pow(rn.navx, 2.0)+pow(rn.navy, 2.0)+pow(rn.navz, 2.0);
  dRadius=sqrt(dRadius);
  if(dRadius<5000000.0 || dRadius>20000000.0)
    return;

  if(rn.navy==0.0 || rn.navz==0.0) return;
  for(j=0;j<rn.num_sats;j++)
  { if(rd[j].obs[0].goodbad>=23) break;
    if(rd[j].svprn<1 || rd[j].svprn>=38) continue;
    // Carry last computed position forward as is done in receiver
    //  for non-condensed formats.
  }

  if(j==rn.num_sats) return;

  rcvt=((long)(rn.rcv_time*100.0+0.5))/100.0;

  // Compute AZ and EL for each satellite
  for(j=0;j<rn.num_sats;j++)
  { // Check for valid sv number
    if(rd[j].svprn<=0 || rd[j].svprn>=38) continue;

    // Check for bad broadcast data and ignore if bad
    if(fabs(enav[rd[j].svprn-1].roota-5100.0)>200.0) continue; // + if no E-file

    // Compute SV Location at codetxmt
    broadcast_xyz(rd[j].svprn-1,rcvt-rd[j].obs[0].rawrange,satp);

    // Compute SV azimuth and elevation relative to rn solution
    elev=elevation(rn.navx,rn.navy,rn.navz,satp[0],satp[1],satp[2])*bRTD;
    az=  azim(rn.navx,rn.navy,rn.navz,satp[0],satp[1],satp[2])/2.0;
    rd[j].elevation=(unsigned char) elev;
    rd[j].azimuth=  (unsigned char) az;
  }
}}

