///////////////////////////////////////////////////////////////////////////////	
//    Ashtech receiver tools
//    Copyright (C) 1999  Magellan Coropration/Ashtech Precision Products 
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
///////////////////////////////////////////////////////////////////////////////


#include <math.h>

#ifndef __TRANS_HPP
#include "trans.hpp"
#endif

#ifndef __TRANSFOR_HPP
#include "transfor.hpp"
#endif

//	===================================================================
//	NAME:		init_variables
//
//	PURPOSE:	To Initialize Variables Needed To Compute A Position
//
//	EXTERNALS:	azel_data	- Structure Containing Misc. Position Data
//
//	RETURNS:	void
//
//	HISTORY:
//	===================================================================
void Trans::init_variables(double mr[12],double jacm[12][4],short satn)
{{
  short i;

  for (i=0;i<satn;i++)
  { jacm[i][3]=1.0;
    mr[i]=(rn.rcv_time-cd.txmt[i])*bCSPEED;
  }
  for (i=0;i<4;i++) cd.ep[i]=cd.azel_cp[i];
  // for i=3 init clock offset to half a millisec
}}

void Trans::calculate_range(double mr[12],double satp[32][3],
			    short idsv[12],short satn,
			    double cr[12],double drv[12])
{{ // To Calculate The Ranges Using The Estimated Positions

  short  i,j;
  double a[3];
  double lat,lon,alt,alpha;
  double xyz[3];

  for (i=0;i<satn;i++)
  { for (j=0;j<3;j++) xyz[j]=cd.ep[j];
    if (cd.ep[0]!=0.0 || cd.ep[1]!=0.0 || cd.ep[2]!=0.0)
    { geod_to_xyz(0,bAE84,bFLAT84,&lat,&lon,&alt,&xyz[0],&xyz[1],&xyz[2]);
      alpha=(bOMEGAE84/bCSPEED)*mr[i];
      alpha *=bRTD;
      lon +=alpha;
      geod_to_xyz(1,bAE84,bFLAT84,&lat,&lon,&alt,&xyz[0],&xyz[1],&xyz[2]);
    }
    for (j=0;j<3;j++) a[j]=satp[idsv[i]][j]-xyz[j];
    cr[i]=sqrt(a[0]*a[0]+a[1]*a[1]+a[2]*a[2]);
    drv[i]=mr[i]-cr[i]-cd.ep[3] ;
  }
}}

void Trans::jacobian(short idsv[12],short satn,
		             double cr[12],double satp[32][3],double jacm[12][4])
{{ // To Compute Partial Derivatives To Form Jacobian Matrix

  short i,j;

  for (i=0;i<satn;i++)
  { for (j=0;j<3;j++)
      jacm[i][j]=(cd.ep[j]-satp[idsv[i]][j])/cr[i];
  }
}}

void transpose(short satn,double jacm[12][4],double jacmt[4][12])
{{ // To Transpose The Jacobian Matrix

  short i,j;

  for (i=0;i<satn;i++)
  { for (j=0;j<4;j++)
      jacmt[j][i]=jacm[i][j];
  }
}}

void multiplyn(short satn,double jacmt[4][12],double jacm[12][4],
	       double jacmt_jacm[4][4])
{{ // To Compute The Working Matrix By Multiplying "jacmt" by "jacm"

  short i,j;

  for (i=0;i<4;i++)
  { for (j=0;j<4;j++)
      jacmt_jacm[i][j]=0.0;
  }
  for (j=0;j<satn;j++)
  { jacmt_jacm[0][0]+=(jacmt[0][j]*jacm[j][0]);
    jacmt_jacm[0][1]+=(jacmt[0][j]*jacm[j][1]);
    jacmt_jacm[0][2]+=(jacmt[0][j]*jacm[j][2]);
    jacmt_jacm[0][3]+=(jacmt[0][j]*jacm[j][3]);
    jacmt_jacm[1][1]+=(jacmt[1][j]*jacm[j][1]);
    jacmt_jacm[1][2]+=(jacmt[1][j]*jacm[j][2]);
    jacmt_jacm[1][3]+=(jacmt[1][j]*jacm[j][3]);
    jacmt_jacm[2][2]+=(jacmt[2][j]*jacm[j][2]);
    jacmt_jacm[2][3]+=(jacmt[2][j]*jacm[j][3]);
    jacmt_jacm[3][3]+=(jacmt[3][j]*jacm[j][3]);
  }
}}

void jacmtdr(short satn,double jacmt[4][12],double drv[12],double jacmt_dr[4])
{{ // To Compute The Right Hand Side Of The Equation By
	 // Multiplying The Jacobian Matrix By The Differential Ranges

  short i,j;

  for (i=0;i<4;i++)
  { jacmt_dr[i]=0.0;
    for (j=0;j<satn;j++)
      jacmt_dr[i] +=jacmt[i][j]*drv[j];
  }
}}

short Trans::calculate_pdop(double jacmt_jacm[4][4],
			  double L[4][4],double &pdop)
{{ // To Compute GDOP and PDOP By Finding The Main Diagonal
   // Elements Of The Inverse Of The Working Matrix

  double domain,H[3];
  double C00,C01,C11,C02,C12,C22,C03,C13,C23;

  L[0][0]=sqrt(jacmt_jacm[0][0]);
  L[0][1]=jacmt_jacm[0][1]/L[0][0];
  L[0][2]=jacmt_jacm[0][2]/L[0][0];
  L[0][3]=jacmt_jacm[0][3]/L[0][0];
  L[1][1]=sqrt(jacmt_jacm[1][1]-L[0][1]*L[0][1]);
  L[1][2]=(jacmt_jacm[1][2]-L[0][1]*L[0][2])/L[1][1];
  L[1][3]=(jacmt_jacm[1][3]-L[0][1]*L[0][3])/L[1][1];
  L[2][2]=sqrt(jacmt_jacm[2][2]-L[0][2]*L[0][2]-L[1][2]*L[1][2]);
  L[2][3]=(jacmt_jacm[2][3]-L[0][2]*L[0][3]-L[1][2]*L[1][3])/L[2][2];
  domain=jacmt_jacm[3][3]-L[0][3]*L[0][3]-L[1][3]*L[1][3]-L[2][3]*L[2][3];
  if (domain<=0.0000000001)
    L[3][3]=0.000000001;
  else
    L[3][3]=sqrt(jacmt_jacm[3][3]
		 -L[0][3]*L[0][3]-L[1][3]*L[1][3]-L[2][3]*L[2][3]);

  C00=1.0/L[0][0];
  C01=-L[0][1]*C00/L[1][1];
  C11=1.0/L[1][1];
  C02=-(L[0][2]*C00+L[1][2]*C01)/L[2][2];
  C12=-L[1][2]*C11/L[2][2];
  C22=1.0/L[2][2];
  C03=-(L[0][3]*C00+L[1][3]*C01+L[2][3]*C02)/L[3][3];
  C13=-(L[1][3]*C11+L[2][3]*C12)/L[3][3];
  C23=-L[2][3]*C22/L[3][3];

  H[0]=C00*C00+C01*C01+C02*C02+C03*C03;
  H[1]=C11*C11+C12*C12+C13*C13;
  H[2]=C22*C22+C23*C23;

  pdop=sqrt(H[0]+H[1]+H[2]);
  if (pdop>200.0)
  { if (pdop>400.0) rn.pdop=(unsigned short)(400.0*100.0);
    else            rn.pdop=(unsigned short)(pdop*100.0);
    return(1);
  }
  return(0);
}}

//	===================================================================
//	NAME:		new_position
//
//	PURPOSE:	To Solve The Linear System By First Decomposing The
//				Working Matrix Into Its LU Elements Then Find The
//				Solution In Two Steps
//					A.x = b
//					A.x = (L.U).x = L.(U.x) = b
//					L.y = b
//					U.x = y
//
//	EXTERNALS:	azel_data	- Structure Containing Misc. Position Data
//
//	RETURNS:	void
//
//	HISTORY:
//	===================================================================
//see functions "new_position" ( azel5002.c )
void Trans::new_position(double jacmt_dr[4],double L[4][4])
{{
  short  i;
  double y0,y1,y2,y3;

  y0=jacmt_dr[0]/L[0][0];
  y1=(jacmt_dr[1]-L[0][1]*y0)/L[1][1];
  y2=(jacmt_dr[2]-L[0][2]*y0-L[1][2]*y1)/L[2][2];
  y3=(jacmt_dr[3]-L[0][3]*y0-L[1][3]*y1-L[2][3]*y2)/L[3][3];

  cd.azel_cp[3]=y3/L[3][3];
  cd.azel_cp[2]=(y2-L[2][3]*cd.azel_cp[3])/L[2][2];
  cd.azel_cp[1]=(y1-L[1][2]*cd.azel_cp[2]-L[1][3]*cd.azel_cp[3])/L[1][1];
  cd.azel_cp[0]=(y0-L[0][1]*cd.azel_cp[1]-L[0][2]*cd.azel_cp[2]-L[0][3]*cd.azel_cp[3])/L[0][0];

  for (i=0;i<4;i++) cd.azel_cp[i] +=cd.ep[i];
}}


void Trans::satellite_data(short &satn,short chan_indx[12],short idsv[12],
			   double satp[32][3])
{{
  short  i,j;
  double rcvt,crange;

  for(i=0;i<32;i++) for(j=0;j<3;j++) satp[i][j]=0.0;

  rcvt=((long)(rn.rcv_time*100.0+0.5))/100.0;

  for(i=0;i<rn.num_sats;i++)
  { crange=rd[i].obs[0].rawrange*bCSPEED;
    if (crange>26000000.0||crange<18000000.0) rd[i].obs[0].goodbad=22;
    if (rd[i].obs[0].goodbad!= 24) continue;
    if ((enav[rd[i].svprn-1].wn == 0)||(enav[rd[i].svprn-1].health))
    { rd[i].obs[0].goodbad=22;
      continue;
    }
    for (j=0;j<3;j++) satp[rd[i].svprn-1][j]=0.0;
    chan_indx[satn]=i;
    idsv[satn]=rd[i].svprn-1;
    cd.txmt[satn]=rcvt-rd[i].obs[0].rawrange;
    cd.txmt[satn]=cd.txmt[satn]-
		  ((double)enav[idsv[satn]].af0+
		   (cd.txmt[satn]-(double)enav[idsv[satn]].toc)*
		   (double)enav[idsv[satn]].af1+
		   pow((cd.txmt[satn]-(double)enav[idsv[satn]].toc),2.0)*
		   (double)enav[idsv[satn]].af2
		  );
    // decremented svprn by 1 
    broadcast_xyz(rd[i].svprn-1,cd.txmt[satn],satp[rd[i].svprn-1]);
					      // x,y,z (satp) .
    satn++;
  }
}}


void Trans::azel_sat(short satn,short chan_indx[12],short idsv[12],
		     double satp[32][3])
{{
  short i;
  short el,az;

  for(i=0;i<satn;i++)
  { if (!satp[idsv[i]][0]&&!satp[idsv[i]][1]&&!satp[idsv[i]][2])
    { if (satn<4)
      { rd[chan_indx[i]].obs[0].goodbad=22;
				// in position(...)
				if (rh.capability==L1CP_L2P || rh.capability==L1C_L2P)
	  			rd[chan_indx[i]].obs[1].goodbad=22;
      }
      continue;
    }
    if (satn<4)
    { rd[chan_indx[i]].obs[0].goodbad=23;
      // in position(...) L2_slot will change to 23 when use l2 for pos. calc.
      if (rh.capability==L1CP_L2P || rh.capability==L1C_L2P)
				rd[chan_indx[i]].obs[1].goodbad = 22;
      if (!cd.ep[0]&&!cd.ep[1]&&!cd.ep[2]) continue;
    }
    el=(short)(bRTD*elevation(cd.ep[0],cd.ep[1],cd.ep[2],
		    satp[idsv[i]][0],satp[idsv[i]][1],satp[idsv[i]][2]));
    az=(short)(azim(cd.ep[0],cd.ep[1],cd.ep[2],
		    satp[idsv[i]][0],satp[idsv[i]][1],satp[idsv[i]][2]));
    if (el>90||el<0)  el=0;
    if (az>360||az<0) az=0;
    rd[chan_indx[i]].elevation=(unsigned char)el;
    rd[chan_indx[i]].azimuth=(unsigned char)(az/2);
  }
}}


void Trans::prec_navigation(double save_pos[4])
{{
  rn.pdop=(unsigned short) 40000L;
  rn.navx=save_pos[0]=cd.ep[0];
  rn.navy=save_pos[1]=cd.ep[1];
  rn.navz=save_pos[2]=cd.ep[2];
  save_pos[3]=0.0;
  rn.navxdot=0.0;
  rn.navydot=0.0;
  rn.navzdot=0.0;
  rn.navtdot=0.0;
}}


void Trans::new_navigation(double save_pos[4],double save_rcv,double pdop)
{{
    rn.navx=cd.azel_cp[0];
    rn.navy=cd.azel_cp[1];
    rn.navz=cd.azel_cp[2];
    rn.navt=cd.azel_cp[3];
    if (save_rcv!=0.0 )
    { rn.navxdot=(float)((rn.navx-save_pos[0])/(rn.rcv_time-save_rcv));
      rn.navydot=(float)((rn.navy-save_pos[1])/(rn.rcv_time-save_rcv));
      rn.navzdot=(float)((rn.navz-save_pos[2])/(rn.rcv_time-save_rcv));
      rn.navtdot=(rn.navt-save_pos[3])/(rn.rcv_time-save_rcv);
    }
    rn.pdop=(unsigned short)(pdop*100.0);
}}


void Trans::position()
{{ // in position(...) param. for obs[] L1_slot->obs[0], L2_slot->obs[1]
  short  i,j;
  short  satn;
  short  chan_indx[12],idsv[12];
  double satp[32][3],save_pos[4];
  double pdop,save_rcv;
  double sumdel,del[4],mr[12],cr[12],drv[12],L[4][4];
  double jacm[12][4],jacmt[4][12],jacmt_dr[4],jacmt_jacm[4][4];

  satn=0;
  for(i=0;i<4;i++)
    save_pos[i]=cd.azel_cp[i];
  save_rcv=((long)(cd.txmt[0]*100.0+0.5))/100.0;

  satellite_data(satn,chan_indx,idsv,satp);

  if (satn<4)
    prec_navigation(save_pos);
  else
  { init_variables(mr,jacm,satn);
    for(i=0;i<10;i++)
    { calculate_range(mr,satp,idsv,satn,cr,drv);
      jacobian(idsv,satn,cr,satp,jacm);
      transpose(satn,jacm,jacmt);
      multiplyn(satn,jacmt,jacm,jacmt_jacm);
      jacmtdr(satn,jacmt,drv,jacmt_dr);
      if (calculate_pdop(jacmt_jacm,L,pdop))
				return;
      new_position(jacmt_dr,L);
      if (i)
      { if (fabs(cd.ep[3]) <= 0.000000000001) break;
				sumdel=0.0;
				for(j=0;j<4;j++)
				{ del[j]=(cd.azel_cp[j]-cd.ep[j])*(cd.azel_cp[j]-cd.ep[j]);
	  			sumdel+=del[j]/(cd.ep[j]*cd.ep[j]);
				}
				if (sumdel <= 0.000000000001) break;
      }
      for(j=0;j<4;j++) cd.ep[j]=cd.azel_cp[j];
    }
    new_navigation(save_pos,save_rcv,pdop);
  }
  azel_sat(satn,chan_indx,idsv,satp);
}}

