///////////////////////////////////////////////////////////////////////////////	
//    Ashtech receiver tools
//    Copyright (C) 1999  Magellan Coropration/Ashtech Precision Products 
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
///////////////////////////////////////////////////////////////////////////////


#include <string.h>
#include <time.h>

#ifndef __TRANS_HPP
#include "trans.hpp"
#endif

short Trans::Rgr1Comp_cword()
{{
  short block_len;

  // com.cnbyte -> bf.ibf

  memcpy(&block_len,bf.ibf,sizeof(block_len));
  swap_nbytes((char *)&block_len,sizeof(block_len));

  if (block_len<8) return(ERROR_LENRANG1);

  rdd.addr +=1L; // after block_len
  rdd.cword=block_len+1; // +1 checksum

  return(TRANS_OK);
}}


short Trans::ReadTINYRANGER1()
{{
  short error=TRANS_OK;                 
  short j,st_byte=1;
  char  last_sitename[5];
  long  ptr_data,sig;

  // com.cnbyte -> bf.ibf
  error=IndentBuf_CheckingChecksum16();
  if (error!=TRANS_OK)	return(error);
  
  memmove(&bf.ibf,&bf.ibf[1],com.cnbyte-1);

  strncpy(last_sitename,rn.sitename,4);

  rn.num_sats=(char) extract_long(&st_byte,5,F_UNSIGNED);
  for (j=0;j<rn.num_sats;j++)
  { rd[j].svprn=(unsigned char) extract_long(&st_byte,5,F_UNSIGNED)+1;
    rd[j].chnind=99; 
    if (fa.rcvr_version==F_6)
    { rd[j].obs[0].rawrange=
		   (double)(extract_long(&st_byte,31,F_UNSIGNED))*1E-10;
      rd[j].obs[0].carphase=0.0;
      rd[j].obs[0].smth_corr=
		   (float)(((double)extract_long(&st_byte,14,F_SIGNED))/100.0);
      rd[j].obs[0].smth_count=(short)extract_long(&st_byte,8,F_UNSIGNED);
    }
    else
    { rd[j].obs[0].rawrange=
		   (double)(extract_long(&st_byte,27,F_UNSIGNED))*1E-9;
      sig=extract_long(&st_byte,1,F_UNSIGNED);
      rd[j].obs[0].carphase=
		   (double)(extract_long(&st_byte,28,F_UNSIGNED));
      rd[j].obs[0].carphase +=
		   0.01*(double)(extract_long(&st_byte,7,F_UNSIGNED));
      rd[j].obs[0].carphase=(long)(rd[j].obs[0].carphase+0.5);
      if (sig) rd[j].obs[0].carphase=-rd[j].obs[0].carphase;
      rd[j].obs[0].smth_corr= 0;
      rd[j].obs[0].smth_count= 0;
    }
    if (extract_long(&st_byte,1,F_UNSIGNED))
    { rd[j].obs[0].warning=4;
      rd[j].obs[0].goodbad=22;
      rd[j].obs[0].polarity_known=0;
    }
    else
    { rd[j].obs[0].warning=0;
      rd[j].obs[0].goodbad=24;
      rd[j].obs[0].polarity_known=5;
    }
    rd[j].obs[0].ireg=0;
    rd[j].obs[0].qa_phase=0;
    rd[j].obs[0].doppler=0;
    rd[j].obs[1].polarity_known= rd[j].obs[2].polarity_known= 0;
    rd[j].obs[1].goodbad       = rd[j].obs[2].goodbad       = 0;
    rd[j].obs[1].rawrange      = rd[j].obs[2].rawrange      = 0.0;
    rd[j].obs[1].doppler       = rd[j].obs[2].doppler       = 0;
    rd[j].obs[1].warning       = rd[j].obs[2].warning       = 0;
    rd[j].obs[1].smth_corr     = rd[j].obs[2].smth_corr     = 0;
    rd[j].obs[1].smth_count    = rd[j].obs[2].smth_count    = 0;
    rd[j].obs[1].ireg          = rd[j].obs[2].ireg          = 0;
    rd[j].obs[1].qa_phase      = rd[j].obs[2].qa_phase      = 0;
    rd[j].obs[1].carphase      = rd[j].obs[2].carphase      = 0.0;
  }
  // Get recieve time
  rn.rcv_time=(double)(extract_long(&st_byte,30,F_UNSIGNED))/1000.0;

  // Init. Azimuth and Elevation
  for (j=0;j<rn.num_sats;j++)
  { rd[j].elevation=0;
    rd[j].azimuth=0;
  }
  // Init. rawnaw
  rn.navx=rn.navy=rn.navz=0.0;
  rn.navxdot=rn.navydot=rn.navzdot=0.0;
  rn.navtdot=0.0; rn.pdop=0;

  position();
  rn.navt=1.0e-5*bLITE001*(double)(extract_long(&st_byte,18,F_SIGNED));

  if (extract_long(&st_byte,1,F_UNSIGNED))
  { ptr_data=extract_long(&st_byte,31,F_UNSIGNED);
    strncpy(last_sitename,(char *)&ptr_data,4);
    swap_nbytes(last_sitename,4);
  }
  strncpy(rn.sitename,(char *)last_sitename,4);

  // Write in B-file
  oneEpoch_forBfile();

  return(error);
}}

short Trans::Rgr2Comp_cword()
{{
  // com.cnbyte -> bf.ibf
  memcpy(&fa.block_len_ranger2,&bf.ibf[0],sizeof(fa.block_len_ranger2));
  swap_nbytes((char *)&fa.block_len_ranger2,sizeof(fa.block_len_ranger2));
  memcpy(&fa.checksum_ranger2, &bf.ibf[2],sizeof(fa.checksum_ranger2));
  swap_nbytes((char *)&fa.checksum_ranger2, sizeof(fa.checksum_ranger2));

  if ((rec_type_code==MICRO_Z)||(rec_type_code==LOCUS))
  { if (fa.block_len_ranger2<=0) return(ERROR_LENRANG2);
  }
  else
    if (fa.block_len_ranger2<14) return(ERROR_LENRANG2);

  rdd.addr +=2L; // rdd.addr -> data
  rdd.cword=fa.block_len_ranger2;

  return(TRANS_OK);
}}

short Trans::ReadTINYRANGER2()
{{
  short          len,st_byte=1;
  unsigned short i,check16;
  long           ptr_data;
  char           tot_svs,block_type;
  short          last_st_byte=1;
  long           yr,mo,dy,hr,mn;
  double         sec;
  static char    fill_bits [5]={ 0, 6,13,4,2};
  static char    delta_bits[5]={32,24,16,8,9};

  // com.cnbyte -> bf.ibf
  memmove(&bf.ibf[1],bf.ibf,com.cnbyte);
  com.cnbyte+=1; bf.ibf[0]=bf.tbf[0];

  com.cnbyte +=2;
  check16=Checksum16(bf.ibf);  // +2 "checksum_ranger2"
  com.cnbyte -=2;
  if (fa.checksum_ranger2!=check16) return(ERROR_CHECKSUM);

  memmove(&bf.ibf,&bf.ibf[1],com.cnbyte-1);

  // Unpack Svs Tracked.
  for (tot_svs=0,i=32;i>=1;i--)
    if (extract_long(&st_byte,1,F_UNSIGNED))
      rd[tot_svs++].svprn=(unsigned char)i;
  rn_ranger2.num_sats=tot_svs;
  // Unpack Sitename.
  ptr_data=extract_long(&st_byte,32,F_UNSIGNED);
  strncpy(rn_ranger2.sitename,(char*)&ptr_data,4);
  swap_nbytes(rn_ranger2.sitename,4);
  // Unpack start position.
  rn_ranger2.navx=0.01*(double)(extract_long(&st_byte,31,F_SIGNED));
  rn_ranger2.navy=0.01*(double)(extract_long(&st_byte,31,F_SIGNED));
  rn_ranger2.navz=0.01*(double)(extract_long(&st_byte,31,F_SIGNED));
  // Unpack time stamps.
  if ((rec_type_code==MICRO_Z)||(rec_type_code==LOCUS))
    wn_ranger2=(long)(extract_long(&st_byte,11,F_UNSIGNED));
  else
    wn_ranger2=(long)(extract_long(&st_byte,10,F_UNSIGNED));
  if (wn_ranger2 < 815) wn_ranger2+=1024; // new epoch 22 august 1999 !!! 
	rn_ranger2.rcv_time=(double)(extract_long(&st_byte,30,F_UNSIGNED))/1000.0;
  edt_ranger2=(float)(((double)extract_long(&st_byte,17,F_UNSIGNED))/100.0);
  // Unpack differential mode: once set, we keep it set here!!!.
  if (extract_long(&st_byte,1,F_UNSIGNED))
    fa.ranger_diff_mode=1;
  else
    fa.ranger_diff_mode=0;
  // Extract PDOP.
  rn_ranger2.pdop=(unsigned short)extract_long(&st_byte,9,F_UNSIGNED);

  fa.block_len_ranger2 -=(st_byte-1)/16; // Set block length to reflect unpacked bits
  gps_to_ymdhms (wn_ranger2,rn_ranger2.rcv_time+0.00000001,&yr,&mo,&dy,&hr,&mn,&sec);

  if (yr>1900) yr -=1900;
  if (yr>=100) yr -=100;

  // Write in C-file
  fwf.f_writeBfile=1;
  len_epoch=0;
	if (!fcf.f_createCfile) write_headerCfile();

  len=sprintf(&one_epochbuf[len_epoch],"%4.4s %2.2ld/%2.2ld/%2.2ld %2.2ld:%2.2ld:%09.6lf %2d %5.1lf ",
							rn_ranger2.sitename,mo,dy,yr,hr,mn,sec+0.00000001,
							tot_svs,(float)rn_ranger2.pdop);
  len_epoch+=len;		 	     
  len=sprintf(&one_epochbuf[len_epoch],"%13.3lf %13.3lf %13.3lf\r\n",rn_ranger2.navx,rn_ranger2.navy,rn_ranger2.navz);
  len_epoch+=len;
                              
  if ((rec_type_code==MICRO_Z)||(rec_type_code==LOCUS)) return(TRANS_OK);
  
  while (fa.block_len_ranger2>0)
  { // Unpack each position offset.
    last_st_byte=st_byte;
    // Determine block type
    block_type=0;
    while (extract_long(&st_byte,1,F_UNSIGNED))
    { if (block_type++ >= R_MODE2_BTYPES-1)
				return(ERROR_FORMATRANG2); // invalid format
    }
    // Extract fill bits
    if (fill_bits[block_type])
      extract_long(&st_byte,fill_bits[block_type],F_UNSIGNED);
    if (block_type!=4)
    { // Extract delta position.
      if (block_type)
      { rn_ranger2.navx +=0.01*
	  		(double)(extract_long(&st_byte,delta_bits[block_type],F_SIGNED));
      }
      else
      { // Block type 0 has only 31 bits for deltax
				rn_ranger2.navx +=0.01*
	  		(double)(extract_long(&st_byte,delta_bits[block_type]-1,F_SIGNED));
      }
      rn_ranger2.navy +=0.01*
			(double)(extract_long(&st_byte,delta_bits[block_type],F_SIGNED));
      rn_ranger2.navz +=0.01*
			(double)(extract_long(&st_byte,delta_bits[block_type],F_SIGNED));
      // Updata time tags for current epoch.
      rn_ranger2.rcv_time +=edt_ranger2;
      if (rn_ranger2.rcv_time>604800.0)
      { // Week roll-over.
				wn_ranger2++;
				rn_ranger2.rcv_time -= 604800.0;
      }
      gps_to_ymdhms (wn_ranger2,rn_ranger2.rcv_time+0.00000001,
		       &yr, &mo, &dy, &hr, &mn, &sec);

      // Write in C-file
      if (yr>1900) yr -=1900;
      if (yr>=100) yr -=100;
      len=sprintf(&one_epochbuf[len_epoch],
	      "%4.4s %2.2ld/%2.2ld/%2.2ld %2.2ld:%2.2ld:%09.6lf %2d %5.1lf ",
	      rn_ranger2.sitename,mo,dy,yr,hr,mn,sec+0.00000001,
	      tot_svs,(float)rn_ranger2.pdop);    
			len_epoch+=len;    
      len=sprintf(&one_epochbuf[len_epoch],"%13.3lf %13.3lf %13.3lf\r\n",rn_ranger2.navx,rn_ranger2.navy,rn_ranger2.navz);
      len_epoch+=len;
    }
    else // if (block_type==4)
    { // Extract New PDOP.
			rn_ranger2.pdop= (unsigned short) extract_long(&st_byte,delta_bits[block_type],F_UNSIGNED);    
		}
    // Set block length to reflect unpacked bits.
    fa.block_len_ranger2 -=(st_byte-last_st_byte)/16;
  }

  return(TRANS_OK);
}}

short Trans::ReadTINYDRANGER2()
{{
  short          len,st_byte=1;
  long           yr,mo,dy,hr,mn;
  unsigned short check16;
  double         sec;
  char           block_type; 
  static char    fill_bits [5]={ 0, 6,13,4,2};
  static char    delta_bits[5]={32,24,16,8,9};
  
  memmove(&bf.ibf[1],bf.ibf,com.cnbyte);
  com.cnbyte+=1; bf.ibf[0]=bf.tbf[0];

  com.cnbyte +=2;
  check16=Checksum16(bf.ibf);  // +2 "checksum_ranger2"
  com.cnbyte -=2;
  if (fa.checksum_ranger2!=check16) return(ERROR_CHECKSUM);

  memmove(&bf.ibf,&bf.ibf[1],com.cnbyte-1);

  fwf.f_writeBfile=1;
  len_epoch=0;

  block_type=0;
  while (extract_long(&st_byte,1,F_UNSIGNED))
  { if (block_type++ >= R_MODE2_BTYPES-1)
      return(ERROR_FORMATRANG2); // invalid format
  }
  // Extract fill bits
  if (fill_bits[block_type])
    extract_long(&st_byte,fill_bits[block_type],F_UNSIGNED);
  if (block_type!=4)
  { // Extract delta position.
    if (block_type)
    { rn_ranger2.navx +=0.01*
	  	(double)(extract_long(&st_byte,delta_bits[block_type],F_SIGNED));
    }
    else
    { // Block type 0 has only 31 bits for deltax
			rn_ranger2.navx +=0.01*
	  	(double)(extract_long(&st_byte,delta_bits[block_type]-1,F_SIGNED));
    }
    rn_ranger2.navy +=0.01*
	  (double)(extract_long(&st_byte,delta_bits[block_type],F_SIGNED));
    rn_ranger2.navz +=0.01*
	  (double)(extract_long(&st_byte,delta_bits[block_type],F_SIGNED));
    // Updata time tags for current epoch.
    rn_ranger2.rcv_time +=edt_ranger2;
    if (rn_ranger2.rcv_time>604800.0)
    { // Week roll-over.
			wn_ranger2++;
			rn_ranger2.rcv_time -= 604800.0;
    }
    gps_to_ymdhms(wn_ranger2,rn_ranger2.rcv_time+0.00000001,
		       	  &yr, &mo, &dy, &hr, &mn, &sec);

    // Write in C-file
    if (yr>1900) yr -=1900;
    if (yr>=100) yr -=100;
    len=sprintf(&one_epochbuf[len_epoch],
	            "%4.4s %2.2ld/%2.2ld/%2.2ld %2.2ld:%2.2ld:%09.6lf %2d %5.1lf ",
	            rn_ranger2.sitename,mo,dy,yr,hr,mn,sec+0.00000001,
	            rn_ranger2.num_sats,(float)rn_ranger2.pdop);    
		len_epoch+=len;    
    len=sprintf(&one_epochbuf[len_epoch],"%13.3lf %13.3lf %13.3lf\r\n",rn_ranger2.navx,rn_ranger2.navy,rn_ranger2.navz);
    len_epoch+=len;
  }
  else // if (block_type==4)
  { // Extract New PDOP.
		rn_ranger2.pdop= (unsigned short) extract_long(&st_byte,delta_bits[block_type],F_UNSIGNED);
  }
  return(TRANS_OK);
}}

void Trans::write_headerCfile()
{{ 
  struct tm *ptr;
  time_t    lt;
  short     len; 
  char      t_buff[40];
  char      form[250];
  char      strang[300];

  lt=time(NULL);
  ptr=localtime(&lt);
  sprintf(t_buff,asctime(ptr));
  t_buff[strlen(t_buff)-1] = '\0';

  strcpy((char *)form,
  "Ashtech, Inc. GPPS-2          Program: DOWNLOAD          Version: %6.6s\r\n");
  strcat((char *)form,
  "        %30.30s    Differentially Corrected: %c\r\n");
  strcat((char *)form,
  "SITE MM/DD/YY HH:MM:SS        SVs PDOP         X             Y             Z\r\n");
  sprintf (strang,form,"2.00  ",t_buff,(fa.ranger_diff_mode)?'Y':'N');
  len=strlen(strang);
  memcpy(&one_epochbuf[0],strang,len);
  len_epoch+=len;
}}


short Trans::Rgr3Comp_cword()
{{
	unsigned short mbens;
  short          st_byte=1;

	// com.cnbyte -> bf.ibf
  mbens = (unsigned short) extract_long(&st_byte,4,F_UNSIGNED);
	if ((mbens<0)||(mbens>12)) return(ERROR_MEASDATA);
	
	rdd.cword=4+mbens*239+62;
	if ( rdd.cword%16 ) rdd.cword =rdd.cword-rdd.cword%16+16;
  rdd.cword=rdd.cword/16;
  rdd.cword +=1; // +1 checksum	

  return(TRANS_OK);
}}


short Trans::ReadTINYRANGER3()
{{
  short					error=TRANS_OK;
  unsigned char warning;
	short					i,j,st_byte=1;
  char					last_sitename[5];
  long					ptr_data,sig;

	// com.cnbyte ->bf.ibf
	error=IndentBuf_CheckingChecksum16(); 	
	if (error!=TRANS_OK) return(error);
  memmove(&bf.ibf,&bf.ibf[1],com.cnbyte-1);

	rn.num_sats =(char) extract_long(&st_byte,4,F_UNSIGNED);

  for (j=0;j<rn.num_sats;j++)
  { rd[j].svprn  =(unsigned char) extract_long(&st_byte,5,F_UNSIGNED)+1;
    rd[j].chnind =(unsigned char) extract_long(&st_byte,4,F_UNSIGNED);

    warning=(unsigned char) extract_long(&st_byte,8,F_UNSIGNED);
    // Bit 5 is for p1 only. xx0xxxxx
    rd[j].obs[0].warning = warning & 0XDF;
    rd[j].obs[0].goodbad = (rd[j].obs[0].warning & 0X03)+22;

    // Bit 1, 2 not used for P1 xxxxxx00
    rd[j].obs[1].warning = warning & 0XFC;
    // Because P1 is not used for position computation, it is always 22.
    rd[j].obs[1].goodbad=(rd[j].obs[1].warning & 0X03)+22;

    rd[j].obs[1].rawrange=(double) (extract_long(&st_byte,32,F_UNSIGNED))*100.0; // 0.1ns->ps 1E-12;
    // retrieve CA smooth rang from (CA smooth - P1 unsmooth)
    // CA smooth correction always = 0
    rd[j].obs[0].rawrange=(double) (extract_long(&st_byte,21,F_SIGNED))+rd[j].obs[1].rawrange;
		//extract_long(&st_byte, 21, F_UNSIGNED);
		//rd[j].obs[0].rawrange =  rd[j].obs[1].rawrange;

    // because CA smooth correction always 0, set smooth counter to some
    // constant value, so post processing can be proceed.
    rd[j].obs[0].smth_count=200;
    rd[j].obs[0].ireg=(unsigned char) extract_long(&st_byte,8,F_UNSIGNED);

	  sig=extract_long(&st_byte,1,F_UNSIGNED);
    rd[j].obs[0].carphase=(double)(extract_long(&st_byte,28,F_UNSIGNED));
    rd[j].obs[0].carphase+=((double)(extract_long(&st_byte,11,F_UNSIGNED)))*5E-4;
    if(sig) rd[j].obs[0].carphase *=-1.0;
    rd[j].obs[1].carphase=rd[j].obs[0].carphase;

    rd[j].obs[1].smth_corr=(float)(((double)extract_long(&st_byte,14,F_SIGNED))*1E-2);
    rd[j].obs[1].smth_count=(short)extract_long(&st_byte,8,F_UNSIGNED);

    rd[j].obs[2].warning = (unsigned char) extract_long(&st_byte,8,F_UNSIGNED);
    rd[j].obs[2].goodbad = (rd[j].obs[2].warning & 0x03)+22;

    // retrieve P2 raw range from (P2 raw range -P1 raw range) (ps)
    rd[j].obs[2].rawrange =(double)(extract_long(&st_byte,21,F_SIGNED))+rd[j].obs[1].rawrange;
    // convert ps to s
    for(i=0;i<3;i++)
      rd[j].obs[i].rawrange*=1E-12;

    rd[j].obs[2].ireg=(unsigned char) extract_long(&st_byte,8,F_UNSIGNED);
    // P1 S/N is the same as P2
    rd[j].obs[1].ireg=rd[j].obs[2].ireg;

		sig = extract_long(&st_byte, 1, F_UNSIGNED);
    rd[j].obs[2].carphase=(double) (extract_long(&st_byte,28,F_UNSIGNED));
    rd[j].obs[2].carphase +=((double)(extract_long(&st_byte,11,F_UNSIGNED)))*5E-4;
    if (sig) rd[j].obs[2].carphase *=-1.0;

    rd[j].obs[2].smth_corr=(float)(((double)extract_long(&st_byte,14,F_SIGNED))*1E-2);
    rd[j].obs[2].smth_count=(short)extract_long(&st_byte,8,F_UNSIGNED);
  
		rd[j].obs[0].qa_phase=0;
		rd[j].obs[0].doppler=0;
		rd[j].obs[0].polarity_known=0;
		rd[j].obs[0].smth_corr=0.0;
		rd[j].obs[1].qa_phase				= rd[j].obs[2].qa_phase				= 0;
		rd[j].obs[1].doppler			  = rd[j].obs[2].doppler				= 0;	
		rd[j].obs[1].polarity_known = rd[j].obs[2].polarity_known	= 0;
	}
  // Get recieve time
  rn.rcv_time=(double)(extract_long(&st_byte,30,F_UNSIGNED))*1E-3;

  // Init. Azimuth and Elevation
  for (j=0;j<rn.num_sats;j++)
  { rd[j].elevation=0;
    rd[j].azimuth=0;
  }
  // Init. rawnaw
  rn.navx=rn.navy=rn.navz=0.0;
  rn.navxdot=rn.navydot=rn.navzdot=0.0;
  rn.navt=0.0; rn.navtdot=0.0; rn.pdop=0;

  position();

  ptr_data = extract_long(&st_byte,32,F_UNSIGNED);
  strncpy(last_sitename,(char *)&ptr_data,4);
  swap_nbytes(last_sitename,4);
	strncpy(rn.sitename,(char *)last_sitename,4);

  // Write in B-file
  oneEpoch_forBfile();
	
	return(error);
}}