///////////////////////////////////////////////////////////////////////////////	
//    Ashtech receiver tools
//    Copyright (C) 1999  Magellan Coropration/Ashtech Precision Products 
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
///////////////////////////////////////////////////////////////////////////////


#include <string.h>
#include <math.h>
#include <ctype.h>

#ifndef __RDRTIME_HPP
#include "rdrtime.hpp"
#endif

short RdRtime::AlmanacNameFile()
{{
  long   day_year,h,m,lsec,year=0L;
  long   month,mday;
  double sec;

  len_almanac=0;					
	if (rtt.alm_wn>0) 
  { gps_to_ydhms(rtt.alm_wn,rtt.alm_tow,&year,&day_year,&h,&m,&sec);
    lsec=(long)(sec+0.5);
  }
  else
    rtf.f_wr_almGPS=0;
  if (year>0)
  { if (lsec>=60L) { m++; lsec -=60L; }
  	if (m>=60L)    { h++; m    -=60L; }
    month_and_day(year,day_year,&month,&mday);
    if (year>1900) year -=1900;
    if (year>=100) year -=100;
    sprintf((char *)file_rec.file_name,"alm%2.2ld.%3.3ld\0",year,day_year);  
    sprintf((char *)file_rec.date,"%2.2ld/%2.2ld/%2.2ld\0",month,mday,year);
    sprintf((char *)file_rec.time,"%2.2ld/%2.2ld/%2.2ld\0",h,m,lsec);
    sprintf((char *)file_rec.file_type,"ALMANAC \0");
    if (rtf.f_wr_almGPS) len_almanac+=sizeof(almstrct)*32;
    sprintf((char *)file_rec.bytes,"%7ld\0",(long)len_almanac);
    return(TRANS_OK);
  }
  return(NO_FILE);
}}

short RdRtime::Read_strctSNAV()
{{
  short len,error;

  error=swap_strctSNAV();
  if (error!=TRANS_OK)
  { if (error==ERROR_CHECKSUM) rtc.index_static_buf-=10;
    if (error==NO_FILE) error=TRANS_OK;
    return( error );
  }

  rtf.beg_SNV_data=1;
	fwf.f_writeEfile=1;
  len_satbuf=0;
  len=sizeof(fa.satnum); 
  memcpy(&one_satbuf[len_satbuf],&fa.satnum,len);
  len_satbuf+=len;
  len=sizeof(enav[fa.satnum-1]);
  memcpy(&one_satbuf[len_satbuf],&enav[fa.satnum-1],len);
  len_satbuf+=len;  

	rtc.snav++;

  return(TRANS_OK);
}}


void RdRtime::Number_Sat()
{{
  if (rtc.last_count_mben==99) rn.num_sats=rtc.count_mben+1;
  if (rtc.count_mben>=rtc.last_count_mben) rdd.block_no=0;
}}


void RdRtime::Read_idfstrct(idfstrct &idf)
{{
  swap_nbytes((char *)&idf.sequence_tag,sizeof(idf.sequence_tag));
  rtc.count_mben=(unsigned) idf.left;

  Number_Sat();
}}


void RdRtime::Read_satstrct(satstrct &sat)
{{
  rd[rdd.block_no].svprn     = sat.svprn;
  rd[rdd.block_no].elevation = sat.elevation;
  rd[rdd.block_no].azimuth   = sat.azimuth;
  rd[rdd.block_no].chnind    = sat.chnind;
}}


void RdRtime::Read_meastrct(short i,meastrct &mea)
{{
  long part;

  if ((rec_type_code!=Z_18)||((rec_type_code==Z_18)&&(f_swap_dataZ18)))
  { swap_nbytes((char *)&mea.full_phase,sizeof(mea.full_phase));
    swap_nbytes((char *)&mea.raw_range,sizeof(mea.raw_range));
    swap_nbytes((char *)&mea.doppler,sizeof(mea.doppler));
    swap_nbytes((char *)&mea.smoothing,sizeof(mea.smoothing));
  }

  rd[rdd.block_no].obs[i].warning        = mea.warning;
  rd[rdd.block_no].obs[i].goodbad        = mea.goodbad;
  rd[rdd.block_no].obs[i].polarity_known = mea.polarity_know;
  rd[rdd.block_no].obs[i].ireg           = mea.ireg;
  rd[rdd.block_no].obs[i].qa_phase       = mea.qa_phase;
  rd[rdd.block_no].obs[i].carphase       = mea.full_phase;
  rd[rdd.block_no].obs[i].rawrange       = mea.raw_range;
  rd[rdd.block_no].obs[i].doppler        = mea.doppler;

  // 32 bits where 31-24 are the smooth_count
  //          bits 23- 0 are the smooth_corr
  part = mea.smoothing & 0X00FFFFFFL;
  if ((part & 0x00800000L) != 0L)
  { part &= 0x007FFFFFL;
    part=-part;
  }
  rd[rdd.block_no].obs[i].smth_corr      = (float)(part/100.0);
  part=((unsigned long) mea.smoothing) >> 24;
  rd[rdd.block_no].obs[i].smth_count     = (unsigned int) part;
}}


void RdRtime::Read_codstrct(codstrct &cod)
{{
  swap_nbytes((char *)&cod.carphase2,sizeof(cod.carphase2));

  rd[rdd.block_no].obs[1].warning        = 0;
  rd[rdd.block_no].obs[1].goodbad        = 0;
  rd[rdd.block_no].obs[1].polarity_known = 0;
  rd[rdd.block_no].obs[1].ireg           = cod.ireg2/2;
  rd[rdd.block_no].obs[1].qa_phase       = cod.qa_phase2;
  rd[rdd.block_no].obs[1].carphase       = cod.carphase2;
  rd[rdd.block_no].obs[1].rawrange       = 0.0;
  rd[rdd.block_no].obs[1].doppler        = 0L;

  rd[rdd.block_no].obs[1].smth_corr      = 0.0;
  rd[rdd.block_no].obs[1].smth_count     = 0;
}}


void RdRtime::Extract_only_namerec()
{{
  char *c;

  c=strchr(rh.rcvr_type,' ');
  if (c) c[0]='\0';
}}


short RdRtime::Read_strctold()
{{
  unsigned  check16,checksum;
  double    part;

  check16=Checksum16(bf.ibf);
  memcpy(&checksum,&bf.ibf[com.cnbyte-2],sizeof(checksum));
  swap_nbytes((char *)&checksum,sizeof(checksum));
  if (checksum!=check16)
  { rtc.index_static_buf-=10;
    return(ERROR_CHECKSUM);
  }

  memcpy(&mben_old,&bf.ibf[1],sizeof(mben_old));

  swap_nbytes((char *)&mben_old.lost_lock_ctr,sizeof(mben_old.lost_lock_ctr));
  swap_nbytes((char *)&mben_old.codetxmt,sizeof(mben_old.codetxmt));
  swap_nbytes((char *)&mben_old.doppler,sizeof(mben_old.doppler));
  swap_nbytes((char *)&mben_old.intdoppler,sizeof(mben_old.intdoppler));
  swap_nbytes((char *)&mben_old.unint_p2,sizeof(mben_old.unint_p2));
  swap_nbytes((char *)&mben_old.int_L2diff,sizeof(mben_old.int_L2diff));
  swap_nbytes((char *)&mben_old.elevation,sizeof(mben_old.elevation));
  swap_nbytes((char *)&mben_old.azimuth,sizeof(mben_old.azimuth));

  rtc.count_mben=(unsigned) mben_old.data_count;
  Number_Sat();

  rd[rdd.block_no].svprn     = mben_old.svprn;
  rd[rdd.block_no].chnind    = mben_old.chnind;
  rd[rdd.block_no].elevation = (char) ((float) mben_old.elevation/100.0);
  rd[rdd.block_no].azimuth   = (unsigned char) mben_old.azimuth;

  if (rh.num_obs_types!=1)
  { rh.capability=L1;
    rh.num_obs_types=1;

    Extract_only_namerec();
  }

  part=fmod(mben_old.codetxmt,1.0);
  rd[rdd.block_no].obs[0].rawrange       = 1.0 - part;
  rd[rdd.block_no].obs[0].smth_corr      = 0.0;
  rd[rdd.block_no].obs[0].smth_count     = 0;
  rd[rdd.block_no].obs[0].polarity_known = mben_old.polarity_known;
  rd[rdd.block_no].obs[0].warning        = mben_old.warning;
  rd[rdd.block_no].obs[0].goodbad        = mben_old.goodbad;
  rd[rdd.block_no].obs[0].ireg           = mben_old.ireg;
  rd[rdd.block_no].obs[0].qa_phase       = 0;
  rd[rdd.block_no].obs[0].doppler        = mben_old.doppler;
  rd[rdd.block_no].obs[0].carphase       = mben_old.intdoppler;

  rtc.last_count_mben=rtc.count_mben;
  rdd.block_no++;

  return(TRANS_OK);
}}


short RdRtime::Read_strctL1()
{{
  unsigned scheck,checksum;

  scheck=ChecksumXOR(bf.ibf,1); // -1 unused byte (checksum)
  checksum=(unsigned) bf.ibf[com.cnbyte-1];
  if (checksum!=scheck)
  { rtc.index_static_buf-=10;
    return(ERROR_CHECKSUM);
  }

  memcpy(&mben_L1,&bf.ibf[1],sizeof(mben_L1));
  Read_idfstrct(mben_L1.idf);
  Read_satstrct(mben_L1.sat);

  if (rh.num_obs_types!=1)
  { rh.capability=L1;
    rh.num_obs_types=1;

    Extract_only_namerec();
  }

  Read_meastrct(0,mben_L1.mea);

  rtc.last_count_mben=rtc.count_mben;
  rdd.block_no++;

  return(TRANS_OK);
}}


short RdRtime::Read_strctL2C()
{{
  unsigned scheck,checksum;

  scheck=ChecksumXOR(bf.ibf,1); // -1 unused byte (checksum)
  checksum=(unsigned) bf.ibf[com.cnbyte-1];
  if (checksum!=scheck)
  { rtc.index_static_buf-=10;
    return(ERROR_CHECKSUM);
  }

  memcpy(&mben_L2C,&bf.ibf[1],sizeof(mben_L2C));
  Read_idfstrct(mben_L2C.idf);
  Read_satstrct(mben_L2C.sat);

  if (rh.num_obs_types!=2)
  { rh.capability=L2C;
    rh.num_obs_types=2;

    Extract_only_namerec();
    strcat(rh.rcvr_type," C\0");
  }

  Read_meastrct(0,mben_L2C.mea);
  Read_codstrct(mben_L2C.cod);

  rtc.last_count_mben=rtc.count_mben;
  rdd.block_no++;

  return(TRANS_OK);
}}


short RdRtime::Read_strctL1CL12P()
{{
  short    i;
  unsigned scheck,checksum;

  scheck=ChecksumXOR(bf.ibf,1); // -1 unused byte (checksum)
  checksum=(unsigned) bf.ibf[com.cnbyte-1];
  if (checksum!=scheck)
  { rtc.index_static_buf-=10;
    return(ERROR_CHECKSUM);
  }

  memcpy(&mben_L1CL12P,&bf.ibf[1],sizeof(mben_L1CL12P));
  Read_idfstrct(mben_L1CL12P.idf);
  Read_satstrct(mben_L1CL12P.sat);

  if (rh.num_obs_types!=2)
  { rh.num_obs_types=2;

    Extract_only_namerec();
    if (fa.data_type==MP1_L1CP)
    { rh.capability=L1CP;
      strcat(rh.rcvr_type," P1\0");
    }
    if (fa.data_type==MP2_L1CL2P)
    { rh.capability=L1C_L2P;
      strcat(rh.rcvr_type," P2\0");
    }
  }

  for (i=0;i<rh.num_obs_types;i++)
    Read_meastrct(i,mben_L1CL12P.mea[i]);

  rtc.last_count_mben=rtc.count_mben;
  rdd.block_no++;

  return(TRANS_OK);
}}

short RdRtime::Read_strctL1CPL2P()
{{
  short    i;
  unsigned scheck,checksum;

  scheck=ChecksumXOR(bf.ibf,1); // -1 unused byte (checksum)
  checksum=(unsigned) bf.ibf[com.cnbyte-1];
  if (checksum!=scheck)
  { rtc.index_static_buf-=10;
    return(ERROR_CHECKSUM);
  }

  memcpy(&mben_L1CPL2P,&bf.ibf[1],sizeof(mben_L1CPL2P));
  Read_idfstrct(mben_L1CPL2P.idf);
  Read_satstrct(mben_L1CPL2P.sat);

  if (rh.num_obs_types!=3)
  { rh.capability=L1CP_L2P;
    rh.num_obs_types=3;

    Extract_only_namerec();
    strcat(rh.rcvr_type," P3\0");
  }

  for (i=0;i<rh.num_obs_types;i++)
    Read_meastrct(i,mben_L1CPL2P.mea[i]);

  rtc.last_count_mben=rtc.count_mben;
  rdd.block_no++;

  return(TRANS_OK);
}}


short RdRtime::Read_strctPBEN()
{{
  short  cur_day_week,error;
  double dif_rcv_time;
  short	 sum_mben;

  sum_mben=rdd.block_no;
  rtc.last_count_mben=99;
  rdd.block_no=0;

  if (!sum_mben) return(NO_FILLMBEN);
  if (sum_mben!=(short)rn.num_sats) rn.num_sats=(char)sum_mben;

  error=swap_strctPBEN();
  if (error!=TRANS_OK)
  { if (error==ERROR_CHECKSUM) rtc.index_static_buf-=10;
    rtc.error_epoch++;
    return(error);
  }

  if (!f_GPS_time)
  { if (rtc.snav>0L)
    { dif_rcv_time=fabs(rn.rcv_time-tow_ephem);
      rn.rcv_time-=10800.0;
      if (rn.rcv_time<0) rn.rcv_time+=86400.0;
      if (rn.rcv_time<86400.000001)
      { cur_day_week=day_week;
				dif_rcv_time=rn.rcv_time-tk_utc;
				if (dif_rcv_time<-43200.0) cur_day_week++;
				if (dif_rcv_time> 43200.0) cur_day_week--;
				if (cur_day_week<0) cur_day_week=6;
				if (cur_day_week>6) cur_day_week=0;
				rn.rcv_time=rn.rcv_time+86400.0*cur_day_week;
      }
    }
    else
      return(TRANS_OK);
  }

  if (!f_beg_rcv_time)
  { f_beg_rcv_time=1;
    beg_rcv_time=(long) rn.rcv_time;
  }
  
	// Write in B-file
  // rawheader ----------------------------------------------------
  oneEpoch_forBfile();

  return(TRANS_OK);
}}


short RdRtime::Read_strctSALM()
{{
  unsigned    scheck,checksum;
  rt_almstrct rt_alm;

  scheck=Checksum16(bf.ibf);
  memcpy(&checksum,&bf.ibf[com.cnbyte-2],sizeof(checksum));
  swap_nbytes((char *)&checksum,sizeof(checksum));
  if (checksum!=scheck)
  { rtc.index_static_buf-=10;
    return(ERROR_CHECKSUM);
  }

  memcpy(&rt_alm,&bf.ibf[1],sizeof(rt_alm));
  swap_nbytes((char *)&rt_alm.prn,sizeof(rt_alm.prn));
  if ((rt_alm.prn<0)||(rt_alm.prn>31)) return(TRANS_OK);

    alm[rt_alm.prn].e         = rt_alm.e;
    alm[rt_alm.prn].toe       = rt_alm.toe;
    alm[rt_alm.prn].i0        = rt_alm.i0;
    alm[rt_alm.prn].omegadot  = rt_alm.omegadot;
    alm[rt_alm.prn].health    = rt_alm.health;
    alm[rt_alm.prn].roota     = rt_alm.roota;
    alm[rt_alm.prn].omega0    = rt_alm.omega0;
    alm[rt_alm.prn].omega     = rt_alm.omega;
    alm[rt_alm.prn].m0        = rt_alm.m0;
    alm[rt_alm.prn].af0       = rt_alm.af0;
    alm[rt_alm.prn].af1       = rt_alm.af1;
    alm[rt_alm.prn].bulhealth = 256;
    alm[rt_alm.prn].wna       = rt_alm.wna;
    alm[rt_alm.prn].wn        = rt_alm.wn;
    alm[rt_alm.prn].tow       = rt_alm.tow;
    alm[rt_alm.prn].config    = 256;

    if (!rtf.f_wr_almGPS) rtf.f_wr_almGPS=1;

	swap_nbytes((char *)&rt_alm.wn,sizeof(rt_alm.wn));
  swap_nbytes((char *)&rt_alm.tow,sizeof(rt_alm.tow));
  if (rt_alm.wn>=0)
  { if (rt_alm.wn < 815) rt_alm.wn+=1024; // new epoch 22 august 1999 !!!
    rtt.alm_wn=rt_alm.wn;
    rtt.alm_tow=rt_alm.tow;
  }

  rtc.GPS_alm++;

  return(TRANS_OK);
}}

short RdRtime::Read_data()
{{
  short error=TRANS_OK;

  com.cnbyte-=2; // delete "\r\n"
  memmove(&bf.ibf[1],bf.ibf,com.cnbyte);
  com.cnbyte+=1; bf.ibf[0]=bf.tbf[0];

  switch ( fa.data_type )
  { case MBN_L1:
    case MCA_L1:
      if (rtf.old_mben_mode)
				error = Read_strctold();
      else
				error = Read_strctL1();
      break;
    case MCL_L2C:
      error = Read_strctL2C();
      break;
    case MP1_L1CP:
    case MP2_L1CL2P:
      error = Read_strctL1CL12P();
      break;
    case MPC_L1CPL2P:
      error = Read_strctL1CPL2P();
      break;
    case PBEN:
      error = Read_strctPBEN();
      break;
    case SNAV:
      error = Read_strctSNAV();
      break;
    case SALM:
      error = Read_strctSALM();
      break;
  }
  com.cnbyte=11; // read data !!!
	
	if (error!=TRANS_OK)
	{ FILE  *fL=NULL;
  	short length_path;
		char  _pathErrorfile[200];

  	if (pathDataFile[0]!='\0')
		{	strcpy(_pathErrorfile,pathDataFile);
			length_path=strlen(_pathErrorfile);
#ifdef __GNUC__				
			if (_pathErrorfile[length_path-1]!='/') strcpy(&_pathErrorfile[length_path],"/\0");
#else
			if (_pathErrorfile[length_path-1]!='\\')strcpy(&_pathErrorfile[length_path],"\\\0");			
#endif			
			strcat(_pathErrorfile,"error.log\0");
		}
		else
			strcpy(_pathErrorfile,"error.log\0");
		rdd.number_error++;
		fL=fopen(_pathErrorfile,"a+");
  	fprintf(fL,"\n\n Name:                    %s",pathImagefile);
  	fprintf(fL,"\n Data type:               %d",fa.data_type);

  	fprintf(fL,"\n Index in raw data:       %lu",curr_bytes-rtc.num_VIBF_SIZE+rtc.index_static_buf);
  	fprintf(fL,"\n Number of an error:      %lu", rdd.number_error);
		fclose(fL); fL=NULL;
  }

  return(error);
}}
