///////////////////////////////////////////////////////////////////////////////	
//    Ashtech receiver tools
//    Copyright (C) 1999  Magellan Coropration/Ashtech Precision Products 
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
///////////////////////////////////////////////////////////////////////////////


#include <string.h>
#include <math.h>
#include <stdlib.h>

#ifndef __RDRTIME_HPP
#include "rdrtime.hpp"
#endif

enum { SET_COM, ANS_COM };

RdRtime::RdRtime() : Trans()
{{
  rtf.real_time_mode=0;
  rec_type_code=UNKNOWN;
  rtt.difGMT=0.0;
	rtc.error_epoch=0L;
	fVWfile=NULL;
}}

short RdRtime::ansReceiverInfo(unsigned short numbytes)
{{
  short i;

  // -> com.cnbyte (\r\n) 
  // "$PASHR,RID,UZ,30,U122,BUEXM-T3J,0A12*5A\r\n\0" for MicroZ at work with disc only !!!
  // numbytes = 41
  com.cnbyte=numbytes;
  if (bf.ibf[0]!=bf.tbf[0]) return(ERROR_TYPEREC);
  if (strncmp((char *)bf.ibf,"$PASHR,RID,\0",11)) return(ERROR_TYPEREC);

  if (com.cnbyte<32)
  { for (i=com.cnbyte;i<32;i++) bf.ibf[i]='_';
    bf.ibf[i]='\0';
    com.cnbyte=32;
  }
  i=com.cnbyte-10;
  memmove(&bf.ibf[1],&bf.ibf[11],i);
  bf.ibf[0]='\x06';

  Identification_Receiver();
	if (rec_type_code==VISPA) return(TRANS_OK);
  if (rec_type_code==LCS_XII) rtf.old_mben_mode=1; else rtf.old_mben_mode=0;

  // for B-file:
  // rawheader ----------------------------------------------------
  BfileRawhead();

  return(TRANS_OK);
}}


void RdRtime::Init_almanac_strct()
{{
  short i;

  for (i=0;i<32;i++)
  { alm[i].e=alm[i].i0=alm[i].omegadot=alm[i].af0=alm[i].af1=0.0;
    alm[i].toe=alm[i].tow=0L;
    alm[i].health=alm[i].wna=alm[i].config=0;
    alm[i].wn=-1;
    alm[i].bulhealth=16128;
    alm[i].roota=alm[i].omega0=alm[i].omega=alm[i].m0=0.0;
  }
  rtf.f_wr_almGPS=0;
  rtc.GPS_alm=0;

  rtt.alm_wn=-1;
  rtt.alm_tow=0L;
	rdd.percent=0.0;
}}

short RdRtime::OpenLogFile(const char* _pathImagefile)
{{
  strcpy((char *)pathImagefile, _pathImagefile);
	fImage=fopen((char *)pathImagefile,"rb");
  if (fImage==NULL) return(NO_FILE);

	fseek(fImage,0L,SEEK_END); 
	rawfile_length=ftell(fImage);
	if (rawfile_length<0L) 
	{	fclose(fImage);
		fImage=NULL;
		return(NO_FILE);
	}
	fseek(fImage,0L,SEEK_SET); 
	curr_bytes=0L;
	
	return(TRANS_OK);
}}

void RdRtime::Init_analys_LogData()
{{
	bf.ibf[0]=header_buf[0]=bf.vibf[0]='\0';
	com.cnbyte=11;
  rtf.beg_SNV_data=0;
  rtc.last_count_mben=99;
  rdd.block_no=0;
  rtc.beg_buf=0;
  rtc.num_VIBF_SIZE=VIBF_SIZE;
  rtc.index_static_buf=0;
	rtc.snav=0L;
	rtc.GPS_alm=0;
	f_beg_rcv_time=0;
  beg_rcv_time=0L;
	rdd.number_error=0L;

	file_header.oper.mode=(char)0;
	file_header.segwn=-1;
	file_header.segtime=-1;
	memcpy(file_header.segname,"____\0",4);
	file_header.project.ses_name='A';	
	InitFileHeader_strct();
	
  FormInfFileRec();
	memcpy(&all_file_rec,&file_rec,sizeof(file_dir));
  FormVarNamesFiles();
	InitDownloadData();
}}


void RdRtime::RewriteHeaderBfile()
{{
	short len=0;

	if ((f_one_epoch)&&(rtc.snav>0L))
	{ file_header.segwn=esnav.wn;
		rh.wn_start=file_header.segwn;
		fwf.f_writeBfile=1;
		fwf.f_writeheaderCBfile=1; 
		len_epoch=0;
		len=sizeof(rh);
  	memcpy(&one_epochbuf[len_epoch],&rh,len);
    len_epoch+=len;
	}
}}


short RdRtime::ReadBuf()
{{
	if (curr_bytes==rawfile_length) return(TRANS_OK);
	if ((curr_bytes+rtc.num_VIBF_SIZE)>=rawfile_length)
		rtc.num_VIBF_SIZE=(unsigned short)(rawfile_length-curr_bytes);
	fread(&bf.vibf,rtc.num_VIBF_SIZE,1,fImage);	
	curr_bytes+=rtc.num_VIBF_SIZE;
	return(COLLECTING_DATA);
}}

short RdRtime::AnalysRdRtime()
{{
  int i,j,error;

  if(rtc.index_static_buf==rtc.num_VIBF_SIZE) 
	{	rtc.index_static_buf=0;
		return(FILLBUF);
	}

  if(rtc.index_static_buf+com.cnbyte>rtc.num_VIBF_SIZE)
  { rtc.beg_buf=rtc.num_VIBF_SIZE-rtc.index_static_buf;
    memcpy(bf.ibf,&bf.vibf[rtc.index_static_buf],rtc.beg_buf);
    rtc.index_static_buf=0;
		return(FILLBUF);
  }
  else
  { if(rtc.beg_buf)
    { memcpy(&bf.ibf[rtc.beg_buf],&bf.vibf[rtc.index_static_buf],
             com.cnbyte-rtc.beg_buf);
      rtc.index_static_buf=rtc.index_static_buf+(com.cnbyte-rtc.beg_buf);
      rtc.beg_buf=0;
    }
    else
    { memcpy(bf.ibf,&bf.vibf[rtc.index_static_buf],com.cnbyte);
      rtc.index_static_buf+=com.cnbyte;
    }
    bf.ibf[com.cnbyte]='\0';

    if (com.cnbyte<=11) // read Header !!!
    { if (com.cnbyte<11)
      { strcat((char *)header_buf,(char *)bf.ibf);
				strcpy((char *)bf.ibf,header_buf);
				header_buf[0]='\0';
      }
      if (bf.ibf[0]=='$')
			Analys_type_data(); // com.cnbyte for data !!!
      if (com.cnbyte==11)
      { if (bf.ibf[0]=='$') j=1; else j=0;
				for (i=j;i<11;i++) if (bf.ibf[i]=='$') break;
				if (i<11)
				{ strcpy(header_buf,(char *)&bf.ibf[i]);
					com.cnbyte=i;
				}
				else
					com.cnbyte=11;
			}
    }
    else
    { error=Read_data();
      return(error);
    }
    return(COLLECTING_DATA);
  }
}}

void RdRtime::Analys_type_data()
{{
  if (!strcmp((char *)bf.ibf,"$PASHR,MBN,\0"))
  { if (rtf.old_mben_mode) com.cnbyte=44; else com.cnbyte=39;
    fa.data_type=MBN_L1;
  }
  else
  if (!strcmp((char *)bf.ibf,"$PASHR,MCA,\0"))
  { com.cnbyte=39;  fa.data_type=MCA_L1;
  }
  else
  if (!strcmp((char *)bf.ibf,"$PASHR,MCL,\0"))
  { com.cnbyte=51;  fa.data_type=MCL_L2C;
  }
  else
  if (!strcmp((char *)bf.ibf,"$PASHR,MP1,\0"))
  { com.cnbyte=68;  fa.data_type=MP1_L1CP;
  }
  else
  if (!strcmp((char *)bf.ibf,"$PASHR,MP2,\0"))
  { com.cnbyte=68;  fa.data_type=MP2_L1CL2P;
  }
  else
  if (!strcmp((char *)bf.ibf,"$PASHR,MPC,\0"))
  { com.cnbyte=97;  fa.data_type=MPC_L1CPL2P;
  }
  else
  if (!strcmp((char *)bf.ibf,"$PASHR,PBN,\0"))
  { com.cnbyte=58;  fa.data_type=PBEN;
  }
  else
  if (!strcmp((char *)bf.ibf,"$PASHR,SNV,\0"))
  { com.cnbyte=134; fa.data_type=SNAV;
  }
  else
  if ((!strcmp((char *)bf.ibf,"$PASHR,SAL,\0")) ||
      (!strcmp((char *)bf.ibf,"$PASHR,ALM,\0"))
     )
  { com.cnbyte=72; fa.data_type=SALM;
  }
  else
    com.cnbyte=11;
}}

