///////////////////////////////////////////////////////////////////////////////	
//    Ashtech receiver tools
//    Copyright (C) 1999  Magellan Coropration/Ashtech Precision Products 
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
///////////////////////////////////////////////////////////////////////////////


#ifndef __RDRTIME_HPP
#define __RDRTIME_HPP

#ifndef __TRANS_HPP
#include "trans.hpp"
#endif

enum { MBN_L1=4, MCA_L1, MCL_L2C, MP1_L1CP, MP2_L1CL2P, MPC_L1CPL2P,
       SALM, GLONASS_SALM, ION, VARIATION
     };

enum { YREC=0, BHDR, BRX00X09, WHX00X16, WRDT, EREC, AREC, MREC, DRC_,
			 NO_TYPE_HEADER=-1	
		 };
 
const long idf_header_Ufile[9] =
{
  1128616537L,  // YREC
  1380206658L,	// BHDR
  151016002L,		// BR\X00\x09
  369117271L,		// WH\X00\x16
  1413763671L,	// WRDT		
  1128616517L,  // EREC
  1128616513L,	// AREC
  1128616525L,	// MREC
  541282884L,		// DRC' '
};

struct memory_count
{ unsigned long  num_blocks;
  unsigned long  sum_blocks;
  unsigned short index_MBN_PBN;
  unsigned char  f_almanac_time;
};

struct real_time_count
{ unsigned short num_com;

  unsigned long  epoch;
  unsigned long  snav;
  unsigned long  error_epoch;
  unsigned long  error_mc;
  unsigned short GPS_alm;
  unsigned short GLN_alm;

  unsigned short last_count_mben;
  unsigned short count_mben;
  unsigned short num_mben;

  unsigned short num_reiter;
  unsigned short nofull_readblock;
  
  unsigned long  index_Bfile;
  unsigned long  index_Efile;

  unsigned short index_bufBfile;
  unsigned short index_static_buf;
  unsigned short len_static_buf;
  unsigned short beg_buf;
  unsigned short num_VIBF_SIZE;
};

struct real_time_feature
{ char   real_time_mode;
  char   old_mben_mode;

  unsigned char f_wr_Bfile;

  unsigned char f_wr_almGPS;

  unsigned char beg_openBfile;
  unsigned char beg_openEfile;
  unsigned char beg_SNV_data;
};

struct real_time_time
{ double        difGMT;
  short         alm_wn;
  long          alm_tow;
  long          alm_Na;
  short         ion_wn;
  unsigned long ion_tow;
  long          day_year_stime;
};

#pragma pack(1) // begin 1 byte-alignment

struct mbenstrct_old
{ unsigned char   energy2;        // 1
  unsigned char   data_count;     // 1
  unsigned char   svprn;          // 1
  unsigned char   chnind;         // 1
  unsigned long   lost_lock_ctr;  // 4
  unsigned char   polarity_known; // 1
  unsigned char   goodbad;        // 1
  unsigned char   warning;        // 1
  unsigned char   ireg;           // 1
  double          codetxmt;       // 8
  long            doppler;        // 4
  double          intdoppler;     // 8
  unsigned short  unint_p2;       // 2
  short           int_L2diff;     // 2
  unsigned short  elevation;      // 2
  unsigned short  azimuth;        // 2
}; // 40 bytes

struct idfstrct
{ unsigned short sequence_tag;  // 2
  unsigned char  left;          // 1
}; // 3 bytes

struct satstrct
{ unsigned char  svprn;         // 1
  unsigned char  elevation;     // 1
  unsigned char  azimuth;       // 1
  unsigned char  chnind;        // 1
}; // 4 bytes

struct meastrct
{ unsigned char  warning;       // 1
  unsigned char  goodbad;       // 1
  unsigned char  polarity_know; // 1
  unsigned char  ireg;          // 1
  unsigned char  qa_phase;      // 1
  double         full_phase;    // 8
  double         raw_range;     // 8
  long           doppler;       // 4
  long           smoothing;     // 4
}; // 29 bytes

struct codstrct
{ double         carphase2;     // 8
  unsigned char  ireg2;         // 1
  char           qa_phase2;     // 1
  unsigned short spare;         // 2
}; // 12 bytes

struct mbenstrct_L1
{ idfstrct idf;
  satstrct sat;
  meastrct mea;
}; // 36 bytes

struct mbenstrct_L2C
{ idfstrct idf;
  satstrct sat;
  meastrct mea;
  codstrct cod;
}; // 48 bytes

struct mbenstrct_L1CL12P // for L1CP or L1C_L2P
{ idfstrct idf;
  satstrct sat;
  meastrct mea[2];
}; // 65 bytes

struct mbenstrct_L1CPL2P
{ idfstrct idf;
  satstrct sat;
  meastrct mea[3];
}; // 94 bytes

struct rt_almstrct
{ short  prn;
  short  health;
  float  e;
  long   toe;
  float  i0;
  float  omegadot;
  double roota;
  double omega0;
  double omega;
  double m0;
  float  af0;
  float  af1;
  short  wna;
  short  wn;
  long   tow;
}; // 68 bytes

struct rt_almstrct_g
{ short prn;
  short litera;
  short health;
  float e;
  long  Na;
  float di;
  float la;
  float tla;
  float w0;
  float dT;
  float dTt;
  float tn;
}; // 42 bytes
#pragma pack() // end 1 byte-alignment

class RdRtime : public Trans
{
  public:

    RdRtime();

    short ansReceiverInfo(unsigned short numbytes);
    short AlmanacNameFile();
    void  Init_almanac_strct();
    
    unsigned char buf_block[518];
    short         len_almanac;

    // for GG-receivers
    short AnalysDataBlocks(); 
		short EndReadFile(short error);

    // for MicroZ receiver
    short ans_FileDirCountMZ(const char *ansFirstNext);
    short AvailabilityDownloadFile(const unsigned char *varrawfile_name);
    void  InitHeadYFile();
    void  InitforReadVWfile();
		void  CloseReadVWfile();
		short ReadBlocksUfile(const char* _pathVWfile);
		
    // for Log RealTime file
		short OpenLogFile(const char* _pathImagefile);
		void	Init_analys_LogData();
		short ReadBuf();
		short AnalysRdRtime();
		void	RewriteHeaderBfile();

    unsigned long GetCurrbytesRawfiles() { return(curr_bytes); }
              
    unsigned long rawsession_bytes;
    
	  
  protected:
    //for MZ
    void  Init_varrawfile_name();
    void  Set_varrawfile_name(short index_file);
    void  FillInitCurrSession();
    void  FillSession();
    short ControlReceiverFileName();
    short ControlSession();
		void 	ReadfromYstrct(unsigned short &_i_head);
		void 	Identification_data_type();
		short	Read_data_fromRawfile();                
		short GetYbody();
		short GetHeaderBfile();
		short Read_ALMstrct();
		short GetPhotoData();
		short CompLengthEvent();
		short GetEventData();
    short CurrAlmanacName();

		void	Analys_type_data();
		short	Read_strctSNAV();
		void	Number_Sat();
		void	Read_idfstrct(idfstrct &idf);
		void	Read_satstrct(satstrct &sat);
		void	Read_meastrct(short i,meastrct &mea);
		void	Read_codstrct(codstrct &cod);
		void	Extract_only_namerec();
		short	Read_strctold();
		short	Read_strctL1();
		short	Read_strctL2C();
		short	Read_strctL1CL12P();
		short	Read_strctL1CPL2P();
		short	Read_strctPBEN();
		short	Read_strctSALM();
		short Read_data();
		
    unsigned long curr_bytes; 
    FILE          *fVWfile;
    char 					pathVWfile[200]; // curr V-file.
    unsigned long rawfile_length;                    
    char 					filedir_name[200];

		memory_count			vmem;	

    real_time_count   rtc;
    real_time_feature rtf;
    real_time_time    rtt;

    mbenstrct_old     mben_old;
    mbenstrct_L1      mben_L1;
    mbenstrct_L2C     mben_L2C;
    mbenstrct_L1CL12P mben_L1CL12P;
    mbenstrct_L1CPL2P mben_L1CPL2P;

		char header_buf[15];
};
#endif
