///////////////////////////////////////////////////////////////////////////////	
//    Ashtech receiver tools
//    Copyright (C) 1999  Magellan Coropration/Ashtech Precision Products 
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
///////////////////////////////////////////////////////////////////////////////


#include <math.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <time.h>

#ifndef __TRANS_HPP
#include "trans.hpp"
#endif


short Trans::swap_rawSNAV()
{{
  if (((rec_type_code!=Z_18)&&(rec_type_code!=MICRO_Z))||
  	 	((rec_type_code==Z_18)&&(f_swap_dataZ18))
  	 )
  { swap_nbytes((char *)&esnav.wn,sizeof(esnav.wn));
    swap_nbytes((char *)&esnav.tow,sizeof(esnav.tow));
    swap_nbytes((char *)&esnav.tgd,sizeof(esnav.tgd));
    swap_nbytes((char *)&esnav.aodc,sizeof(esnav.aodc));
    swap_nbytes((char *)&esnav.toc,sizeof(esnav.toc));
    swap_nbytes((char *)&esnav.af2,sizeof(esnav.af2));
    swap_nbytes((char *)&esnav.af1,sizeof(esnav.af1));
    swap_nbytes((char *)&esnav.af0,sizeof(esnav.af0));
    swap_nbytes((char *)&esnav.aode,sizeof(esnav.aode));
    swap_nbytes((char *)&esnav.deltan,sizeof(esnav.deltan));
    swap_nbytes((char *)&esnav.m0,sizeof(esnav.m0));
    swap_nbytes((char *)&esnav.e,sizeof(esnav.e));
    swap_nbytes((char *)&esnav.roota,sizeof(esnav.roota));
    swap_nbytes((char *)&esnav.toe,sizeof(esnav.toe));
    swap_nbytes((char *)&esnav.cic,sizeof(esnav.cic));
    swap_nbytes((char *)&esnav.crc,sizeof(esnav.crc));
    swap_nbytes((char *)&esnav.cis,sizeof(esnav.cis));
    swap_nbytes((char *)&esnav.crs,sizeof(esnav.crs));
    swap_nbytes((char *)&esnav.cuc,sizeof(esnav.cuc));
    swap_nbytes((char *)&esnav.cus,sizeof(esnav.cus));
    swap_nbytes((char *)&esnav.omega0,sizeof(esnav.omega0));
    swap_nbytes((char *)&esnav.omega,sizeof(esnav.omega));
    swap_nbytes((char *)&esnav.i0,sizeof(esnav.i0));
    swap_nbytes((char *)&esnav.omegadot,sizeof(esnav.omegadot));
    swap_nbytes((char *)&esnav.dot,sizeof(esnav.dot));
    swap_nbytes((char *)&esnav.accuracy,sizeof(esnav.accuracy));
    swap_nbytes((char *)&esnav.health,sizeof(esnav.health));
    swap_nbytes((char *)&esnav.fit,sizeof(esnav.fit));
  }
  fa.satnum=esnav.prnnum+1;

  if ((fa.satnum<1)||(fa.satnum>32)) return( NO_FILE );
  if (fabs(esnav.roota-5100.0)>200.0) return( NO_FILE );
	if (enav[fa.satnum-1].aodc==esnav.aodc) return( NO_FILE );
	if (esnav.wn < 815) esnav.wn+=1024; // new epoch 22 august 1999 !!! 
  wn_ephem=esnav.wn;
  tow_ephem=esnav.tow;

  day_week=(short) floor(tow_ephem/86400.0);
  tk_utc=tow_ephem-day_week*86400.0;

  memcpy(&enav[fa.satnum-1],&esnav,sizeof(enav[fa.satnum-1]));

  return(TRANS_OK);
}}


short Trans::swap_strctSNAV()
{{
  short          error;
  unsigned short check16,checksum;

  check16=Checksum16(bf.ibf);
  memcpy(&checksum,&bf.ibf[com.cnbyte-2],sizeof(checksum));
  swap_nbytes((char *)&checksum,sizeof(checksum));
  if (checksum!=check16) return(ERROR_CHECKSUM);

  memcpy(&esnav,&bf.ibf[1],sizeof(esnav));

  error=swap_rawSNAV();

  return( error );
}}

short Trans::ReadSNAV()
{{
  short len,error;

  // com.cnbyte -> bf.ibf
  memmove(&bf.ibf[1],bf.ibf,com.cnbyte);
  com.cnbyte+=1; bf.ibf[0]=bf.tbf[0];

  error=swap_strctSNAV();
  if (error!=TRANS_OK)
  { if (error==NO_FILE) error=TRANS_OK;
    return( error );
  }

  // Write in E-file
  // for write call Get_satnum()
  // (char) fa.satnum, sizeof(fa.satnum)
  // &enav[fa.satnum-1], sizeof(enav[fa.satnum-1]) 
  fwf.f_writeEfile=1;
  len_satbuf=0;
  len=sizeof(fa.satnum); 
  memcpy(&one_satbuf[len_satbuf],&fa.satnum,len);
  len_satbuf+=len;
  len=sizeof(enav[fa.satnum-1]);
  memcpy(&one_satbuf[len_satbuf],&enav[fa.satnum-1],len);
  len_satbuf+=len;  
  return( TRANS_OK );
}}

short Trans::ReadATT_3DF()
{{ // new format S-file
	short error=TRANS_OK;

  // com.cnbyte -> bf.ibf
	error=IndentBuf_CheckingChecksum16();
	if (error!=TRANS_OK) return(error);

  memcpy(&att_strct,&bf.ibf[1],sizeof(att_strct));

  swap_nbytes((char *)&att_strct.head,sizeof(att_strct.head));
  swap_nbytes((char *)&att_strct.pitch,sizeof(att_strct.pitch));
  swap_nbytes((char *)&att_strct.roll,sizeof(att_strct.roll));
  swap_nbytes((char *)&att_strct.brms,sizeof(att_strct.brms));
  swap_nbytes((char *)&att_strct.mrms,sizeof(att_strct.mrms));
  swap_nbytes((char *)&att_strct.tow,sizeof(att_strct.tow));

  // + S-file
  fwf.f_writeSfile=1;
  len_strS= sprintf(str_Sfile,"%9.2lf  %9.4lf %9.4lf %9.4lf %9.4lf %9.4lf    %d\r\n",
	      						((double) att_strct.tow)/1000.0,
	      						att_strct.head,att_strct.pitch,
	      						att_strct.roll,att_strct.mrms,
	      						att_strct.brms,att_strct.reset);

  return(TRANS_OK);
}}

short Trans::ReadHeadEVENT()
{{
  // com.cnbyte -> bf.ibf

  fa.event_type=bf.ibf[0];
  fa.string_length_event=bf.ibf[1];

  if (fa.string_length_event<8) return(ERROR_FAT);

  rdd.cword=fa.string_length_event/2+2; // string_length_event - event_time(8 byte)
					//                       and bar code
					// +1 checksum(2 byte)
					// +1 string_length as follow(2 byte)
  return(TRANS_OK);
}}

short Trans::ReadEVENT()
{{
	short					 wn_event,error=TRANS_OK;	
  unsigned short i;
  double         event_time;
  char           string[MAX_LEN_DSTR];

  // com.cnbyte -> bf.ibf
	error=IndentBuf_CheckingChecksum16();
	if (error!=TRANS_OK) return(error);

  memcpy(&event_time,&bf.ibf[3],sizeof(event_time));
  swap_nbytes((char *)&event_time,sizeof(event_time));
  event_time *=pow(10.0,-6.0);

  for (i=0;i<((unsigned short)fa.string_length_event-8);i++)
    string[i]=bf.ibf[11+i];
  string[i] = '\0';

  if (!isalnum(fa.event_type)) fa.event_type='C';

  if (esnav.wn>815) 
		wn_event=esnav.wn; 
	else
	if (rh.wn_start>815)
		wn_event=(short) rh.wn_start;
	else
		wn_event=0;	
	fwf.f_writeDfile=1;
	len_strD=0;
	str_Dfile[0]='\0';
	if (f_open_Dfile==0)
	{	f_open_Dfile=1;
		len_strD=sprintf(str_Dfile,"%s\r\n","%VER=1\0");
		len_strD+=sprintf(&str_Dfile[len_strD],"%s%s,%s,%s,%s\r\n","%RID=\0",rh.rcvr_type,rh.nav_ver,rid_options,ser_num_rec);	
	}
  len_strD+=sprintf(&str_Dfile[len_strD],"%c %lf,%4d\r\n%s\r\n",fa.event_type,event_time,wn_event,string);
		
  return(error);
}}

short Trans::ReadMBEN()
{{
	short			error=TRANS_OK;	
  mbenstrct mben;
  char      svprn;
  double    approx;
  double    del_intdop;
  long      N;

  // rawdata ------------------------------------------------------
  // com.cnbyte -> bf.ibf
	error=IndentBuf_CheckingChecksum16();
	if (error!=TRANS_OK) return(error);

  memcpy(&mben,&bf.ibf[1],sizeof(mben));
  swap_nbytes((char *)&mben.lost_lock_ctr,sizeof(mben.lost_lock_ctr));
  swap_nbytes((char *)&mben.codetxmt,sizeof(mben.codetxmt));
  swap_nbytes((char *)&mben.doppler,sizeof(mben.doppler));
  swap_nbytes((char *)&mben.intdoppler,sizeof(mben.intdoppler));
  swap_nbytes((char *)&mben.carphase1,sizeof(mben.carphase1));
  swap_nbytes((char *)&mben.carphase2,sizeof(mben.carphase2));
  swap_nbytes((char *)&mben.elevation,sizeof(mben.elevation));
  swap_nbytes((char *)&mben.azimuth,sizeof(mben.azimuth));

  rd[rdd.block_no].svprn =mben.svprn;
  rd[rdd.block_no].chnind=mben.chnind;
  rd[rdd.block_no].obs[0].polarity_known=mben.polarity_known;
  rd[rdd.block_no].obs[0].goodbad=mben.goodbad;
  rd[rdd.block_no].obs[0].rawrange=mben.codetxmt;
  rd[rdd.block_no].obs[0].ireg=mben.ireg;
  rd[rdd.block_no].obs[1].ireg=(unsigned char) mben.data_type;
  rd[rdd.block_no].obs[0].doppler=mben.doppler;
  rd[rdd.block_no].obs[0].carphase=mben.carphase1/10000.0;
  rd[rdd.block_no].obs[1].carphase=mben.carphase2/10000.0;
  rd[rdd.block_no].elevation=(unsigned char)(mben.elevation/100);
  rd[rdd.block_no].azimuth =(unsigned char)(mben.azimuth/2);
  rd[rdd.block_no].obs[0].warning=mben.warning;
  svprn= rd[rdd.block_no].svprn;
  rd[rdd.block_no].obs[0].smth_corr=0.0;
  rd[rdd.block_no].obs[1].smth_corr=0.0;
  rd[rdd.block_no].obs[0].smth_count=0;
  rd[rdd.block_no].obs[1].smth_count=0;

  del_intdop=(mben.intdoppler-cd.lastintdop[svprn-1]);
  cd.lastintdop[svprn-1]=mben.intdoppler;
  approx=del_intdop-(rd[rdd.block_no].obs[0].carphase-cd.carrier[svprn-1][0]);
  N=(approx>0.0)?(long)(approx+0.5):(long)(approx-0.5);
  rd[rdd.block_no].obs[0].qa_phase=(char)(fabs(N-approx)*100.0);
  cd.carrier[svprn-1][0]=N+rd[rdd.block_no].obs[0].carphase;
  rd[rdd.block_no].obs[0].carphase=cd.carrier[svprn-1][0];

  del_intdop *=(2.0*bFREQL2/bFREQL1);
  approx=del_intdop-(rd[rdd.block_no].obs[1].carphase-cd.carrier[svprn-1][1]);
  N=(approx>0.0)?(long)(approx+0.5):(long)(approx-0.5);
  rd[rdd.block_no].obs[1].qa_phase=(char)(fabs(N-approx)*100.0);
  cd.carrier[svprn-1][1]=N+rd[rdd.block_no].obs[1].carphase;
  rd[rdd.block_no].obs[1].carphase=cd.carrier[svprn-1][1];

  if (mben.lost_lock_ctr<cd.con_lloss_ctr[svprn-1][0])
    rd[rdd.block_no].obs[0].warning |=0x80;
  cd.con_lloss_ctr[svprn-1][0]=mben.lost_lock_ctr;

  rd[rdd.block_no].obs[1].polarity_known= rd[rdd.block_no].obs[2].polarity_known= 0;
  rd[rdd.block_no].obs[1].goodbad=        rd[rdd.block_no].obs[2].goodbad=        0;
  rd[rdd.block_no].obs[1].rawrange=       rd[rdd.block_no].obs[2].rawrange=       0.0;
  rd[rdd.block_no].obs[1].doppler=        rd[rdd.block_no].obs[2].doppler=        0;
  rd[rdd.block_no].obs[1].warning=        rd[rdd.block_no].obs[2].warning=        0;

  rdd.block_no++;
  rdd.addr +=(unsigned long) rdd.cword;

  if (mben.data_count<=0)
  { rn.num_sats=(char)rdd.block_no;
    rdd.addr-=(unsigned long) rdd.cword;    // -rdd.cword(mben)
    fa.cl_write_rawdata=1; //
    return(TRANS_OK);
  }

  return(NO_FILLMBEN);
}}


void Trans::swap_rawPBEN()
{{
  short i;

  if ((rec_type_code!=Z_18)||((rec_type_code==Z_18)&&(f_swap_dataZ18)))
  { swap_nbytes((char *)&pben.rcv_time,sizeof(pben.rcv_time));
    swap_nbytes((char *)&pben.navx,sizeof(pben.navx));
    swap_nbytes((char *)&pben.navy,sizeof(pben.navy));
    swap_nbytes((char *)&pben.navz,sizeof(pben.navz));
    swap_nbytes((char *)&pben.navxdot,sizeof(pben.navxdot));
    swap_nbytes((char *)&pben.navydot,sizeof(pben.navydot));
    swap_nbytes((char *)&pben.navzdot,sizeof(pben.navzdot));
    swap_nbytes((char *)&pben.navtdot,sizeof(pben.navtdot));
    swap_nbytes((char *)&pben.navt,sizeof(pben.navt));
    swap_nbytes((char *)&pben.pdop,sizeof(pben.pdop));
  }

  for (i=0;i<4;i++) rn.sitename[i]=pben.sitename[i];
  rn.rcv_time = (double)(pben.rcv_time)/1000.0;
  rn.navx=pben.navx;
  rn.navy=pben.navy;
  rn.navz=pben.navz;
  rn.navxdot=pben.navxdot;
  rn.navydot=pben.navydot;
  rn.navzdot=pben.navzdot;
  rn.navt=pben.navt;
  rn.navtdot=pben.navtdot;
  rn.pdop=pben.pdop;

  f_GPS_time=0;
  for (i=0;i<rn.num_sats;i++)
  { if (rd[i].svprn<33)
    { if ((rd[i].obs[0].rawrange<1.0)&&(rd[i].obs[0].rawrange>0))
      { f_GPS_time=1;
				break;
      }
    }
    else
    { if (rd[i].obs[0].rawrange>1.0)
      { f_GPS_time=1;
				break;
      }
    }
  }
}}


short Trans::swap_strctPBEN()
{{
  unsigned short check16,checksum;

  check16=Checksum16(bf.ibf);
  memcpy(&checksum,&bf.ibf[com.cnbyte-2],sizeof(checksum));
  swap_nbytes((char *)&checksum,sizeof(checksum));
  if (checksum!=check16) return(ERROR_CHECKSUM);

  memcpy(&pben,&bf.ibf[1],sizeof(pben));
  swap_rawPBEN();

  return(TRANS_OK);
}}

short Trans::ReadPBEN()
{{
  short  error;
  short  j;
  double dummy;

  // rawnaw ------------------------------------------------------
  // com.cnbyte -> bf.ibf
  memmove(&bf.ibf[1],bf.ibf,com.cnbyte);
  com.cnbyte+=1; bf.ibf[0]=bf.tbf[0];

  error=swap_strctPBEN();
  if (error!=TRANS_OK) return(error);

  if (fa.cl_write_rawdata) // ok data epoch !!!
  { fa.cl_write_rawdata=0;
    for (j=0;j<rn.num_sats; j++)
    { if ((rd[j].svprn<1)||(rd[j].svprn>=38)) continue;
      rd[j].obs[0].rawrange=1.0-modf(rd[j].obs[0].rawrange,&dummy);
    }
    // Write in B-file
    // rawheader ----------------------------------------------------
    oneEpoch_forBfile();
  }

  return(TRANS_OK);
}}

short Trans::ReadRNAV()
{{
  short word_num;
	short error=TRANS_OK;

  // com.cnbyte -> bf.ibf
	error=IndentBuf_CheckingChecksum16();
	if (error!=TRANS_OK) return(error);

  memcpy(&rnav,&bf.ibf[1],sizeof(rnav));

  word_num=rnav.prnnum;
  swab((char *)&word_num,(char *)&rnav.prnnum,2);
  for (word_num=0;word_num<10;word_num++)
  { pack_word_bits(rnav.frame1,word_num);
    pack_word_bits(rnav.frame2,word_num);
    pack_word_bits(rnav.frame3,word_num);
  }

  // -> &rnav, sizeof(rnav)
  fwf.f_writeNfile=1;
  len_rnav=sizeof(rnav); 
  
  return(error);
}}

short Trans::ReadRALMnpages()
{{
  // com.cnbyte -> bf.ibf
  memcpy(&fa.num_pages_ralm,bf.ibf,
	    sizeof(fa.num_pages_ralm));
  swap_nbytes((char *)&fa.num_pages_ralm,
      sizeof(fa.num_pages_ralm));

  rdd.cword=20*fa.num_pages_ralm+1; // +1 checksum
  rdd.addr +=1L;

  if (fa.num_pages_ralm<0) return(ERROR_FAT);
  return(TRANS_OK);
}}

short Trans::ReadRALM()
{{
  short error=TRANS_OK;
  short len,page;
  short word_num;

  // com.cnbyte -> bf.ibf
	error=IndentBuf_CheckingChecksum16();
	if (error!=TRANS_OK) return(error);

  memcpy(&ralm,&bf.ibf[1],20*fa.num_pages_ralm);

  for (page=0;page<fa.num_pages_ralm;page++)
    for (word_num=0;word_num<10;word_num++)
      pack_word_bits(ralm.frame[page],word_num);

  // -> fa.num_pages_ralm
  // -> &ralm , sizeof = fa.num_pages_ralm*40
  fwf.f_writeAfile=1;
  len_strRA=0;
  len=sizeof(fa.num_pages_ralm);
  memcpy(&str_RAfile[len_strRA],&fa.num_pages_ralm,len);
  len_strRA+=len;
  len=fa.num_pages_ralm*40;     
  memcpy(&str_RAfile[len_strRA], &ralm, len);
  len_strRA+=len;
  
  return(TRANS_OK);
}}
