///////////////////////////////////////////////////////////////////////////////	
//    Ashtech receiver tools
//    Copyright (C) 1999  Magellan Coropration/Ashtech Precision Products 
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
///////////////////////////////////////////////////////////////////////////////


#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <math.h>

#ifndef __TRANS_HPP
#include "trans.hpp"
#endif 

#ifndef __HOSEUTIL_HPP
#include "hoseutil.hpp"
#endif 

void Trans::BfileRawhead()
{{
  unsigned short i;

  for (i=0;i<10;i++)
    rh.version[i]=rh.rcvr_type[i]=rh.chan_ver[i]=rh.nav_ver[i]='\0';
  for (i=0;i<42;i++) rh.spare[i]=0;
  strcpy(rh.rcvr_type,"unknown");
  strcpy(rh.chan_ver, "unknown");
  strcpy(rh.nav_ver,  "unknown");
  strcpy(rh.version,"Version:\0");
  rh.raw_version=3;

  strcpy(rh.rcvr_type,ReceiverName(rec_type_code));

  strcpy(rh.chan_ver,(char *)typ_rec.channel_version);
  strcpy(rh.nav_ver,(char *)typ_rec.nav_version);

  if (typ_rec.channel_option<=3)
  { switch (typ_rec.channel_option)
    { case 0 : 
				if (typ_rec.codeless_option)
	      { rh.capability=L2C;
		      rh.num_obs_types=2;
		      strcat(rh.rcvr_type," C\0");
	      }
	      else
	      { rh.capability=L1;
		      rh.num_obs_types=1;
	      }
	      break;
      case 1 :
				rh.capability=L1CP;
	      rh.num_obs_types=2;
	      strcat(rh.rcvr_type," P1\0");
	      break;
      case 2 :
				rh.capability=L1C_L2P;
	      rh.num_obs_types=2;
	      strcat(rh.rcvr_type," P2\0");
	      break;
      case 3 :
				rh.capability=L1CP_L2P;
	      rh.num_obs_types=3;
	      strcat(rh.rcvr_type," P3\0");
	      break;
      default: 
				rh.capability=UNKN;
    }
  }
  else
  { /* Determine if L1 or L2 capable */
    if (isdigit(typ_rec.channel_version[0]))
      typ_rec.channel_option=typ_rec.channel_version[0]-'0';
    else
      typ_rec.channel_option=typ_rec.channel_version[1]-'0';
    if (!strcmp((char *)typ_rec.channel_version,"N/A"))
      typ_rec.channel_option=1;
    switch (typ_rec.channel_option & 0x03)
    { case 1 :
				rh.capability   =L1;
	      rh.num_obs_types=1;
	      break;
      case 2 : 
				rh.capability   =L2C;
	      rh.num_obs_types=2;
	      strcat(rh.rcvr_type," C\0");
	      break;
      case 3 :
				rh.capability   =L1CP_L2P;
	      rh.num_obs_types=3;
	      strcat(rh.rcvr_type," P3\0");
	      break;
      default: 
				rh.capability=UNKN;
    }
  }
  cd.capability=    rh.capability;
  cd.num_obs_types= rh.num_obs_types;
}}


short Trans::Change_num_obs_types()
{{ // for "AnalysReadfRec"{rfromrec.cpp} and "GetDataVtoDiscFiles"{mzmemory.cpp}
  if (rec_type_code==LOCUS) 
  { rh.capability=L1;
    rh.num_obs_types=1;
		return(TRANS_OK);
  }

  /* Look for Change from P-Code to codeless. */
  if ((rh.capability == L1CP_L2P || rh.capability == L1C_L2P) &&
			fa.data_type == TINYCODELESS
		 )
  { rh.capability     = L2C;
    rh.num_obs_types  = 2;
  }
  else
  /* Look for Change from Codeless to P-Code */
  if ((rh.capability == L2C) && (fa.data_type == PTINYTYPE))
  { switch (rh.rcvr_type[strlen (rh.rcvr_type) - 1])
    { case '2':
		  	rh.capability    = L1C_L2P;
				rh.num_obs_types = 2;
				break;
      case '3':
				rh.capability    = L1CP_L2P;
				rh.num_obs_types = 3;
				break;
    }
  }
  else
  if (fa.curr_file_type == RANGER_MODE1)
  { rh.capability    = CDPHASE;
    rh.num_obs_types = 1;
  }
  else
  { rh.capability=    cd.capability;
    rh.num_obs_types= cd.num_obs_types;
  }

  if ((rh.capability == UNKN)&&(fa.curr_file_type != RANGER_MODE2))
    return(ERROR_CAPABILITY);
  return(TRANS_OK);
}}

//-------------------------------------------------------------------------
// if NAV version greater than follow condition, antenna height max from 6.0->64
// Z12  version > 1I17
// RZ   version > 5J00
// SCA  version > 1I00
void Trans::adj_ant_factor(void)
{{
  char  chg = 0;
  short num = atoi((char *)typ_rec.nav_version);
  if(rec_type_code == SUPER_CA)
  { // check whether greater than 1I00
    if(num>=1)
    { if(num >1)
				chg = 1;
      else if(typ_rec.nav_version[1] >= 'I')
      { if((num=atoi((char *)&typ_rec.nav_version[2]))>0)
	  			chg = 1;
      }
    }
  }
  else if(rec_type_code == Z_XII)
  { // RZ   check greater than 5J00
    if(typ_rec.nav_version[1] == 'J' && num >= 5)
    { chg = 1;
    }
    // Z12
    else if(num>=1 && (typ_rec.nav_version[1] == 'I' || typ_rec.nav_version[1] >= 'K'))
    { if(num>1)
				chg = 1;
      else if( typ_rec.nav_version[1] == 'I')
      {	// 1I18 and above
				if((num = atoi((char *)&typ_rec.nav_version[2]))>17)
	  			chg = 1;
      }
      else  // 1K and above
				chg = 1;
    }
  }
  if(chg)
    ant_factor=(float)(ant_factor/10.0);
}}

short Trans::Identification_Receiver()
{{
  char  types_receiver[3];
  short n,ns,i,j;

  strncpy((char *)typ_rec.receiver_type,(char *)&bf.ibf[1],2); typ_rec.receiver_type[2]='\0';
  for (i=1;i<KNOWN_RECEIVERS_NUMBER;i++)
  { strcpy(types_receiver,ReceiverNameEcho(i));
    if ( types_receiver[1]=='x' &&
				 types_receiver[0]==typ_rec.receiver_type[0] )
      break;
    if ( !strncmp((char *)typ_rec.receiver_type,types_receiver,2) )
      break;
  }
  rec_type_code=i%KNOWN_RECEIVERS_NUMBER;    
													// UNKNOW  or LM_XII   or
					     						// MS_XII  or M_XII    or
					     						// D_XII   or P_XII    or
					     						// DF3     or RANGER   or
					     						// Z_XII   or SUPER_CA or
					     						// LCS_XII or G_XII    or
													// VISPA	 or G_XXIV   or 
													// MICRO_Z or Locus		 or 
													// HP48    or Z_18		 or 
													// GGG
  if (rec_type_code==VISPA) return(-1);
	typ_rec.channel_option=bf.ibf[4]-48;
  typ_rec.codeless_option=bf.ibf[5]-48;
  if (rec_type_code==LOCUS) { typ_rec.channel_option--; typ_rec.codeless_option=0; }
  if ((rec_type_code==SUPER_CA)||(rec_type_code==LCS_XII)) typ_rec.channel_option=4;
  if ((rec_type_code==GGG)||(rec_type_code==Z_18)) typ_rec.channel_option=3;
  if (rec_type_code>=Z_XII) ns=4; else ns=2;

  for (j=7;j>0;j--)
    if (bf.ibf[j]==',') break;
  n=j+1;
  for (i=n,j=0;(bf.ibf[i]!=',')&&(i<4+n);i++,j++)
    typ_rec.nav_version[j]=bf.ibf[i];
  typ_rec.nav_version[j]='\0';
	// rid_options -> for write in D-file only !!!
	for (j=0;j<99;j++)
	{	i++;
		if ((bf.ibf[i]==',')||(bf.ibf[i]=='*'))	break;
		rid_options[j]=bf.ibf[i];
	}
	rid_options[j]='\0';
	
  i=8+ns;
  typ_rec.co_processor=bf.ibf[i];
  if (rec_type_code>Z_XII) typ_rec.co_processor='N';
  i=i+1;
  strncpy((char *)typ_rec.receiver_option,(char *)&bf.ibf[i],2);
  typ_rec.receiver_option[2]='\0';
  i=i+3;
  strncpy((char *)typ_rec.channel_version,(char *)&bf.ibf[i],ns);
  typ_rec.channel_version[ns]='\0';

  if (!isalnum(typ_rec.nav_version[0]))
    strcpy ((char *)typ_rec.nav_version,"N/A\0");
  if (!isalnum(typ_rec.channel_version[0]))
    strcpy ((char *)typ_rec.channel_version,"N/A\0");
  if (!isalnum(typ_rec.receiver_option[0]))
    strcpy ((char *)typ_rec.receiver_option,"N/A\0");
  if ((rec_type_code==M_XII)   || (rec_type_code==D_XII)    ||
      (rec_type_code==MS_XII)  || (rec_type_code==SUPER_CA) ||
      (rec_type_code==LCS_XII) || (rec_type_code==G_XII)    ||
			(rec_type_code==G_XXIV)  || (rec_type_code==MICRO_Z)  ||
			(rec_type_code==LOCUS)   || (rec_type_code==Z_18)     ||
			(rec_type_code==GGG)
     )
  { strcpy ((char *)typ_rec.receiver_option,"N/A\0");
    strcpy ((char *)typ_rec.channel_version,"N/A\0");
  }

  if ((typ_rec.nav_version[0]=='5')&&(typ_rec.nav_version[1]<'E'))
    fa.rcvr_version=A_5;
  else
  { if ((typ_rec.nav_version[0]=='6')&&(typ_rec.nav_version[1]<'F'))
      fa.rcvr_version=A_6;
    else
    if ((typ_rec.nav_version[0]=='5')&&(typ_rec.nav_version[1]<'F'))
			fa.rcvr_version=E_5;	
		else
			fa.rcvr_version=F_6;
  }

  i=i+ns+1;

  if ((rec_type_code>=SUPER_CA)&&(rec_type_code!=MICRO_Z)) i=27;
  if ((!isalnum(bf.ibf[i]))&&(rec_type_code==SUPER_CA)) i++;

  ant_factor=10000.0;
  adj_ant_factor();
	
	return(i);
}}

short Trans::ChangeSessionName()
{{
  short  j;
  long   year,day;
  long   time[2];
  double sec;

	if ((!f_renfile_cline)&&(file_header.segwn>0))
	{	file_header.segtime=(long) rn.rcv_time;
		gps_to_ydhms(file_header.segwn,file_header.segtime,
                 &year,&day,&time[0],&time[1],&sec);
		if (year>=1900) year -=1900;
		if (year>=100) year -=100;
		strncpy((char *)file_rec.file_name,rn.sitename,4);
		for (j=0;j<4;j++)
			if ((file_rec.file_name[j]=='\0')||(file_rec.file_name[j]=='?'))
				file_rec.file_name[j] = '_';
		sprintf((char *)&file_rec.file_name[5],"%2.2ld.%3.3ld\0",year,day);
		return(TRANS_OK);
	}
	return(NO_FILE);
}}


void Trans::FormInfFileRec()
{{ 
  short  j;
  long   year,month,day;
  long   mday,time[3];
  double sec;

  if (file_header.segwn<0)
  { year=0L;
    day=0L;
    time[0]=time[1]=time[2]=0L;
    month=0L;
    mday=0L;

    sprintf((char *)file_rec.date,"__/__/__\0");
    sprintf((char *)file_rec.time,"__/__/__\0");
  }
  else
  { gps_to_ydhms(file_header.segwn,file_header.segtime,
                 &year,&day,&time[0],&time[1],&sec);
    time[2]=(long) (sec+0.5);
    if (time[2]>=60L) { time[1]++; time[2] -=60L; }
    if (time[1]>=60L) { time[0]++; time[1] -=60L; }
    month_and_day(year,day,&month,&mday);
    if (year>=1900) year -=1900;
    if (year>=100) year -=100;

    sprintf((char *)file_rec.date,"%2.2ld/%2.2ld/%2.2ld\0",month,mday,year);
    sprintf((char *)file_rec.time,"%2.2ld/%2.2ld/%2.2ld\0",time[0],time[1],time[2]);
  }

	if (file_rec.file_name[0]=='\0')
  { f_renfile_cline=0;
		if (rec_type_code!=MICRO_Z) // for MicroZ file_name fill before ( see GetHeadYFile ).
		{ strcpy((char *)file_rec.file_name,"____A__.___\0");
			strncpy((char *)file_rec.file_name,file_header.segname,4);
			for (j=0;j<4;j++)
				if ((file_rec.file_name[j]=='\0')||(file_rec.file_name[j]=='?'))
					file_rec.file_name[j] = '_';
		}
 		// !! -- session -- !!
		if ((rec_type_code!=MICRO_Z)&&(file_header.segwn>=0))
		{ if (!isalnum(file_header.project.ses_name))
			  file_rec.session='A';
			else
				file_rec.session=file_header.project.ses_name;
			file_rec.file_name[4]=file_rec.session;
			sprintf((char *)&file_rec.file_name[5],"%2.2ld.%3.3ld\0",year,day);
		}
	}
	else
		f_renfile_cline=1;
  rh.wn_start=file_header.segwn; // for rawheader B-file !!!!!!!!!!!!!
  fa.change_wn=0;
	strncpy((char *)file_rec.site,file_header.segname,4); file_rec.site[4]='\0';
	if (rec_type_code!=MICRO_Z)
    sprintf((char *)file_rec.bytes,"%7ld\0",file_header.wordcount*2L);
  strncpy((char *)file_rec.rcv_num,file_header.project.rcv_num,3);
  file_rec.rcv_num[3]='\0';
  strncpy((char *)file_rec.ant_num,file_header.project.ant_num,3);
  file_rec.ant_num[3]='\0';
  strncpy((char *)file_rec.operators,file_header.project.operat,3);
  file_rec.operators[3]='\0';
  strcpy((char *)file_rec.file_type,(char *)file_types[file_header.oper.mode]);
  fa.curr_file_type=
	      file_header.oper.mode; // GEODETIC or RANGER_MODE1 or -"-2 or -"-4
  strncpy((char *)file_rec.comment,file_header.project.comment,13);
  file_rec.comment[13]='\0';
  if (rec_type_code!=MICRO_Z) 
  { file_rec.ahibef=    file_header.anthibef/ant_factor; 
    file_rec.ahiaft=    file_header.anthiaft/ant_factor; 
  }
  file_rec.presbef=     file_header.pressurebef/10.0f;
  file_rec.presaft=     file_header.pressureaft/10.0f;
  file_rec.temp_drybef= file_header.drybef;
  file_rec.temp_dryaft= file_header.dryaft;
  file_rec.temp_wetbef= file_header.wetbef;
  file_rec.temp_wetaft= file_header.wetaft;
  file_rec.humiditybef= file_header.humidbef;
  file_rec.humidityaft= file_header.humidaft;
}}

void Trans::FormVarNamesFiles()
{{ // Form. names B-file,E-file,S-file and another. 
  // C-file image B-file for RANGER 2
  vfile_name.nameB_file[0]=(fa.curr_file_type==RANGER_MODE2) ? 'C' : 'B';
  strcpy((char *)&vfile_name.nameB_file[1],(char *)file_rec.file_name);

  vfile_name.nameE_file[0]='E';
  strcpy((char *)&vfile_name.nameE_file[1],(char *)file_rec.file_name);
  if (fa.curr_file_type==RANGER_MODE2)
    strcpy((char *)vfile_name.nameE_file,"N/A\0");

  vfile_name.nameS_file[0]='S';
  strcpy((char *)&vfile_name.nameS_file[1],(char *)file_rec.file_name);

  vfile_name.nameD_file[0]='D';
  strcpy((char *)&vfile_name.nameD_file[1],(char *)file_rec.file_name);

  vfile_name.nameN_file[0]='N';
  strcpy((char *)&vfile_name.nameN_file[1],(char *)file_rec.file_name);

  vfile_name.nameA_file[0]='A';
  strcpy((char *)&vfile_name.nameA_file[1],(char *)file_rec.file_name);

  vfile_name.nameP_file[0]='P';
  strcpy((char *)&vfile_name.nameP_file[1],(char *)file_rec.file_name);
}}

void Trans::InitDownloadData() 
{{
  unsigned short i,j;
  
  // Init. data !!!
  esnav.wn=-1; // for correction wn in D-file.
	f_open_Dfile=0;
	for (j=0;j<32;j++)
  { enav[j].roota=0.0; 
    enav[j].wn=0;      
    enav[j].health=0;
  }

  for (i=0;i<3;i++) cd.azel_cp[i]=0.0; 
  cd.azel_cp[3]=150000.0;              // navx,navy,navz,navt
  for (i=0;i<12;i++) cd.txmt[i]=0.0;
  for (i=0;i<4;i++)  cd.ep[i]=0.0;

  for (j=0;j<32;j++)   
  { cd.lastintdop[j]=0.0; // for warning
    for (i=0;i<3;i++)  // for ReadA5BENstruct,ReadBENstruct
    { cd.carrier[j][i]=0.0;
      cd.con_lloss_ctr[j][i]=0.0;
    }
  }
  rn.rcv_time=0.0;
  rn_ranger2.rcv_time=0.0;
  fa.cl_write_rawdata=0;
	rdd.addr_begBEN=0L;
}}

short Trans::GetHeadFile()
{{
  unsigned short check16;
  
  // com.cnbyte(80) -> bf.ibf
  memmove(&bf.ibf[1],bf.ibf,com.cnbyte);
  com.cnbyte+=1; bf.ibf[0]=bf.tbf[0];

  check16=Checksum16(bf.ibf);
  memcpy(&file_header,&bf.ibf[1],sizeof(file_header));
  swap_nbytes((char *)&file_header.checksum,sizeof(file_header.checksum));
	// for read R-file ERROR_CHECKSUM is not inspected.
	// if (file_header.checksum!=check16) return(ERROR_CHECKSUM); 

  swap_nbytes((char *)&file_header.segbeg ,sizeof(file_header.segbeg));
  swap_nbytes((char *)&file_header.wordcount ,sizeof(file_header.wordcount));
  if (!file_header.wordcount) return(ERROR_FAT);

  swap_nbytes((char *)&file_header.segwn ,sizeof(file_header.segwn));
  if (file_header.segwn < 815) file_header.segwn+=1024; // new epoch 22 august 1999 !!! 
  swap_nbytes((char *)&file_header.segtime ,sizeof(file_header.segtime));
  swap_nbytes((char *)&file_header.anthibef ,sizeof(file_header.anthibef));
  swap_nbytes((char *)&file_header.anthiaft ,sizeof(file_header.anthiaft));
  swap_nbytes((char *)&file_header.pressurebef,
		sizeof(file_header.pressurebef));
  swap_nbytes((char *)&file_header.pressureaft,
		sizeof(file_header.pressureaft));

  strcpy(rn.sitename,file_header.segname);

  FormInfFileRec();
  FormVarNamesFiles();

  InitDownloadData();
   
  return(TRANS_OK);
}}


void Trans::InitFileHeader_strct()
{{
  short i;

	for (i=0;i<3;i++)
	{	file_header.project.rcv_num[i]='_';
		file_header.project.ant_num[i]='_';
    file_header.project.operat[i]='_';
	}
	for (i=0;i<2;i++)
	{ file_header.project.month[i]='_';
		file_header.project.day[i]='_';
	}
	for (i=0;i<13;i++) file_header.project.comment[i]='_';
  file_header.anthibef=0;
  file_header.anthiaft=0; 
  file_header.pressurebef=0;
  file_header.pressureaft=0;
  file_header.drybef=file_header.dryaft=(char)0;
  file_header.wetbef=file_header.wetaft=(char)0;
  file_header.humidbef=file_header.humidaft=(char)0;
}}


short Trans::GetRingHead()
{{
  long  curr_pos;

	// com.cnbyte(14) -> bf.ibf
  if (IndentBuf_CheckingChecksum16()!=TRANS_OK) return(ERROR_CHECKSUM);
	
	memcpy(&ring_head,&bf.ibf[1],sizeof(ring_head));
	swap_nbytes((char *)&ring_head.num_records,sizeof(ring_head.num_records));
  swap_nbytes((char *)&ring_head.slot_idx,sizeof(ring_head.slot_idx));
  swap_nbytes((char *)&ring_head.record_size,sizeof(ring_head.record_size));
  swap_nbytes((char *)&ring_head.wkstop,sizeof(ring_head.wkstop));
  swap_nbytes((char *)&ring_head.timestop,sizeof(ring_head.timestop));
	
	file_header.segbeg=sizeof(ring_head)/2L;
	if ( ring_head.record_size != 0 )
	{ file_header.wordcount=(long)(ring_head.record_size*ring_head.num_records);
		file_header.wordcount+=(16L*sizeof(snavstrct)); // 16 -> 32/2 in words
	}
	else
	{	curr_pos=ftell(fImage);
		fseek(fImage,0L,SEEK_END); 
		file_header.wordcount=(ftell(fImage)-curr_pos)/2L;
		fseek(fImage,curr_pos,SEEK_SET); 
	}

	file_header.segwn=ring_head.wkstop;
	if (file_header.segwn < 815) file_header.segwn+=1024; // new epoch 22 august 1999 !!! 
	file_header.segtime=ring_head.timestop;
  file_header.oper.mode=(char)3;
	file_header.project.ses_name=pathImagefile[f_letter_name+5];
	memcpy(file_header.segname,&pathImagefile[f_letter_name+1],4);	

	InitFileHeader_strct();

  FormInfFileRec();
  FormVarNamesFiles();

  InitDownloadData();
	
	return(TRANS_OK);
}}

void Trans::GetRingAddrOffset(char mode)
{{
  unsigned long offset;
	long					curr_pos;					

 	rdd.ben_number=0;
	rdd.curr_ben_number=0;
	switch (mode)
	{ // find first record ram address and number of records in the first part
    case 0:
      if(ring_head.slot_idx>1) 
			{ // save the begining address of record data for second call
        rdd.addr_begBEN=rdd.addr;
        rdd.ben_number=ring_head.num_records-ring_head.slot_idx+1;
        offset=(ring_head.slot_idx-1)*ring_head.record_size;
        rdd.addr+=offset;
      }
      else
        rdd.ben_number=ring_head.num_records;
      break;
    // find number of records in the second part, retrieve ram add to
    // the begining of the record data
    case 1:
      if(rdd.addr_begBEN)
        rdd.addr=rdd.addr_begBEN;
      rdd.ben_number=ring_head.slot_idx-1;
      break;
    default:
      break;
  }
	curr_pos=sizeof(image_head)+rdd.addr*2;
	fseek(fImage,curr_pos,SEEK_SET);
}}

void Trans::InitSNV_parameters()
{{
  short i;

  esnav.wn=-1; // for correction wn in D-file.
	for (i=0;i<32;i++) enav[i].aodc=0L;
  for (i=0;i<24;i++) enav_g[i].reftime=0L;

	f_GPS_time=0;               
  f_one_epoch=0; // flag write rh in one_epochbuf.
	rdd.snav_number=0L;
}}

short Trans::GetTypeData()
{{
  short data_type;

  // com.cnbyte(2) -> bf.ibf
  memcpy(&data_type,bf.ibf,sizeof(data_type));
  swap_nbytes((char *)&data_type,sizeof(data_type));

  return(data_type);
}}

void Trans::oneEpoch_forBfile()
{{
  short j,len;

  fwf.f_writeBfile=1;
  len_epoch=0;
  if (!f_one_epoch)
  { f_one_epoch=1;
  	strcpy(rh.spare,ser_num_rec);
		len=sizeof(rh);
  	memcpy(&one_epochbuf[len_epoch],&rh,len);
    len_epoch+=len;
		if (rec_type_code==LOCUS) return;  
  }  

  len=sizeof(rn); 
  memcpy(&one_epochbuf[len_epoch],&rn,len);
  len_epoch+=len;

  len=4+sizeof(chan_obs)*rh.num_obs_types;
  for (j=0;j<rn.num_sats;j++)
  { memcpy(&one_epochbuf[len_epoch],&rd[j].svprn,len);
    len_epoch+=len;
  }
}}
