///////////////////////////////////////////////////////////////////////////////	
//    Ashtech receiver tools
//    Copyright (C) 1999  Magellan Coropration/Ashtech Precision Products 
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
///////////////////////////////////////////////////////////////////////////////


#include <math.h>
#include <string.h>

#ifndef __TRANS_HPP
#include "trans.hpp"
#endif


void Trans::extract_navData( short &st_byte )
{{
  long  ptr_data;             
  short nav_wn;

  //  Get recieve time
  rn.rcv_time=(double)(extract_long(&st_byte,30,F_UNSIGNED))/1000.0;
  if ((rec_type_code==MICRO_Z)||(rec_type_code==LOCUS)) nav_wn=(short)(extract_long(&st_byte,11,F_UNSIGNED));
  ptr_data=extract_long(&st_byte,32,F_UNSIGNED);
  strncpy(rn.sitename,(char*)&ptr_data,4);
  swap_nbytes(rn.sitename,4);

  rn.navx   =0.01*((double) extract_long(&st_byte,31,F_SIGNED));
  rn.navy   =0.01*((double) extract_long(&st_byte,31,F_SIGNED));
  rn.navz   =0.01*((double) extract_long(&st_byte,31,F_SIGNED));
  rn.navxdot=(float)(0.01*((double) extract_long(&st_byte,21,F_SIGNED)));
  rn.navydot=(float)(0.01*((double) extract_long(&st_byte,21,F_SIGNED)));
  rn.navzdot=(float)(0.01*((double) extract_long(&st_byte,21,F_SIGNED)));

  if (fa.data_type == ZTINYTYPE)
    rn.navt=1.0e-6*bLITE001*((double) extract_long(&st_byte,22,F_SIGNED));
  else
    rn.navt=1.0e-5*bLITE001*((double) extract_long(&st_byte,18,F_SIGNED));

  rn.navtdot=0;
  rn.pdop=(unsigned short) (extract_long(&st_byte,9,F_UNSIGNED)*100);

}}

short Trans::ZtinyComp_cword()
{{
  unsigned short vchan,mbens;
  short          st_byte=1;

  // com.cnbyte -> bf.ibf
  vchan= (unsigned short) extract_long(&st_byte,4,F_UNSIGNED);
  mbens = (unsigned short) extract_long(&st_byte,6,F_UNSIGNED);
  if (rec_type_code!=LOCUS)
  { 
  	if (mbens % 3) 
  		return(ERROR_MEASDATA) ;
  	else 
  		mbens/=3;
  }
  if (mbens>vchan) return(ERROR_DATALENGTH);

  if (rec_type_code==LOCUS)
  	rdd.cword=10+mbens*185+249;
  else
  	rdd.cword=10+mbens*393+249;
  if ((rec_type_code==MICRO_Z)||(rec_type_code==LOCUS)) rdd.cword+=11; // nav_wn
  if ( rdd.cword%16 ) rdd.cword =rdd.cword-rdd.cword%16+16;
  rdd.cword=rdd.cword/16;
  rdd.cword +=1; // +1 checksum

  return(TRANS_OK);
}}


short Trans::ReadZTINYTYPE()
{{
  short          j,i;
  short					 error=TRANS_OK;
  long           sig;
  short          st_byte=1;
  unsigned short vchan,mbens;
  unsigned char  chn;
  unsigned long  range2h[2];
  double         *range,dbl;

	// com.cnbyte ->bf.ibf
	error=IndentBuf_CheckingChecksum16(); 	
	if (error!=TRANS_OK) return(error);

  memmove(&bf.ibf,&bf.ibf[1],com.cnbyte-1);
  // rawnaw  ------------------------------------------------------
  vchan=(unsigned short) extract_long(&st_byte,4,F_UNSIGNED);
  mbens=(unsigned short) extract_long(&st_byte,6,F_UNSIGNED);
  if (rec_type_code==LOCUS) 
  	rn.num_sats=(char) mbens;
  else
  	rn.num_sats=(char) (mbens/3);
  // If it is a 3DF receiver, multiply vchan by 3 for get vchan = 24
  if (rec_type_code == DF3) vchan *= 3;
  // rawdata ------------------------------------------------------
  for (i=0;i<rh.num_obs_types;i++)
    for (j=0;j<rn.num_sats;j++)
    { rd[j].svprn = (unsigned char)extract_long(&st_byte,5,F_UNSIGNED);
			if (rec_type_code != LOCUS)
				rd[j].svprn++; // get number sat.-1 
	    chn =         (unsigned char) (extract_long(&st_byte,6,F_UNSIGNED)-1);
      rd[j].chnind= (unsigned char) ((chn % vchan)+1);
      rd[j].obs[i].polarity_known=0;
      rd[j].obs[i].warning=(char)extract_long(&st_byte,8,F_UNSIGNED);
      rd[j].obs[i].goodbad=(rd[j].obs[i].warning & 0x03)+22;
      if (i==0)
      { range2h[1]=extract_long(&st_byte,32,F_UNSIGNED);
				range2h[0]=extract_long(&st_byte,32,F_UNSIGNED);
				range=(double *)range2h;
				if ((*range>1.0)||(*range<0))	return(ERROR_MEASDATA); // for very bad value only !!!
				rd[j].obs[i].rawrange=*range;
				rd[j].obs[i].doppler=10*extract_long(&st_byte,25,F_SIGNED);
				if (!(rd[j].obs[i].warning & 0x08))
				{ rd[j].obs[1].doppler=rd[j].obs[0].doppler;
	  			rd[j].obs[2].doppler=(long)(rd[j].obs[0].doppler*(120.0/154.0));
				}
				else
				{ rd[j].obs[1].doppler=0;
		  	  rd[j].obs[2].doppler=0;
				}
      }
      else
      { dbl=((double) extract_long(&st_byte,21,F_SIGNED))*1.0E-12;
				if (!(rd[j].obs[i].warning & 0x08))
					  rd[j].obs[i].rawrange=rd[j].obs[0].rawrange + dbl;
				else rd[j].obs[i].rawrange=0.0;
      }

      rd[j].obs[i].ireg=(unsigned char)extract_long(&st_byte,8,F_UNSIGNED);

      if (i == 1)
      { dbl=((double) extract_long(&st_byte,14,F_SIGNED))/1000.0;
				if (!(rd[j].obs[i].warning & 0x08))
	  		  rd[j].obs[i].carphase=rd[j].obs[0].carphase+dbl;
				else
	  			rd[j].obs[i].carphase=0.0;
      }
      else
      { sig=extract_long(&st_byte,1,F_UNSIGNED);
				rd[j].obs[i].carphase=(double) extract_long(&st_byte,28,F_UNSIGNED);
				rd[j].obs[i].carphase+= ((double) extract_long(&st_byte,11,F_UNSIGNED))/2000.0;
				if (sig) rd[j].obs[i].carphase *= -1.0;
      }
      rd[j].obs[i].qa_phase =  (char)extract_long(&st_byte,7, F_UNSIGNED);
      rd[j].obs[i].smth_corr=  (float)(extract_long(&st_byte,14,  F_SIGNED)/100.0);
      rd[j].obs[i].smth_count= (unsigned short)extract_long(&st_byte,8, F_UNSIGNED);
    }

  // rawnaw  ------------------------------------------------------

  extract_navData(st_byte);

  // Azimuth and Elevation
  for (j=0;j<rn.num_sats;j++)
  { rd[j].elevation=0;
    rd[j].azimuth=0;
  }
  compute_azel(); // compute elevation and azimuth

  // Write in B-file
  oneEpoch_forBfile();
  
  return(TRANS_OK);
}}

void Trans::PtinyComp_cword()
{{
  unsigned short mbens;
  short          st_byte=1;
                   
  // com.cnbyte -> bf.ibf
  mbens = (unsigned short) extract_long(&st_byte,4,F_UNSIGNED);
  mbens = (unsigned short) extract_long(&st_byte,6,F_UNSIGNED);

  if (rec_type_code == Z_XII)
    rdd.cword=10+(182*mbens)+249;
  else
    rdd.cword=10+(149*mbens)+245;
  if ( rdd.cword%16 ) rdd.cword =rdd.cword-rdd.cword%16+16;
  rdd.cword=rdd.cword/16;
  rdd.cword +=1; // +1 checksum
}}


short Trans::ReadPTINYTYPE()
{{
  short          j,i,ii;
	short					 error=TRANS_OK;	
  long           sig;
  short          st_byte=1;
  short          vchan,mbens;
  unsigned char  chn;
  char           svprn,slot,base_chnind;
  long           temp;

  // com.cnbyte -> bf.ibf
	error=IndentBuf_CheckingChecksum16();
	if (error!=TRANS_OK) return(error);

  memmove(&bf.ibf,&bf.ibf[1],com.cnbyte-1);
  st_byte=1;
  for (j=0;j<12;j++) rd[j].svprn=0;

  // rawnaw  ------------------------------------------------------
  vchan=(unsigned short) extract_long(&st_byte,4,F_UNSIGNED);
  mbens=(unsigned short) extract_long(&st_byte,6,F_UNSIGNED);
  // mbens = amount measurment L1 (obs[0]) + L1P + L2P !
  // If it is a 3DF receiver, multiply vchan by 3 for get vchan = 24
  if (rec_type_code == DF3) vchan *= 3;
  // Set num SVS to zero and increment when new SV found.
  rn.num_sats = 0;

  // rawdata ------------------------------------------------------
  for (j=0;j<12;j++)    // init warning ( not measur.: 7)
    for (i=0;i<rh.num_obs_types;i++)
      rd[j].obs[i].warning=7;
  for(j=ii=0;ii<mbens;ii++,j++)
  { svprn=(char) extract_long(&st_byte,5,F_UNSIGNED)+1;
    chn  =(char) extract_long(&st_byte,6,F_UNSIGNED)-1;
    // "vchan" indicates the number of base channels contained within
    //         receiver: i.e., a receiver with 6 C/A L1, 6 Pcode L1 and
    //         6 Pcode L2 will have vchan = 6.
    // "base_chnind" determines the channel on which the C/A L1 would
    //               exists.
    slot       = (char)(chn/vchan); // L1C/A, L1P or L2P
    base_chnind= (chn % vchan)+1;

    // NOTE: L2P-code rd always computed to be in slot 2 but should
    //       be in slot 1 when L1P not present.
    if ((slot==2)&&(rh.capability!=L1CP_L2P)) slot = 1;
    if (slot)
    { // Determine if other rd for SV has been unpacked and if not
      //  allocate a new one: The base_chnind test is done for channel
      //  testing (i.e, one SV locked to 12 channels).
      temp=rn.num_sats;   // Temp was set up to override a BORLAND BUG
      for (j=0;j<temp;j++)
				if ((rd[j].svprn==svprn)&&(rd[j].chnind==base_chnind))
				 	break;  // Found a match.
      if (j>=temp)  // No match found: allocate a new slot.
				rn.num_sats++;
    }
    else // for SCA 1I00 ( L1 only !!! )
      rn.num_sats++;

    rd[j].svprn =svprn;
    rd[j].chnind=base_chnind;

    rd[j].obs[slot].goodbad=(unsigned char)extract_long(&st_byte,2,F_UNSIGNED)+22;
    rd[j].obs[slot].rawrange =
      ((double) extract_long(&st_byte,31,F_UNSIGNED))*1.0E-10;
    rd[j].obs[slot].doppler=10*extract_long(&st_byte,25,F_SIGNED);
    rd[j].obs[slot].ireg=(unsigned char)extract_long(&st_byte,7,F_UNSIGNED)*2;

    sig = extract_long(&st_byte,1,F_UNSIGNED);
    rd[j].obs[slot].carphase = (double)(extract_long(&st_byte,28,F_UNSIGNED));
    rd[j].obs[slot].carphase +=
		    ((double)(extract_long(&st_byte,11,F_UNSIGNED)))/2000.0;
    if (sig) rd[j].obs[slot].carphase*=-1.0;

    rd[j].obs[slot].qa_phase=(char)extract_long(&st_byte,7,F_UNSIGNED);
    rd[j].obs[slot].warning=(unsigned char)extract_long(&st_byte,3,F_UNSIGNED);
    rd[j].obs[slot].polarity_known=5;
    if (rd[j].obs[slot].warning>=4)
      rd[j].obs[slot].polarity_known=0;

    rd[j].obs[slot].warning |=
		    (unsigned char) extract_long(&st_byte,1,F_UNSIGNED) << 7;
    rd[j].obs[slot].smth_corr =(float)(extract_long(&st_byte,14,F_SIGNED)/ 100.0);
    rd[j].obs[slot].smth_count=
		    (unsigned short)extract_long(&st_byte,8,F_UNSIGNED);
  }

  extract_navData(st_byte);

  // Azimuth and Elevation
  for (j=0;j<rn.num_sats;j++)
  { rd[j].elevation=0;
    rd[j].azimuth=0;
  }
  compute_azel(); // compute elevation and azimuth

  for (j=0;j<rn.num_sats;j++)
  { for (i=0;i<rh.num_obs_types;i++)
    { if (rd[j].obs[i].warning==7)
      { rd[j].obs[i].rawrange=0.0;
				rd[j].obs[i].smth_corr=0.0;
				rd[j].obs[i].smth_count=0;
				rd[j].obs[i].polarity_known=0;
				rd[j].obs[i].goodbad=0;
				rd[j].obs[i].ireg=0;
				rd[j].obs[i].qa_phase=0;
				rd[j].obs[i].doppler=0;
				rd[j].obs[i].carphase=0.0;
      }
    }
  }

  // Write in B-file
  oneEpoch_forBfile();

  return(error);
}}


void Trans::A5benComp_cword()
{{
  unsigned short mbens;
  short          st_byte=1;

  // com.cnbyte -> bf.ibf
  mbens=(unsigned short) extract_long(&st_byte,4,F_UNSIGNED);

  rdd.cword=(fa.data_type==TINYTINYTYPE) ? 138 : 182;
  rdd.cword=4+(rdd.cword*mbens)+271;
  if ( rdd.cword%16 ) rdd.cword=rdd.cword-rdd.cword%16+16;
  rdd.cword=rdd.cword/16;
  rdd.cword +=1; // +1 checksum
}}


short Trans::ReadA5BENstruct()
{{
  short          j,svprn;
  short					 error=TRANS_OK;
  short          st_byte=1;
  double	   		 lost_lock_ctr=0.0,ftxmt;
  double		 		 approx,dummy;
  double         intdoppler,del_intdop;
  long           N;
  long           sig;

  // com.cnbyte -> bf.ibf
	error=IndentBuf_CheckingChecksum16();
	if (error!=TRANS_OK) return(error);
  memmove(&bf.ibf,&bf.ibf[1],com.cnbyte-1);

  st_byte=1;
  for (j=0;j<12;j++) rd[j].svprn=0;

  // rawnaw  ------------------------------------------------------
  rn.num_sats = (char)extract_long(&st_byte, 4, F_UNSIGNED);

  // rawdata ------------------------------------------------------
  for (j=0;j<rn.num_sats;j++)
  {
    if (fa.data_type == TINYTYPE)
      lost_lock_ctr=500.0*(double)(extract_long(&st_byte,15,F_UNSIGNED));

    rd[j].svprn=(unsigned char)extract_long(&st_byte,5,F_UNSIGNED)+1;

    rd[j].chnind=0;
    if (fa.data_type == TINYTYPE)
      rd[j].chnind=(unsigned char)extract_long(&st_byte,4,F_UNSIGNED);

    rd[j].obs[0].goodbad=(unsigned char)extract_long(&st_byte,2,F_UNSIGNED)+22;

    rd[j].obs[0].rawrange=(double)(extract_long(&st_byte,24,F_SIGNED))*1E-8;

    rd[j].obs[0].doppler=0;
    if (fa.data_type == TINYTYPE)
      rd[j].obs[0].doppler=10*extract_long(&st_byte,25,F_SIGNED);

    rd[j].obs[0].ireg=(unsigned char)(2*extract_long(&st_byte,7,F_UNSIGNED));
    rd[j].obs[1].ireg=(unsigned char)extract_long(&st_byte,7,F_UNSIGNED);
    rd[j].obs[2].ireg=0;

    sig=extract_long(&st_byte,1,F_UNSIGNED);
    intdoppler=(double) (extract_long(&st_byte,28,F_UNSIGNED));
    intdoppler += 0.01*(double)(extract_long(&st_byte,7,F_UNSIGNED));
    if (sig) intdoppler=-intdoppler;

    rd[j].obs[0].carphase=0.0005*
			  (double)(extract_long(&st_byte,11,F_UNSIGNED));
    dummy=(double)(extract_long(&st_byte,11,F_UNSIGNED));
    rd[j].obs[1].carphase=0.0005*
			  (double)(extract_long(&st_byte,11,F_UNSIGNED));
    dummy=(float)(extract_long(&st_byte,21,F_SIGNED));

    rd[j].obs[0].warning=(unsigned char)extract_long(&st_byte,3,F_UNSIGNED);
    rd[j].obs[0].polarity_known=5;
    if (rd[j].obs[0].warning >= 4) rd[j].obs[0].polarity_known=0;

    //
    //  Convert to new format: txmt to be converted later.
    //
    svprn = rd[j].svprn;
    rd[j].obs[0].smth_corr=rd[j].obs[1].smth_corr=rd[j].obs[2].smth_corr=0.0;
    rd[j].obs[0].smth_count=rd[j].obs[1].smth_count=rd[j].obs[2].smth_count=0;

    del_intdop=(intdoppler-cd.lastintdop[svprn-1]);
    cd.lastintdop[svprn-1]=intdoppler;
    approx = del_intdop-(rd[j].obs[0].carphase-cd.carrier[svprn-1][0]);
    N = (approx > 0.0) ? (long)(approx+0.5) : (long)(approx-0.5);
    rd[j].obs[0].qa_phase=(char)(fabs(fmod(approx,1.0))*100.0);
    cd.carrier[svprn-1][0]=N+rd[j].obs[0].carphase;
    rd[j].obs[0].carphase=cd.carrier[svprn-1][0];

    del_intdop*=(2.0*bFREQL2/bFREQL1);
    approx=del_intdop-(rd[j].obs[1].carphase-cd.carrier[svprn-1][1]);
    N = (approx > 0.0) ? (long)(approx+0.5) : (long)(approx-0.5);
    rd[j].obs[1].qa_phase=(char)(fabs(fmod(approx,1.0))*100.0);
    rd[j].obs[2].qa_phase=0;
    cd.carrier[svprn-1][1]=N+rd[j].obs[1].carphase;
    rd[j].obs[1].carphase=cd.carrier[svprn-1][1];
    rd[j].obs[2].carphase=0.0;

    if (lost_lock_ctr < cd.con_lloss_ctr[svprn-1][0])
      rd[j].obs[0].warning |= 0x80;
    cd.con_lloss_ctr[svprn-1][0]=lost_lock_ctr;

    rd[j].obs[1].polarity_known= rd[j].obs[2].polarity_known= 0;
    rd[j].obs[1].goodbad       = rd[j].obs[2].goodbad       = 0;
    rd[j].obs[1].rawrange      = rd[j].obs[2].rawrange      = 0.0;
    rd[j].obs[1].doppler       = rd[j].obs[2].doppler       = 0;
    rd[j].obs[1].warning       = rd[j].obs[2].warning       = 0;
  }

  extract_navData(st_byte);

  ftxmt=0.01*(double)(extract_long(&st_byte,26,F_UNSIGNED));

  dummy=((long)(rn.rcv_time*100.0+0.5))/100.0;
  for (j=0;j<12;j++)
  { if ((rd[j].svprn==99)||(rd[j].svprn==0)) continue;
    rd[j].obs[0].rawrange = rd[j].obs[0].rawrange + ftxmt;
    rd[j].obs[0].rawrange = dummy - rd[j].obs[0].rawrange;
  }

  // Azimuth and Elevation
  for (j=0;j<rn.num_sats;j++)
  { rd[j].elevation=0;
    rd[j].azimuth=0;
  }
  compute_azel(); // compute elevation and azimuth

  // Write in B-file
  oneEpoch_forBfile();  

  return(error);
}}


void Trans::TinycodComp_cword()
{{
  unsigned short mbens;
  short      	 st_byte=1;

  // com.cnbyte -> bf.ibf
  mbens=(unsigned short)extract_long(&st_byte,5,F_UNSIGNED);

  rdd.cword = 202;
  rdd.cword = 5+(rdd.cword*mbens)+245;
  if ( rdd.cword%16 ) rdd.cword=rdd.cword-rdd.cword%16+16;
  rdd.cword=rdd.cword/16;
  rdd.cword +=1; // +1 checksum
}}


short Trans::ReadTINYCODELESS()
{{
  short j;
  short error=TRANS_OK;
  short st_byte=1;
  long     		 sig;

  // com.cnbyte -> bf.ibf
	error=IndentBuf_CheckingChecksum16();
	if (error!=TRANS_OK) return(error);

  memmove(&bf.ibf,&bf.ibf[1],com.cnbyte-1);
  st_byte=1;

  // rawnaw  ------------------------------------------------------
  rn.num_sats = (char)extract_long(&st_byte, 5, F_UNSIGNED);

  // rawdata  ------------------------------------------------------
  for (j=0;j<rn.num_sats;j++)
  { rd[j].svprn=(unsigned char)(extract_long(&st_byte,5,F_UNSIGNED)+1);
    rd[j].chnind=(unsigned char)extract_long(&st_byte,5,F_UNSIGNED);
    rd[j].obs[0].goodbad=(unsigned char)extract_long(&st_byte,2,F_UNSIGNED)+22;

    rd[j].obs[0].rawrange=
		 ((double)extract_long(&st_byte,31,F_UNSIGNED))*1.0E-10;

    rd[j].obs[0].doppler=10*extract_long(&st_byte,25,F_SIGNED);

    rd[j].obs[0].ireg=(unsigned char)(2*extract_long(&st_byte,7,F_UNSIGNED));
    rd[j].obs[1].ireg=(unsigned char)extract_long(&st_byte,7,F_UNSIGNED);
    rd[j].obs[2].ireg=0;

    sig=extract_long(&st_byte,1,F_UNSIGNED);
    rd[j].obs[0].carphase=(double)(extract_long(&st_byte,28,F_UNSIGNED));
    rd[j].obs[0].carphase +=
	  ((double)(extract_long(&st_byte,11,F_UNSIGNED)))/2000.0;
    if (sig) rd[j].obs[0].carphase *=-1.0;

    rd[j].obs[0].qa_phase=(char)extract_long(&st_byte,7,F_UNSIGNED);

    sig=extract_long(&st_byte,1,F_UNSIGNED);
    rd[j].obs[1].carphase=(double)(extract_long(&st_byte,28,F_UNSIGNED));
    rd[j].obs[1].carphase +=
	  ((double)(extract_long(&st_byte,11,F_UNSIGNED)))/2000.0;
    if (sig) rd[j].obs[1].carphase *=-1.0;
    rd[j].obs[2].carphase=0.0;

    rd[j].obs[1].qa_phase=(char)extract_long(&st_byte,7,F_UNSIGNED);
    rd[j].obs[2].qa_phase=0;
    rd[j].obs[0].warning=(unsigned char)extract_long(&st_byte,3,F_UNSIGNED);
    rd[j].obs[0].polarity_known=5;
    if ((rd[j].obs[0].warning & (0X7F)) >= 4)
      rd[j].obs[0].polarity_known=0;

    rd[j].obs[0].warning |=
	  (unsigned char)extract_long(&st_byte,1,F_UNSIGNED) << 7;
    sig=extract_long(&st_byte,1,F_UNSIGNED);
    rd[j].obs[0].smth_corr=(float)extract_long(&st_byte,13,F_UNSIGNED);
    if (sig) rd[j].obs[0].smth_corr*=-1.0f;
    rd[j].obs[0].smth_corr /=100.0f;
    rd[j].obs[0].smth_count=
	  (unsigned short)extract_long(&st_byte,8,F_UNSIGNED);

    rd[j].obs[1].polarity_known= rd[j].obs[2].polarity_known= 0;
    rd[j].obs[1].goodbad       = rd[j].obs[2].goodbad       = 0;
    rd[j].obs[1].rawrange      = rd[j].obs[2].rawrange      = 0.0;
    rd[j].obs[1].doppler       = rd[j].obs[2].doppler       = 0;
    rd[j].obs[1].warning       = rd[j].obs[2].warning       = 0;
    rd[j].obs[1].smth_corr     = rd[j].obs[2].smth_corr     = 0;
    rd[j].obs[1].smth_count    = rd[j].obs[2].smth_count    = 0;
  }

  extract_navData(st_byte);

  // Azimuth and Elevation
  for (j=0;j<rn.num_sats;j++)
  { rd[j].elevation=0;
    rd[j].azimuth=0;
  }
  compute_azel(); // compute elevation and azimuth

  // Write in B-file
  oneEpoch_forBfile();

  return(error);
}}


void Trans::BenComp_cword()
{{
  short      	 st_byte=1;
  unsigned short mbens;

  // com.cnbyte -> bf.ibf
  mbens=(unsigned short) extract_long(&st_byte,5,F_UNSIGNED);

  rdd.cword=(fa.data_type==TINYTINYTYPE) ? 138 : 186;
  rdd.cword=5+(rdd.cword*mbens)+245;
  if ( rdd.cword%16 ) rdd.cword=rdd.cword-rdd.cword%16+16;
  rdd.cword=rdd.cword/16;
  rdd.cword +=1; // +1 checksum
}}

short Trans::ReadBENstruct()
{{
  short  j,svprn;
  short	 error=TRANS_OK;
  short  st_byte=1;
  double lost_lock_ctr=0.0;
  double approx;
  double intdoppler,del_intdop;
  long   N;
  long   sig;

  // com.cnbyte -> bf.ibf
	error=IndentBuf_CheckingChecksum16();
	if (error!=TRANS_OK) return(error);

  memmove(&bf.ibf,&bf.ibf[1],com.cnbyte-1);
  st_byte=1;

  // rawnaw  ------------------------------------------------------
  rn.num_sats = (char)extract_long(&st_byte, 5, F_UNSIGNED);

  // rawdata  ------------------------------------------------------
  for (j=0;j<rn.num_sats;j++)
  { if (fa.data_type == TINYTYPE)
      lost_lock_ctr=500.0*(double)(extract_long(&st_byte,15,F_UNSIGNED));

    rd[j].svprn=(unsigned char)extract_long(&st_byte,5,F_UNSIGNED)+1;

    rd[j].chnind=0;
    if (fa.data_type == TINYTYPE)
      rd[j].chnind=(unsigned char)extract_long(&st_byte,5,F_UNSIGNED);

    rd[j].obs[0].goodbad=(unsigned char)extract_long(&st_byte,2,F_UNSIGNED)+22;

    rd[j].obs[0].rawrange =
		 (double)(extract_long(&st_byte,27,F_UNSIGNED))*1E-9;

    rd[j].obs[0].doppler=0;
    if (fa.data_type == TINYTYPE)
      rd[j].obs[0].doppler=10*extract_long(&st_byte,25,F_SIGNED);

    rd[j].obs[0].ireg=(unsigned char)(2*extract_long(&st_byte,7,F_UNSIGNED));
    rd[j].obs[1].ireg=(unsigned char)extract_long(&st_byte,7,F_UNSIGNED);
    rd[j].obs[2].ireg=0;

    sig=extract_long(&st_byte,1,F_UNSIGNED);
    intdoppler=(double)(extract_long(&st_byte,28,F_UNSIGNED));
    intdoppler += 0.01*(double)(extract_long(&st_byte,7,F_UNSIGNED));
    if (sig) intdoppler=-intdoppler;

    rd[j].obs[0].carphase=0.0005*
			  (double)(extract_long(&st_byte,11,F_UNSIGNED));
    approx=(double)(extract_long(&st_byte,11,F_UNSIGNED));
    rd[j].obs[1].carphase=0.0005*
			  (double)(extract_long(&st_byte,11,F_UNSIGNED));
    approx=(float)(extract_long(&st_byte,21,F_SIGNED));

    rd[j].obs[0].warning=(unsigned char)extract_long(&st_byte,3,F_UNSIGNED);
    rd[j].obs[0].polarity_known=5;
    if (rd[j].obs[0].warning >= 4) rd[j].obs[0].polarity_known=0;

    //
    //  Convert to new format: txmt to be converted later.
    //
    svprn=rd[j].svprn;
    rd[j].obs[0].smth_corr=rd[j].obs[1].smth_corr=rd[j].obs[2].smth_corr=0.0;
    rd[j].obs[0].smth_count=rd[j].obs[1].smth_count=rd[j].obs[2].smth_count=0;

    del_intdop=(intdoppler-cd.lastintdop[svprn-1]);
    cd.lastintdop[svprn-1]=intdoppler;
    approx=del_intdop-(rd[j].obs[0].carphase-cd.carrier[svprn-1][0]);
    N = (approx > 0.0) ? (long)(approx+0.5) : (long)(approx-0.5);
    rd[j].obs[0].qa_phase=(char)(fabs(fmod(approx,1.0))*100.0);
    cd.carrier[svprn-1][0]=N+rd[j].obs[0].carphase;
    rd[j].obs[0].carphase=cd.carrier[svprn-1][0];

    del_intdop *=(2.0*bFREQL2/bFREQL1);
    approx=del_intdop-(rd[j].obs[1].carphase-cd.carrier[svprn-1][1]);
    N = (approx > 0.0) ? (long)(approx+0.5) : (long)(approx-0.5);
    rd[j].obs[1].qa_phase=(char)(fabs(fmod(approx,1.0))*100.0);
    rd[j].obs[2].qa_phase=0;
    cd.carrier[svprn-1][1]=N+rd[j].obs[1].carphase;
    rd[j].obs[1].carphase=cd.carrier[svprn-1][1];
    rd[j].obs[2].carphase=0.0;

    if (lost_lock_ctr < cd.con_lloss_ctr[svprn-1][0])
      rd[j].obs[0].warning |= 0x80;
    cd.con_lloss_ctr[svprn-1][0]=lost_lock_ctr;

    rd[j].obs[1].polarity_known= rd[j].obs[2].polarity_known= 0;
    rd[j].obs[1].goodbad       = rd[j].obs[2].goodbad       = 0;
    rd[j].obs[1].rawrange      = rd[j].obs[2].rawrange      = 0.0;
    rd[j].obs[1].doppler       = rd[j].obs[2].doppler       = 0;
    rd[j].obs[1].warning       = rd[j].obs[2].warning       = 0;
  }

  extract_navData(st_byte);

  // Azimuth and Elevation
  for (j=0;j<rn.num_sats;j++)
  { rd[j].elevation=0;
    rd[j].azimuth=0;
  }
  compute_azel(); // compute elevation and azimuth

  // Write in B-file
  oneEpoch_forBfile();

  return(TRANS_OK);
}}


