///////////////////////////////////////////////////////////////////////////////	
//    Ashtech receiver tools
//    Copyright (C) 1999  Magellan Coropration/Ashtech Precision Products 
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
///////////////////////////////////////////////////////////////////////////////


#ifndef __TRANS_HPP
#define __TRANS_HPP

#include <stdio.h>

#define HOSE_BUF_SIZE					1024
#define IBF_SIZE	            2350
#define VIBF_SIZE	            4050
#define TBF_SIZE	              60

enum { TRANS_OK = 0,			COLLECTING_DATA, 
			 ERROR_MODEHOSE=14, ERROR_MODEDWL,    ERROR_NUMFILE,     
			 ERROR_PCCARD,      ERROR_TYPEREC,    ERROR_CHECKSUM, 
			 ERROR_PHOTODAT,    ERROR_FAT,        ERROR_DATATYPE,
			 ERROR_DELETE,      ERROR_LENRANG1,   ERROR_LENRANG2,  
			 ERROR_FORMATRANG2, ERROR_MEASDATA,   ERROR_DATALENGTH,  
			 ERROR_CAPABILITY,  ERROR_RCI,        ERROR_CHDIR,       
			 ERROR_FREESPACE,		WARNING_GLONASS,  FATAL_ERROR,  
			 FILLBUF,           NO_FILLMBEN,      NO_FILE,  
			 NO_HOSE,           ERROR_HEALTH,     NEXT };

#define KNOWN_RECEIVERS_NUMBER  19

enum { UNKNOWN,  LM_XII,
       MS_XII,   M_XII,
       D_XII,    P_XII,
       DF3,      RANGER,
       Z_XII,    SUPER_CA,
       LCS_XII,  G_XII,
       VISPA,		 G_XXIV,
			 MICRO_Z,  LOCUS,
			 HP_48,    Z_18,
			 GGG
     };
//  -----------------------------------------------------------------------
//  Geodetic --------------------------------------------------------------
//
//  L-XII     (88) 12  channel { CDPHASE  }          real time
//  M-XII     (89) 12  channel { L1       } memory             , photogram.
//  MD-XII    (90) 12  channel { L2C      } memory , real time
//  MS-XII    (92) 12  channel { L1       } memory , real time
//  DIMENSION (92) 12  channel { L1       } memory
//  P-XII     (92) 12  channel { L1CP_L2P } memory , real time
//  Z-XII     (93) 12  channel { L1CP_L2P } memory , real time
//  SUPER_CA  (94) 12  channel { L1       } memory , real time
//  G-XII     (95) 12  channel { L1       } memory , real time ,
//  G-XXIV    (95) 12  channel { L1       } memory , real time , (+GLONASS)
//  MICRO-Z   (97) 12  channel { L1CP_L2P } memory , real time
//  LOCUS     (98) 12  channel ( L1       } memory , InfraRed 
//  Z18       (98) 12  channel { L1CP_L2P } memory , real time , (+GLONASS)
//  GGG-XII   (96) 12  channel { L1CP_L2P } memory , real time , (+GLONASS)
// 
//  Navigation ------------------------------------------------------------
//
//  RANGER 1  (90) 12  channel { L1       } memory , real time
//  RANGER 2  (91) 12  channel { CDPHASE  } memory , real time
//  3DF       (91) 4x6 channel { L1       } memory , real time
//  DN-XII    (91) 12  channel { L1       }          real time
//  -----------------------------------------------------------------------
//	VISPA			(99)  it is not the receiver.
//  -----------------------------------------------------------------------
// Define data types for dynamic memory
#define MBEN             1
#define PBEN             2
#define SNAV             3
#define TINYTYPE         4
#define TINYTINYTYPE     5
#define EVENT            6
#define TINYCODELESS     7
#define PTINYTYPE        8
#define ZTINYTYPE        9
#define RNAV             10
#define RALM             12
#define ATT_3DF          20
#define TINYRANGER1      21
#define TINYRANGER2      22  
#define TINYRANGER2C     25    // for MicroZ only !!! 
#define TINYDRANGER2     17492 // for MicroZ only !!! 
#define TINYDRANGER2C    17493 // for MicroZ only !!! 
#define TINYRANGER3			 23		 // ring-buffer format	
#define GLONASS_SNAV     55 // new for Glonass (presumably)!
#define EVENT_S					 77	

#define R_MODE2_BTYPES  5

// Define Bit handling for unpacking
#define F_SIGNED        1
#define F_UNSIGNED      2

#define S_BITS_xxxx     3
#define S_BITS          40
#define SEEK_SET        0

#define bLITE001        2.99792458e5
#define bRTD            57.2957795130823229
#define bFREQL1         1.57542e9
#define bFREQL2         1.22760e9
#define bGM84           3.986005e8
#define bOMEGAE84       7.2921151467e-5
#define bPI             3.141592653589793
#define bCSPEED         2.99792458e8
#define bAE84						6378137.0
#define bFLAT84         0.00335281066474

#define MAX_NUM_FILES   5	   // for receivers with generation of name session !!! 
#define ADDR_HEAD_FILES	10L  // addr in image of memory of "Hose" receivers
#define ADDR_NUM_FILES  9L	 // addr number files in image of memory of "Hose" receivers	
#define MAX_LEN_DSTR    500  // for string Barcode in D-file

#define L1CAR           (1540.0*1023000.0)

const double f0_GLN   = 1.6020e+09;
const double step_GLN = 0.5625e+06;

enum { COMHEAD,ANSHEAD,ANSTYPE,DATA };

enum { UNKN,      // 0 Unknown code tracking.
       L1,        // 1 L1 C/A Codephase tracking.
       L2C,       // 2 L1 C/A Codephase and L2 Codeless.
       L1CP_L2P,  // 3 L1 C/A, L1 P-Code, and L2 P-Code
       CDPHASE,   // 4 !!! L1 C/A Codephase (no carrier).
       L1CP,      // 5 L1 C/A and L1 P-Code.
       L1C_L2P    // 6 L1 C/A and L2 P-Code.
     };

//  Receiver Versions
enum { A_5, E_5, A_6, F_6};

enum { GEODETIC,		 RANGER_MODE1,	RANGER_MODE2,
			 RANGER_MODE3, RANGER_MODE4,	PHOTODAT,	
			 ALMANAC,			 IONOSPHERE,		DFILE_HP
		 }; 				 

static unsigned char file_types[9][9] = {"GEODETIC", "RANGER 1", "RANGER 2",
																				 "RANGER 3", "RANGER 4", "PHOTODAT",  
																				 "ALMANAC ", "IONOSPHE", "DFILE HP"																				
																				};
// GEODETIC - ( B- ,E- ,S- ,D- files )
// RANGER1  - ( B- ,E- ,S- ,D- files )
// for MS-XII ,3DF-XXIV ,D-XII + ( N- ,A- files )
// RANGER2  - ( C- ,S- ,D- files )
// RANGER3  - ( B- ,E- ,S- ,D- files )
// RANGER4  - ( B- ,C- ,E- ,S- ,D- files )

static unsigned char bit[8] = { 1, 2, 4, 8, 16, 32, 64, 128 };

struct file_dir
{   
	  unsigned char  file_name[12]; // if [0]=='\0' then dir (name in rawfile_name)
		unsigned char  site[5];
    unsigned char  session;
    unsigned char  rcv_num[4];
    unsigned char  ant_num[4];
    unsigned char  operators[4];
    unsigned char  comment[14];     // width of time -> (long time_width) for VISPA only !!!
    float          ahibef;
    float          ahiaft;
    char           temp_drybef;			// type of record -> (char record_type) for VISPA only !!!
    char           temp_dryaft;			// sample frequency	-> (char freq) for VISPA only !!!
    char           temp_wetbef;
    char           temp_wetaft;
    char           humiditybef;
    char           humidityaft;
    float          presbef;
    float          presaft;
    unsigned char  bytes[12];        // size string
    unsigned char  date[9];          // "mm/dd/yy\0"
    unsigned char  time[9];          // "hh/mm/ss\0"
    unsigned char  file_type[9];     // "GEODETIC","RANGER 1","RANGER 2",
																		 //	"PHOTODAT","RANGER 4","ALMANAC ",
																		 //	"IONOSPHE","DFILE_HP"," UNKNOWN" 
    unsigned char  rawfile_name[13];
};  // 245 bytes

struct file_trans // for write in B-,C-,E-,S-,D-,N-,A- files
{ 
	char  vname_file[13];
	void* pData;
	short DataSize; 
	short fCreate;  // 0 - open file first time
	                // 1 - append in file
	                // 2 - rewrite began file
	long lCurPos;
};

struct buffers
{ unsigned char ibf[IBF_SIZE];
  unsigned char vibf[VIBF_SIZE+1];
  unsigned char tbf[TBF_SIZE];
	unsigned char fbf[HOSE_BUF_SIZE+1];	
};

struct comm_formats
{ unsigned short cnbyte;
  unsigned short srx;
};

struct name_files
{ unsigned char nameB_file[14];
  unsigned char nameE_file[14];
  unsigned char nameS_file[14];
  unsigned char nameD_file[14];
  unsigned char nameN_file[14];
  unsigned char nameA_file[14];
  unsigned char nameP_file[14];
};

struct varVErawfileMZ
{ unsigned char name[13];
  unsigned long bytes; 
};

struct varYrawfileMZ
{ unsigned char  name[13];
  unsigned short bytes; 
};

struct rawfilesMZ // for MicroZ only !!!
{ varVErawfileMZ V_file; // for B-file
  varVErawfileMZ W_file; // for C-file
  varYrawfileMZ  Y_file; // for S-file
  varVErawfileMZ E_file; // E-file
  varVErawfileMZ M_file; // M-file - photo data
  varVErawfileMZ D_file; // D-file   
  unsigned long numbytes_session;
};                             

#pragma pack(1) // begin 1 byte-alignment

struct rec_type                      // for char string
{ unsigned char  receiver_type[3];   // 2 bytes
  unsigned char  channel_option;     // 1
  unsigned char  codeless_option;    // 1
  unsigned char  nav_version[5];     // 2 (4)
  unsigned char  co_processor;       // 1
  unsigned char  receiver_option[4]; // 2
  unsigned char  channel_version[5]; // 2 (4)
  long           data_address;       // 8
  long           data_length;        // 8
  long           photo_address;      // 8
  long           photo_length;       // 8
  long           max_size;           // 8
  unsigned short checksum;           // 2
}; // 62 bytes (66 bytes for Z-12 and later)

struct oper_strct
{ char           mode;
  char           spare1;
  short          wk_beg;
  long           time_beg;
  char           type;
  char           spare2[3];
}; // 12 bytes

struct project_strct
{ char             ses_name;
  char             rcv_num[3];
  char             ant_num[3];
  char             month[2];
  char             day[2];
  char             operat[3];
  char             comment[13];
  char             spare[7];
}; // 34 bytes

struct header_strct
{ long                 segbeg;
  long                 wordcount;   // point 1 wordcount for MicroZ !!!
  char                 segname[4];  // 
  struct oper_strct    oper;        //
  struct project_strct project;     //
  short                segwn;       //
  long                 segtime;     // size 60 bytes
  unsigned short       anthibef;    // copy 4 bytes float
  unsigned short       anthiaft;    // copy 4 bytes float
  char                 drybef;      // point drybef 0 for MicroZ !!!
  char                 dryaft;      //
  char                 wetbef;      //
  char                 wetaft;      //
  char                 humidbef;    //
  char                 humidaft;    //
  short                pressurebef; //
  short                pressureaft; // size 10 bytes
  unsigned short       checksum;
}; // 80 bytes                                                  


//___________________________________________________________________//
// for damp. mem. receiver
struct raw_image_header
{ char rcvr_type[20];
  char version  [20];
  char one_file_mode;
  char res;
  long total_memory;
  long FAT_start;
  long FAT_end;
  long photo_start;
  long photo_end;
  long almanac_start;
  long almanac_end;
  long data_start;
  long data_end;
  char spare[71];
  char image_head_vers;  // 0 first version
			 // 1 rcvr capabilty
}; // 150 bytes

struct ring_header_strct 
{ short					 num_records;
  short					 slot_idx;
  short					 record_size;
  short					 wkstop;
  long					 timestop;
  unsigned short checksum;
};

// __________________________________________________________________//
// ALMANAC data
struct almstrct    // for GPS !
{ float          e;
  long           toe;
  float          i0;
  float          omegadot;
  short          health;
  double         roota;
	double         omega0;
  double         omega;
  double         m0;
  float          af0;
  float          af1;
  short          bulhealth;
  short          wna;
  short          wn;
  long           tow;
  short          config;
}; // 70 bytes

struct malmstrct
{
  unsigned char  almanac_name[12]; // if [0]=='\0' then dir (name in rawfile_name)
  almstrct       alm[32];
};

struct almstrct_g  // for Glonass !
{ short          litera; // 1 - 24
  short          health; // satellite health: 0-bad, 1-good
  float          e;      // eccentricity of the orbit at tla time
  long           Na;     // number of a day within 4-years period [days]
  float          di;     // correction to the mean value of inclination (63 deg) [semicycle]
  float          la;     // longitude of the first ascending node in SGS90 [semicycle]
  float          tla;    // time of the first asc. node passing [s]
  float          w0;     // perigee argument of the orbit at tla time [semicycle]
  float          dT;     // correction to the mean value (43200) of drac. period [s/orbit]
  float          dTt;    // change velocity of the drac. period [s/orbit**2]
  float          tn;     // rough value of the satellite time shift [s]
}; // 40 bytes


// Raw almanac data (for DMA use,special firmware version)
struct ralmstrct
{ unsigned char frame[45][40];
};

// __________________________________________________________________//
// PHOTOGRAMETRY data
struct photo_data
{ char   picname[4];
  double pictime_ms;
}; // 12 bytes


// snav for E-file
//  char           svprn;    for GPS !
struct navstrct
{ short          wn;
  long           tow;
  float          tgd;
  long           aodc;
  long           toc;
  float          af2;
  float          af1;
  float          af0;
  long           aode;
  float          deltan;
  double         m0;
  double         e;
  double         roota;
  long           toe;
  float          cic;
  float          crc;
  float          cis;
  float          crs;
  float          cuc;
  float          cus;
  double         omega0;
  double         omega;
  double         i0;
  float          omegadot;
  float          idot;
  short          accuracy;
  short          health;
  short          fit;
};// 128 bytes

//  char           sys_num;    for Glonass !
struct navstrct_g
{ unsigned short wn_Gln;      // [ week ]  (2)
  unsigned long  tow_Gln;			// [ secs ]  (4)
  unsigned long  day;					// [ days ]  (4)
  unsigned long  frametime;		// [ secs ]  (4)
  double         reftime;			// [ secs ]  (8) // (GLONASS time)
  float          timeoffset;	// [ secs ]  (4)
  float          freqoffset;	// [ - ]     (4)
  double         Gln_UTC; 		// [ secs ]  (8)
  double         pos[3];			// [ m ]     (24)
  double         vel[3];			// [ m/s ]   (24)
  double         acc[3];			// [ m/s/s ] (24)
  double         carrier;			// [ Hz ]    (8)
  char           freq_num;		// []	     (1)
  unsigned char  aode;				// []	     (1)
  unsigned short health;			// []	     (2)
  unsigned short flags;				// []	     (2)
  char           e_version;   // []        (1)
  char           reserved[3]; // []        (3)
}; // 128 bytes

//Each B-file starts with a rawheader struct

//The size of a rawstruct header = 90 bytes

struct rawheader
{ char          version[10];
  unsigned char raw_version;
  char          rcvr_type[10];                      
  char          chan_ver[10];
  char          nav_ver[10];
  short         capability;
  long          wn_start;      // reserved
  char          num_obs_types;
  char          spare[42];
};
//Each epoch starts with a rawnav struct
//The size of rawnav struct = 67 bytes

struct rawnav 
{ char           sitename[4];
  double         rcv_time;
  double         navx;
  double         navy;
  double         navz;
  float          navxdot;
  float          navydot;
  float          navzdot;
  double         navt;
  double         navtdot;
  unsigned short pdop;
  char           num_sats;
};

//The size of struct chan_obs = (31) bytes

struct chan_obs 
{ double         rawrange;			// SV raw range: raw transmit time is this
																//  value subtracted from receive time
																//  (receive time rounded to nearest milisec.
  float          smth_corr;			// Smoothing correction for ranges (meters)
  unsigned short smth_count;		// Number of data points in smoothing.
  char           polarity_known;// Tracking status: 0 to 3 not usable for
																//  carrier phase.
  unsigned char  warning;       // Warning flag (BIT flags):  0 ==> All O.K.
				//  Bit 1 ==> Sv txmt approaching 1ms offset
				//  Bit 2 ==> Sv txmt approaching 1ms offset
				//             but different direction from 1
				//  Bit 3 ==> Carrier phase questionalbe.
				//  Bit 8 ==> Lost_lock_counter reset.
  unsigned char  goodbad;				// Another health indicator:
				//  22 ==> Code and carrier measured.
				//  23 ==> Same as 22 but additionally, nav
				//          message obtained but measurement
				//          was not used in position comp.
				//  24 ==> Same as 23 but codephase was used
				//          in position computation.
  unsigned char  ireg;          // Signal to noise.
  char           qa_phase;      // QA phase check (0.001 cycles).
  long           doppler;       // SV raw doppler.
  double         carphase;      // Full carrier phase (cycles).
};

//Each epoch has a rawdata struct per SV

//The size of L1 rawdata struct = (35 * nav.num_sats) bytes
//The size of L2C rawdata struct = (66 * nav.num_sats) bytes
//The size of L2P rawdata struct = (97 * nav.num_sats) bytes

struct rawdata 
{ unsigned char svprn;
  unsigned char elevation;
  unsigned char azimuth;
  unsigned char chnind;
  struct chan_obs obs[3]; // This Variable is indexed by 3 (currently)
			   //  to support receivers measuring C/A code
			   //  L1 independently of P-Code L1.  Thus the
			   //  indexing is as follows:
			   //    A) For L1 only receivers, L1 is in
			   //       slot 0.
			   //    B) For L2 codeless receivers, L1 is in
			   //       slot 0 and L2 is in slot 1.
			   //    C) For our P-CODE receiver L1 C/A is
			   //       in slot 0, L1 P-Code is in slot 1,
			   //       and L2 P-Code is in slot 2.
};


// __________________________________________________________________//
// SNAV structure for one satelite
//   for GPS !
struct snavstrct
{ short          wn;
  long           tow;
  float          tgd;
  long           aodc;
  long           toc;
  float          af2;
  float          af1;
  float          af0;
  long           aode;
  float          deltan;
  double         m0;
  double         e;
  double         roota;
  long           toe;
  float          cic;
  float          crc;
  float          cis;
  float          crs;
  float          cuc;
  float          cus;
  double         omega0;
  double         omega;
  double         i0;
  float          omegadot;
  float          dot;
  short          accuracy;
  short          health;
  short          fit;
  char           prnnum;
  char           res;
  unsigned short checksum;
}; // 132 bytes

//   for Glonass !
struct snavstrct_g
{ long           tk;       //  (4)
  short          dn;       //  (2)
  long           tb;       //  (4)
  float          gn;       //  (4)
  float          tn;       //  (4)
  double         r[3];     // (24)
  float          rt[3];    // (12)
  float          rtt[3];   // (12)
  double         tc;       //  (8)
  char           en;       //  (1)
  char           p;        //  (1)
  char           bn;       //  (1)
  char           freq_num; //  (1)
  short          sys_num;  //  (2)
  unsigned short checksum; //  (2)
}; // 82 bytes


// RNAV (Raw ephemeris data,for DMA use,special firmware version)
struct rnavstrct
{ short            prnnum;
  unsigned char  frame1[40];
  unsigned char  frame2[40];
  unsigned char  frame3[40];
}; // 122 bytes

// __________________________________________________________________//
// ATT_3DF data (3DF receiver only)
struct sattdstrct
{ double         head;
  double         pitch;
  double         roll;
  double         brms;
  double         mrms;
  long           tow;
  char           reset;
  char           spare;
}; // 48 bytes  ( +2 byte checksum !!! )

// __________________________________________________________________//
// MBEN data (M-XII receiver only)
struct mbenstrct
{ unsigned char  data_type;
  unsigned char  data_count;
  unsigned char  svprn;
  unsigned char  chnind;
  unsigned long  lost_lock_ctr;
  unsigned char  polarity_known;
  unsigned char  goodbad;
  unsigned char  warning;
  unsigned char  ireg;
  double         codetxmt;
  long           doppler;
  double         intdoppler;
  unsigned short carphase1;
  unsigned short carphase2;
  unsigned short elevation;
  unsigned short azimuth;
}; // 40 bytes

// PBEN data (M-XII receiver only)
struct pbenstrct
{ long           rcv_time;
  char           sitename[4];
  double         navx;
  double         navy;
  double         navz;
  float          navt;
  float          navxdot;
  float          navydot;
  float          navzdot;
  float          navtdot;
  unsigned short pdop;
}; // 54 bytes
#pragma pack() // end 1 byte-alignment

struct analyse_data
{ short          prec_day;     // a day of record in the previous file of a receiver
  short          var_analyse;  // variants of analysis given in the memory
  unsigned short type_analyse; // modes of operations with data in memory
  short          data_type;    // a type given in memory of a receiver

  char           event_type;
  char           string_length_event;
  unsigned short num_photo;    // size photo
  unsigned short checksum_ranger2;
  char           ranger_diff_mode;
  short          num_pages_ralm;
  short          block_len_ranger2;
  unsigned char  cl_write_rawdata; // the attribute of availability of a filled in structure
				    // rawdata in is given. A epoch for record in B-file at
				    // reception of structure rawnav
  unsigned char  findex;          // a attribute of installation of indexes of files
  unsigned short nfile;           // the number of a file
  // unsigned char nav_ver_code; // a code of navigating version
  unsigned char  rcvr_version;    // a code of the unified version of a receiver
  char           curr_file_type;  // a code of a common type of a file
  unsigned char  change_wn;
  char           satnum;
};

struct read_data
{ unsigned long  addr;					// address of data
  unsigned long	 addr_begBEN;	
	char					 mode_offset_addr;	
	unsigned short cword;					// quantity of read - out words
  unsigned short curr_cbytes;		// current count of bytes
	unsigned short index_workbuf; // last index of bytes in reading block
	unsigned short fill_byte;
  unsigned long  number_error;
	unsigned short snav_number;
	short          ben_number;
	short					 curr_ben_number;	
  unsigned short fill_buf_block;
	unsigned short block_no;
  unsigned short sum_block;
  double         percent;   // percent of the read out information
};

struct f_writefile
{ short		  f_writeBfile; // if>0 ok for write file !!!
  short		  f_writeheaderCBfile;
  short     f_writeSfile;
  short		  f_writeEfile;
  short		  f_writeDfile;
  short		  f_writeNfile;
  short		  f_writeAfile;
  short     f_writePfile;	
	short			f_writeDumpMemoryfile;
	short			f_writeVispaRawfile;
};


struct f_createfile
{ short		  f_createBfile; 
  short		  f_createheaderCBfile;
  short			f_createCfile;
  short     f_createSfile;
  short		  f_createEfile;
  short		  f_createDfile;
  short		  f_createNfile;
  short		  f_createAfile;
  short     f_createPfile;	
	short			f_createDumpMemoryfile;
	short			f_createVispaRawfile;
};

struct computed_data
{ double    lastintdop[32];  // previous mean. dopler. the interval
  double    carrier[32][3];  // the previous significance of a phase for given sat.
  double    con_lloss_ctr[32][3]; // determinants for warning
  double    azel_cp [4]; // a buffer for navx, navy, navz and navt
  double    txmt[12];
  double    ep[4];
  short     capability;
  char      num_obs_types;
};

class Trans
{
  public:
    Trans();
    ~Trans(){};

    void   ControlRXCount();
    short  Get_curr_file_type() { return((short)fa.curr_file_type); }
		void   InitSNV_parameters();
    void   InitDownloadData(); 
    void   FormVarNamesFiles();
    short  FormSfile();
    void   setCountPercent(double _percent) { rdd.percent=_percent; }
    void   write_headerCfile();

		short	 ReceiverInfofromDumpMemory(const char* _pathImagefile);
		void	 CloseDumpMemoryFile();
		void   Set_type_analyse( unsigned short vtype ) { fa.type_analyse=vtype; }
		void   Set_ring_buffer() { f_ring_buffer=1; }
		void   Set_first_letter_name(short _f_letter_name) { f_letter_name=_f_letter_name; } 
		short  AnalysReadfRec();
		short	 ChangeSessionName();

    buffers       bf; // internal buffers for communications. 
        
    short         rec_type_code;
		char					ser_num_rec[21];
		char					rid_options[100];
    header_strct  file_header;
    rec_type      typ_rec; // C_GETTYPE ("\x6")
    file_dir      file_rec;
    name_files    vfile_name;
    
		file_dir 			all_file_rec;
		rawfilesMZ		rawfile;	
				
		long          data_space; 
    f_writefile   fwf; 
    f_createfile  fcf;
    rawheader     rh;         // for  B-file
    rawnav        rn;         // --------- " --------
    rawdata       rd[24];     // --------- " --------
    rawnav        rn_ranger2;
		navstrct      enav[32];   // for E-file GPS
    navstrct_g    enav_g[24]; // for E-file GLONASS
    sattdstrct    att_strct;  // + S-file
    almstrct      alm[32];
		malmstrct			malm[MAX_NUM_FILES];
		char          str_photofile[30]; // for write in PHOTO-file.
    char          str_Dfile[MAX_LEN_DSTR];
    char          str_Sfile[600];   
    rnavstrct     rnav;
    ralmstrct 	  ralm;     
    char          str_RAfile[1810];
    char          one_epochbuf[4500]; // 4500 for C-file ?
    char          one_satbuf[150];
		short         len_epoch;  
    short         len_satbuf; 
    short         len_strS;
    short         len_strD;
    short         len_rnav;
    short         len_strRA;
    short         len_strP;
		short					len_dump;
    short         malmanac_strctindex;
		

		short          f_fillbuf;
		char					 pathDataFile[200];

  protected:

    void     adj_ant_factor(void); // from Ping !!!
    short    Identification_Receiver();
		void     SendRdfromRec();
		short    GetHeadFile();
		void     InitFileHeader_strct();
		short    GetRingHead();
		short		 GetTypeData();

		void		 GetRingAddrOffset(char mode);

    unsigned short Checksum16(const unsigned char *ibf);
    short 				 IndentBuf_CheckingChecksum16();
    unsigned short ChecksumXOR(const unsigned char *ibf, unsigned short unused_bytes);

    void     extract_ulong (short start_bit, short end_bit, long *result);
    long     extract_long (short *start_bit, short total_bits, short mode);
    void     broadcast_xyz(unsigned char j,double t,double satp[3]);
    void     compute_azel();

    short		 Change_num_obs_types();
		void     FormInfFileRec();
    void     BfileRawhead();
    void     extract_navData( short &st_byte );
    short    ZtinyComp_cword();
    short    Rgr2Comp_cword();
    short    ReadZTINYTYPE();
    short    swap_rawSNAV();
    short    swap_strctSNAV();
    short    ReadSNAV();
    short		 Rgr1Comp_cword();
		short		 ReadTINYRANGER1();
		short    ReadTINYRANGER2();
    short    ReadTINYDRANGER2(); // for MicroZ only ! 
    short		 Rgr3Comp_cword();
		short		 ReadTINYRANGER3();
		void     oneEpoch_forBfile();
		short		 ReadATT_3DF();
		short		 ReadHeadEVENT();
		short		 ReadEVENT();
		short		 ReadMBEN();
		void		 swap_rawPBEN();
		short		 swap_strctPBEN();
		short		 ReadPBEN();
		short		 ReadRNAV();
		short		 ReadRALMnpages();
		short		 ReadRALM();
		void		 PtinyComp_cword();
		short		 ReadPTINYTYPE();
		void     A5benComp_cword();
		short    ReadA5BENstruct();
		void     TinycodComp_cword();
		short    ReadTINYCODELESS();
		void		 BenComp_cword();
		short		 ReadBENstruct();

		void init_variables(double mr[12],double jacm[12][4],short satn);
    void calculate_range(double mr[12],double satp[32][3],
												 short idsv[12],short satn,
												 double cr[12],double drv[12]);
    void jacobian(short idsv[12],short satn,
									double cr[12],double satp[32][3],double jacm[12][4]);
    short  calculate_pdop(double jacmt_jacm[4][4],
													double L[4][4],double &pdop);
    void new_position(double jacmt_dr[4],double L[4][4]);
    void satellite_data(short &satn,short chan_indx[12],short idsv[12],
												double satp[32][3]);
    void azel_sat(short satn,short chan_indx[12],short idsv[12],
									double satp[32][3]);
    void prec_navigation(double save_pos[4]);
    void new_navigation(double save_pos[4],double save_rcv,double pdop);
    void position();

		long		 wn_ranger2;  
    float    edt_ranger2; 

		FILE						  *fImage;	
		raw_image_header  image_head; // for R-files "hose"-receiver.
    unsigned char		  pathImagefile[200];
		ring_header_strct	ring_head;
		short             f_letter_name;
		
    comm_formats   com; 
        
    pbenstrct      pben;
    snavstrct      esnav;
    snavstrct_g    esnav_g;
    photo_data     photo;

    read_data      rdd;
    analyse_data   fa;
    computed_data  cd;

		short          day_week;
    long           wn_ephem;
    double         tow_ephem;
    double         tk_utc;

    short          f_one_epoch; // if==1, write rh for B-file in one_epochbuf.  
    short					 f_open_Dfile;	// if==1, write header for D-file
		unsigned short time_tag;
		char           MZreceiver_option[10]; 
    float          ant_factor; // from Ping !!!	
		short          almanac_number;
		unsigned char  f_GPS_time;
		short          f_swap_dataZ18;
		short					 f_ring_buffer;	
    unsigned char  f_beg_rcv_time;
		unsigned char  f_renfile_cline;
    long           beg_rcv_time;
};

void					 swap_nbytes( char *head, short count ); // motorola -> PC
unsigned short swap_bytes(unsigned short bytes);
void					 pack_word_bits(unsigned char *page,short word_num);
double				 elevation(double locx,double locy,double locz,
												 double svx, double svy, double svz);
double				 azim(double locx,double locy,double locz,
										double svx, double svy, double svz);
void					 gps_to_mjd(long gpswk,double secofwk,long *pmjd,double *pfmjd);
void					 fmjd_to_hms(double fmjd,long *phour,long *pmin,double *psec);
void					 gps_to_ydhms(long 	 gpswk,
														double secofwk,
														long 	 *pyear,
														long 	 *pyday,
														long 	 *phour,
														long	 *pminute,
														double *psecond);
void					 month_and_day(long year,long yday,long *pmonth,long *pmday);
void					 gps_to_ymdhms(long gpswk,
														 double secofwk,
														 long *pyear,
														 long *pmonth,
														 long *pmday,
														 long *phour,
														 long *pminute,
														 double *psecond);
long					 day_of_year (long year, long month, long mday);
void					 ydhms_to_mjd (long year, long yday, long hour, long minute,
														 double second, long *pmjd, double *pfmjd);
void					 mjd_to_gps (long mjd, double fmjd,
													 long *pgpswk, double *psecofwk);
void					 ydhms_to_gps (long year, long yday, long hour, long minute,
														 double second, long *pgpswk, double *psecofwk);
#endif
