///////////////////////////////////////////////////////////////////////////////	
//    Ashtech receiver tools
//    Copyright (C) 1999  Magellan Coropration/Ashtech Precision Products 
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
///////////////////////////////////////////////////////////////////////////////


#include <math.h>

#ifndef __TRANSFOR_HPP
#include "transfor.hpp"
#endif

const double TOL =	1.0e-14;
const double VPI =	3.141592653589793e+00;
const double DTR =	1.745329251994330e-02;
const double RTD =	5.72957795130823229e+01;

//	===================================================================
//	NAME:		geod_to_xyz
//
//	PURPOSE:	To Convert Geodetic North Latitude,East Longitude,And
//				Ellipsoidal Height To Cartesian X,Y,And Z Coordinates
//				Or Visa-Versa (Depending On The "g2xflag" Flag
//
//	RETURNS:	void
//
//	HISTORY:
//	===================================================================
void geod_to_xyz(short g2xflag, double ae, double flat, double *nlat,
		 double *elon, double *ht, double *x, double *y, double *z
		)
{{
 double 	   rsq,r,e,rho,nlatr,cphi,dr,dz,dht,dnlatr;
 double 	   flatfn,funsq,sphi,g1,g2;
 short    	   i;

 flatfn=(2.0-flat)*flat;
 funsq=(1.0-flat)*(1.0-flat);

 if(g2xflag)  /* If g2xflag is non-zero...*/
 { /* geod to xyz */
   sphi=sin(*nlat*DTR);
   g1=ae/sqrt(1.0-flatfn*sphi*sphi);
   g2=g1*funsq+*ht;
   g1=g1+*ht;
   *x=g1*cos(*nlat*DTR);
   *y=*x*sin(*elon*DTR);
   *x=*x*cos(*elon*DTR);
   *z=g2*sphi;
 }
 else         /* Else g2xflag is zero...*/
 { /* xyz to geod */
   rsq=(*x)*(*x)+(*y)*(*y);
   r=sqrt(rsq);
   if(!*x) *x += 0.0001;
   e=atan2(*y,*x);
   if(e<0.0) e=e+2*VPI;
   rho=sqrt((*z)*(*z)+rsq);
   sphi=*z/rho;
   nlatr=asin(sphi);
   *ht=rho-ae*(1.0-flat*sphi*sphi);
   for(i=0; i<10; i++)
   { sphi=sin(nlatr);
     cphi=cos(nlatr);
     g1=ae/sqrt(1.0-flatfn*sphi*sphi);
     g2=g1*funsq+*ht;
     g1=g1+*ht;
     dr=r-g1*cphi;
     dz=*z-g2*sphi;
     dht=dr*cphi+dz*sphi;
     *ht=*ht+dht;
     dnlatr=(dz*cphi-dr*sphi)/(ae+*ht);
     nlatr=nlatr+dnlatr;
     if((fabs(dnlatr)<=TOL)&&((fabs(dht)/(ae+*ht))<=TOL)) goto WRAPUP;
   }
WRAPUP:
   *nlat=nlatr*RTD;
   *elon=e*RTD;
   *ht=*ht;
 }
}}

