#!/usr/bin/perl
#    Ashtech remote receiver access tools
#    Copyright (C) 1999  Magellan Coropration/Ashtech Precision Products 
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
#    FILE:        exec.pl
#
#    DESCRIPTION: This PERL script serves as the shell for another scripts
#             
#
#    $Log: exec.pl,v $
#    Revision 1.6  2000/01/05 21:28:16  carmen
#    Y2K bg was fixed
#
#    Revision 1.5  1999/12/10 05:11:59  serg
#    a) Added support of timeouts using ALARM signal. That is more
#       portable than using stty. (Works for TCP/IP as well, stty - doesn't)
#    b) Added support of TCP connectio
#    c) Time checking was made more reliable in the log.pl. Now we go through
#       the whole buffer, and do compare if time is greater or equal. Before we made
#       equal only comparision.
#    d) We do not delete B,E,S files any more
#    e) For the session programmimg parameters are now read from the same confguration
#       file variables as for logging (MinSvNum,ElevationMask,RecInt)
#
#    Revision 1.4  1999/12/03 22:37:25  serg
#    *** empty log message ***
#
#    Revision 1.3  1999/12/03 03:52:50  serg
#    The raw data logging capability has been added.
#    Some code clean up.
#
#    Revision 1.2  1999/11/11 18:07:16  serg
#    Separate directories added for zmodem, convert and teqc binaries
#    Z12 Session programming and file deletion was added
#    r-convert and u-convert were replaced with convert
#    The $DoCheckSum,$WaitAck optional parameter were added to Command() function
#    WaitFor() function was added
#
#    Revision 1.1  1999/10/14 00:48:44  serg
#    Just new files.
#
#
#############################################################################

require "em1000.pl";  # This library contains the support of ETHERNET modem
require "ash-lib.pl";  # This library contains some useful subroutines
                      

if ($#ARGV  < 0 ) {
  print STDERR "Configuration file name must be specified\n";
  exit 1;
}

# Get the configuration

print STDERR "Get configuration\n";

$ConfigFile  = $ARGV[0];

# Build the full path name to config file

if ( substr ($ConfigFile,0,1 ) ne "/" )  
{
    chop($cwd = `pwd`);
    $ConfigFile = $cwd . "/" . $ConfigFile;
} 

$ScriptName  = ReadVar($ConfigFile,"ScriptName","");

if ( $#ARGV  > 0 )
{
    $ScriptName  = $ARGV[1];
} 

$ScriptDir   = ReadVar($ConfigFile,"ScriptDir",".");
$RootDataDir = ReadVar($ConfigFile,"DataDir",".");
$Port        = ReadVar($ConfigFile,"Port","/dev/ttyS0");
$MailList    = ReadVar($ConfigFile,"MailList","");

#
# Change the directory 
#

print STDERR "Set working directory\n";

my ($sec,$min,$hour,$mday,$mon,$year) = localtime();


$DirBaseName = sprintf("%s%02d%02d%02d.",($RootDataDir . "/" , $mon+1 ,$mday ,  $year % 100  . ".") );

CREATE_DATA_DIR: for (0...999) {
    $DataDir = $DirBaseName . sprintf("%03d/",$_);
    last CREATE_DATA_DIR if mkdir ( $DataDir ,0777 ) ;
}

chdir $DataDir or die "Can not change directry to $DataDir\n";
print STDERR "Directory: $DataDir\n";

#
# Some local variables
#

$LogName      = "fulllog.txt";
$LogFile      = $DataDir . "/" . $LogName;
$ReportFile   = $DataDir . "/mail.txt";

#
# Run script
#

if ( $MailList ne "" )
{
    $ErrorOutput = " 2>$LogFile";
}
else
{
    $ErrorOutput = "";
}


if ( $ScriptName ne "" )
{
    $ScriptName = $ScriptDir . "/" . $ScriptName;
    $Args = $ConfigFile ." " . " " . $ReportFile ; 
    $Command = "perl -w -I $ScriptDir $ScriptName $Args < $Port > $Port " . $ErrorOutput;
    print STDERR ">$Command\n";
    system $Command;
}
else
{
    print STDERR "No script to run was specified in config file.\n";
    print STDERR "Is that what you wanted ?\n";
}

#
# Send email
#

if ( $MailList ne "" )
{
  # UUENCODE the attachment
  $UuEncodedFile = $DataDir . "/fulllog.uue";
  $Command = "uuencode $LogFile $LogName > $UuEncodedFile";
  print STDERR ">$Command\n";
  system $Command;

  $Title  = ReadVar($ConfigFile,"Title","No title specified") . " at  $Port ";
  $Title  = ' " ' . $Title  .  ' "';  # Put title to double qoutes

  # Send e-mail
  $Command = "cat $ReportFile $UuEncodedFile | mail $MailList -s $Title"; 
  print STDERR ">$Command\n";
  system $Command;
}









