#!/usr/bin/perl
#    Ashtech remote receiver access tools
#    Copyright (C) 1999  Magellan Coropration/Ashtech Precision Products 
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
#     FILE:        log.pl
#
#     DESCRIPTION: This PERL script logs the real time data from the
#                  serial port of receiver
#
#     $Id: log.pl,v 1.8 2000/01/05 21:28:16 carmen Exp $
#
#
#############################################################################

require "ash-lib.pl";  # This library contains some useful subroutines
                         # like Connect(), Query(), Command()
require "mdm-lib.pl";  # This library contains some useful subroutines
                       # for all modem related scripts.
require "em1000.pl";  # This library contains the support of ETHERNET modem
                       
$Killed = 0;

if ($#ARGV <1 ) {
  print STDERR "Correct usage is:\n";
  print STDERR "remote33.pl ConfigFile ReportFile\n";
  exit 1;
}

$ConfigFile = $ARGV[0];
$LogFile    = $ARGV[1];

print STDERR "Script params: $ConfigFile $LogFile\n";

#
# Write current directory to the Log Identification file
#

$ScriptDir   = ReadVar($ConfigFile,"ScriptDir",".");
$Port        = ReadVar($ConfigFile,"Port");

$Port =~ tr/\//\./; # Replace / with . 

$LogIdFile  = $ScriptDir . "/" . $Port;
print STDERR "Open ID file [$LogIdFile]\n";
open(LOG_ID,">$LogIdFile") or AbortScript ("Can't open ID file $LogIdFile");
$CurrDir=qx{pwd};
print LOG_ID ($CurrDir);
close(LOG_ID);


# ------------------------------------------------------------------
# Read the configuration
# ------------------------------------------------------------------

$Title        = ReadVar($ConfigFile,"Title","No title specified");
$Connection   = ReadVar($ConfigFile,"Connection");
$ReceiverRate = ReadVar($ConfigFile,"ReceiverRate","7","[3,4,5,6,7,8,9]");

if ( $Connection  eq  "modem" )  
{
    $BaudRate     = ReadVar($ConfigFile,"ConnectionRate",38400,"[2400,4800,9600,19200,38400,57600,115200]");
    $PhoneNumber  = ReadVar($ConfigFile,"PhoneNumber");
    $InitString   = ReadVar($ConfigFile,"PCModemInitString","ATS0=0S12=25&H1X4E0M3&D0&W");
}

$HostName     = ReadVar($ConfigFile,"HostName","www.yahoo.com");

$ScriptDir   = ReadVar($ConfigFile,"ScriptDir",".");

$RCV_UNKN =  0;
$RCV_Z12 =  1;
$RCV_UZ =  2;
$ReceiverType = $RCV_UNKN;


$MS_IN_WEEK = 7*24*3600*1000; # Number of milisecinds in a week

print STDERR "Open log file\n";

open(LOG,">$LogFile") or AbortScript ("Can't open log file $LogFile");

#
# Print the report header 
#

print LOG "Test Name: $Title \n";
$CurrTime = localtime();
print LOG $CurrTime, "\n";
$CurrDir=qx{pwd};
PrintStatus("Data Directory: $CurrDir\n");

# ------------------------------------------------------------------
# Connect to the receiver
# ------------------------------------------------------------------

if ( $Connection  eq  "direct" )
{
    PrintStatus ("Direct serial connection\n");
    $Connected =  Connect( $ReceiverRate );
}
elsif ( $Connection  eq  "modem" )  
{
    PrintStatus ("Dial phone $PhoneNumber to connect to receiver\n");
    HangUp ();
    $Connected =  not DialPhone ($BaudRate, $PhoneNumber, $InitString);
}
elsif ( $Connection  eq  "tcp" ) 
{
    $Connected =  TcpConnect( $HostName );
}
else
{
    AbortScript ("No connection type was specfied");
}

if ( $Connected ) 
{
    PrintStatus ("Connected to receiver\n");
}
else
{
    AbortScript ("Connection to receiver failed\n");
}

#----------------------------------------------------------------------
#  Configure receiver 
#---------------------------------------------------------------------

ConfigureReceiver();

#----------------------------------------------------------------------
#  Log data 
#---------------------------------------------------------------------

LogData();

close(LOG);

if ( $Connection  eq  "modem" )  
{
    HangUp ();
}

1; 

############################################################################
#
# Subroutine:  LogData
#
# Description: Aborts the script
#
# Parameters:  
#
############################################################################
sub LogData
{

    $SessNum =0;
    $BuffLen = 1024;
    $TempFileName = ".temp.log";

    $SiteName     = ReadVar($ConfigFile,"SiteName","____"); 

    $SessDuration = ReadVar($ConfigFile,"SessDuration",60); # Minutes
    $SessDuration *= 60*1000;  # ms

    print STDERR "Open log file $TempFileName\n";
    open(LOGFILE,">$TempFileName") or die "Can not open log file $TempFileName";

    $SIG{INT} = \&catch_zap;  # Install signall handler

    
    print "\$PASHQ,SNV\n\r";  # Query ephemeris

    undef $EndTime;

  LoggingData: while (not $Killed) 
  {
      $ReadLen = 0;
      $timeout = 60;
      eval {
	  local $SIG{ALRM} = sub { die "alarm\n" };       # NB \n required
	  alarm $timeout;
	  $ReadLen = read(STDIN,$Buff,$BuffLen);
	  alarm 0;
      };

      if ( $ReadLen > 0 )
      {
	  ($Time,$LastPbnPos) = CheckTime($Buff,$EndTime);
	  $SetEndTime = 0;
	  $SetEndTime = 1  if ( not defined($EndTime) and defined($Time) );
	  $SetEndTime = 2  if ( defined $LastPbnPos) ;
	  $EndTime = $Time + ($SessDuration -  $Time % $SessDuration ) if ( $SetEndTime );

	  if ( $SetEndTime == 2 )
	  {
	      # Write part of the buffer containing the PBN 
	      # with the given yime tag to the file and then close it
	      $SplitPoint = $LastPbnPos + 58;
	      $SplitPoint = $ReadLen if $SplitPoint > $ReadLen;
	      $Head = substr($Buff,0,$SplitPoint);
	      $Tail = substr($Buff,$SplitPoint);
	      syswrite(LOGFILE,$Head,$SplitPoint);
	      
	      print STDERR "Close log file $TempFileName\n";
	      close(LOGFILE) or die "Can not close log file $FileName";

	      # Give the valid name to the  temporary file
	      $FileName = ComposeLogFileName($SiteName,$SessNum);
	      print STDERR "Rename $TempFileName to $FileName\n";
	      rename ($TempFileName,$FileName) or die "Can not rename files\n";
	      
	      # Open the new file, write remainder of the buffer to it
	      print STDERR "Open  log file $TempFileName\n";
	      open(LOGFILE,">$TempFileName") or die "Can not open file $TempFileName";
	      syswrite(LOGFILE,$Tail,$ReadLen - $SplitPoint);
	      
	      # Make sure we have  ephemeris in the new file
	      print "\$PASHQ,SNV\n\r";  
	      ++$SessNum;
	  }
	  else
	  {
	      syswrite(LOGFILE,$Buff,$ReadLen);
	  }
      }
  }

    if ( $Killed ) 
    {
	print STDERR "Log processed killed, clean up...\n";
	print STDERR "Close log file $TempFileName\n";
	close(LOGFILE) or die "Can not close log file $FileName";

	# Give the valid name to the  temporary file
	$FileName = ComposeLogFileName("____",$SessNum);
	print STDERR "Rename $TempFileName to $FileName\n";
	rename ($TempFileName,$FileName) or die "Can not rename files\n";

	print STDERR "Delete ID file [$LogIdFile]\n";
	unlink $LogIdFile;
    }
}

############################################################################
#
# Subroutine:  CheckTime
#
# Description: Aborts the script
#
# Parameters:  
#
############################################################################
sub CheckTime
{
    my ($Buff,$EndTime) = @_;
    my ($Time,$LastPbnPos,$PbnPos); 

    LOOK_FOR_PBN: while ( $Buff =~ m/\$PASHR,PBN,/g ) # Find PBN mesage
    {
	$LastPbnPos = pos($Buff);
	$Tail = substr($Buff,$LastPbnPos,4);
	($Time) = unpack("N",$Tail);
	next LOOK_FOR_PBN if not defined $Time;

#	print STDERR "Time=$Time\n";
	
	if ( defined($EndTime) ) 
	{
	    if($Time <  ($EndTime - $MS_IN_WEEK/2 ) ) # Check for week rollover
	    {
		$Time += $MS_IN_WEEK;
	    }

	    if (($Time >= $EndTime)  )
	    {
		$PbnPos = $LastPbnPos;
		print STDERR "....LastPbnPos=$LastPbnPos\n";
		last LOOK_FOR_PBN; 
	    }
	}
    }
    return ($Time,$PbnPos);
}

############################################################################
#
# Subroutine:  ExtractTime
#
# Description: Aborts the script
#
# Parameters:  
#
############################################################################
sub ExtractTime
{
    my ($Buff) = @_;
    undef $Time;

    if ( $Buff =~ m/\$PASHR,PBN,/g ) # Find PBN mesage
    {
	$LastPbnPos = pos($Buff);
	$Tail = substr($Buff,$LastPbnPos,4);
	($Time) = unpack("N",$Tail);
    }
    return $Time;
}


############################################################################
#
# Subroutine:  AbortScript
#
# Description: Aborts the script
#
# Parameters:  
#
############################################################################
sub AbortScript
{
    print LOG @_ , "\n";
    print STDERR @_, "\n";

    print LOG "Test aborted\n";

    close(LOG);
    die "Test aborted";
}


############################################################################
#
# Subroutine:  PrintStatus
#
# Description: 
#
# Parameters:  
#
############################################################################
sub PrintStatus
{
    print STDERR @_;
}


############################################################################
#
# Subroutine:  ConfigureReceiver
#
# Description: 
#
# Parameters:  
#
############################################################################

sub ConfigureReceiver
{
    $MinSvNum      = ReadVar($ConfigFile,"MinSvNum",3,1,12);
    $ElevationMask = ReadVar($ConfigFile,"ElevationMask",10,0,90);
    $RecInt        = ReadVar($ConfigFile,"RecInt",10,0.2,999);

    $MultiPathMode = ReadVar($ConfigFile,"MultiPathMode",0, "[0,1,2]");
    $GPSSyncMode   = ReadVar($ConfigFile,"GPSSyncMode","N", "[Y,N]");

    ($Port) = Query("PRT");  # Find what port we are connected to 

# Turn off all current output
    Command("OUT,$Port");
    Command("NME,ALL,$Port,OFF");

# Set up recording parameters
    Command("SES,DEL");  # Turn off session programming

    Command("MSV,$MinSvNum");
    Command("ELM,$ElevationMask");
    Command("RCI,$RecInt");

    # Set Multipath mode
    $Line = "\$PASHG,MULM,$MultiPathMode\n\r";
    print STDERR "< " , $Line; 
    print $Line; 
    WaitFor(3,('PASHR\,MULM') ) eq 0  
	or AbortScript("Can not set Multipath Mode");
       
    #Set GPS sync mode
    Command("UTS,$GPSSyncMode") eq "ACK" 
	or AbortScript("Can not set GPS Sync Mode");
    
# Turn output on
    Command("OUT,$Port,MBN,PBN,SNV,BIN");
}

sub ComposeLogFileName{
    my($SiteName,$SessNum) = @_;
    my($sec,$min,$hour,$mday,$mon,$year,$wday,$yday) = gmtime(time);
    my($SessId,$FileName);

    $SiteName =~tr/\?/_/; # Replace ???? with ____
	
    if ( $SessNum < 26 ) 
    {
	$SessId = chr(65+$SessNum) . sprintf("%02d",$year % 100);
    }
    else
    {
	$SessId = chr(65+($SessNum / 10)) . chr(65+($SessNum % 10)) . $year % 10;
    }

    $FileName = "L" . $SiteName . $SessId  . sprintf(".%03d",$yday+1);

    return $FileName;
} 

sub catch_zap {
    $Killed = 1;
}


