#!/usr/bin/perl
#    Ashtech remote receiver access tools
#    Copyright (C) 1999  Magellan Coropration/Ashtech Precision Products 
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
#
# FILE:        mdm-lib.pl
#
# DESCRIPTION: This script contains some useful subroutines
#              for all modem related scripts.
#
#############################################################


############################################################################
#
# Subroutine: DialPhone
#
# Description: This function returns
#              0 - successful connection to the receiver's modem.
#              1 - The PC modem does not respond to 'AT' command.
#              2 - The PC modem does not respond the initialization
#                  command.
#              3 - The PC modem cannot establish communication with the 
#                  receiver modem. 
#
############################################################################
sub DialPhone {
    ($BaudRate, $PhoneNumber, $InitString) = @_;

    $NumOfTries  = 10;

    # Set the 2 sec timeout on reading data from port
    system "stty time 50 min 0 -icanon";
    system  "stty   $BaudRate";  # Set  the port baud rate

    # Print a separation line.
    $Line = "\n===== Beginning of dialing process.\n";
    print STDERR $Line;

    # Check if the modem is present 

    if ( ModemCommand("AT") ne "OK" )
    {
        $Line = "** Test is aborted at " . localtime () . "\n" . "** Modem is not detected.\n";
        print STDERR $Line;
	    return (1);
    }

    # Initialize the modem 

    if ( ModemCommand($InitString) ne "OK" )
    {
        $Line = "** Test is aborted at " . localtime () . "\n" . "** Can not initialize the modem.\n";
        print STDERR $Line;
	    return (2);
    }

    # Dial the phone

    $DialString =  "ATDT$PhoneNumber" . "\n\r"; 

    $LoopCnt = 0;

    RepeatDialing: while ($LoopCnt <= $NumOfTries)
    {
	    $Line = "** " . localtime () . "\n" . "** Number of retries = " . $LoopCnt . "\n";
        print STDERR $Line;
	    $Line = "< " . $DialString; 
        print STDERR $Line;

	    # Send  command itself
	    print $DialString; 

	    $EndWaitTime = time() + 60 ; 

	    $status = "";

	    WaitForConnect: while ( time() < $EndWaitTime )
	    {
		    while (<STDIN>) {
			    $Line = "> " . $_; 
                print STDERR $Line;
			    if ( /^CONNECT/ )    {$status = "ONLINE"; last  RepeatDialing;}
			    else
                {
                    $status = "RETRY";
                    if (/^BUSY/ or /^NO CARRIER/) 
                    {
                        last WaitForConnect;
                    }
                }
		    }
	    }	
	    if ($status eq "RETRY")
	    {
		    $LoopCnt++;
		    sleep (30); # wait for 30 seconds before re-dialing.
	    }
    }

    if ( $status eq  "ONLINE" ) 
    {
        sleep (5); # wait for a while before talking to the modem.

        @Params = Query("PRT"); #Send $PASHQ,PRT and wait for $PASHR,RPT,x,y
	    if ( $#Params >= 1 )    # The $PASHR,RPT,x,y was recived
	    {
            ($port,$rate) = @Params;   # Put x and y to $port,$rate vars
            $Line = "** Connected to port " . $port . " of remote  receiver\n";
            print STDERR $Line;
	    }
	    else
	    {
		    $Line = "** No response from the receiver for \$PASHQ,PRT\n";
            print STDERR $Line;
	    }

        return (0); 
    }
    else
    {
        $Line = "** There is error at " . localtime () . "\n" . "** Connection failed.\n"; 
        print STDERR $Line;
        return (3);
    }
}




############################################################################
#
# Subroutine: HangUp
#
# Description: 
#
############################################################################
sub HangUp {

    # Set the 2 sec timeout on reading data from port
    system "stty time 50 min 0 -icanon";
	# Send escape sequence to hang up the Modem
    sleep (2);
    syswrite (STDOUT, "+++", 3);
    sleep (2);

    ModemCommand("ATH0");

    sleep (90); # wait for the modem to be able to detect the CARRIER.
}


############################################################################
#
# Subroutine: ModemCommand
#
# Description: 
#
############################################################################
sub ModemCommand {
    my ($command) = @_;
    my $Line;

    # Clean the input buffer
    while (<STDIN>) {
        $Line = "%>" . $_;
        print STDERR $Line;
    }

    # Build command string
    $Line = $command  . "\n\r";

    print STDERR "< ", $Line;
 
    # Send  command itself
    print $Line; 

    WaitForResponse: while (<STDIN>) {
        $Line = "> " . $_;
        print STDERR $Line;

        if ( $_ eq "OK\r\n" ) {  return "OK" }
    }
    
    return "";
}

############################################################################
#
# Subroutine: VerifyModemSetup
#
# Description: Check the modem setup configuration saved in the receiver.
#
# Arguments:
#              Receiver port which is connected to the modem.
#              The Port baud rate.
# Return:
#              0 - Modem setup does not match.
#              1 - Modem setup is good.
############################################################################
sub VerifyModemSetup
{
    
    ($ModemPort, $BaudRate) = @_;

    $Status             = "ON";
    $TypeIdx            = "4";
    $UserInitString     = "ATS0=1S12=25&H1X4E0M3&D0&W&C1";
    $UserDefaultConfStr = "AT\&F1";
    $UserEscapeStr      = "+++";
    $UserCmdToDataStr   = "ATO";
    $ModemName          = "MY_MODEM";


    # Configuration string for $PASHS,MDM command
    $ModemConfig = join(',',($Status,$ModemPort,$TypeIdx,$BaudRate,
            "CFG",$UserInitString,
            "MOD",$UserDefaultConfStr,
            "NAM",$ModemName,
            "D2C",$UserEscapeStr,
            "C2D",$UserCmdToDataStr));

    # Configuratoion expected from $PASHQ,MDM
    @DesiredConfig = ($ModemPort, $BaudRate, $Status, $TypeIdx, 
            $UserInitString,$UserDefaultConfStr,
            $UserEscapeStr,$UserCmdToDataStr);


    #   
    # Get current modem settings
    #

    GetModemCfg();

    if ($CurrentModemSettings[2] eq "INITOK")
    {
        $DesiredConfig[2] = "INITOK";
    }

    #pop @CurrentModemSettings; # Get rid of the chec

    if ( join(',',@CurrentModemSettings) ne  join(',',@DesiredConfig) )
    {
		$CurList = join(',',@CurrentModemSettings);
		$Line = "** Cur=[" . $CurList . "]\n";
		print STDERR $Line;
		$DesList = join(',',@DesiredConfig);
		$Line = "** Des=[" . $DesList . "]\n";
		print STDERR $Line;
        return (0);
    }
    else 
    {
        return (1);
    }
}


############################################################################
#
# Subroutine: SetupModem
#
# Description: this function sends a desired modem initialization string to
#              the receiver modem.
# Return:
#              0 - when $PASHR,MDM,INI,FAIL response is received.
#              1 - when $PASHR,MDM,INI,OK   response is received.
#
############################################################################
sub SetupModem
{

    Command("MDM,". $ModemConfig);
    sleep (5);
    Command ("SAV,Y");
    GetModemCfg();
    Command("MDM,INI"); # Initialize the modem with the new setting.
    sleep (5);

    $EndWaitTime = time() + 60 ; 

	WaitForINI: while ( time() < $EndWaitTime )
	{
	    while (<STDIN>) {
            print STDERR ">", $_;
            ($hdr,$Mdm, $Ini, $ReceivedCode) = split /[,\*]/, $_;
            if ( $ReceivedCode eq "FAIL" ) {  $status = 0; last WaitForINI; }
            if ( $ReceivedCode eq "OK" )   {  $status = 1; last WaitForINI; }
	    }	
    }

    if ($status == 1)
    {
        $Line = "** Modem configuration changed at " . localtime () . "\n";
        print STDERR $Line;

        pop @CurrentModemSettings; # Get rid of the chec
        $NewList = join(',',@CurrentModemSettings);
        $Line = "** New=[" . $NewList . "]\n";
        print STDERR $Line;
        return (1);
    }
    else
    {
        $Line = "** Modem Initialization Fails at " . localtime () . "\n";
        print STDERR $Line;
        return (0);
    }

}


############################################################################
#
# Subroutine: GetModemCfg
#
# Description:
#
############################################################################
sub GetModemCfg
{
    @CurrentModemSettings = Query("MDM");

    ($ModemPort, $BaudRate, $Status, $TypeIdx, 
     $UserInitString,$UserDefaultConfStr,
     $UserEscapeStr,$UserCmdToDataStr) = @CurrentModemSettings;
 
print STDERR <<MODEM_SETUP;
  Modem port:   $ModemPort at rate $BaudRate
  Modem status: $Status
  Modem type:   $TypeIdx
		
  User init string:   $UserInitString
  User save string:   $UserDefaultConfStr
  User escape string: $UserEscapeStr
  User online string: $UserCmdToDataStr
MODEM_SETUP
}



# This line has to be the very last line of the module, for some 
# obscured for me reason 
1; #return true , to please the require satement in main module


