#!/usr/bin/perl
#    Ashtech remote receiver access tools
#    Copyright (C) 1999  Magellan Coropration/Ashtech Precision Products 
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
#     FILE:        remote33.pl
#
#     DESCRIPTION: This PERL script downloads the data from  uZ,
#              conevrt it to RINEX format. The test 
#              This script has to be started like this:
#              ./remote33.pl <ConfiFile> <ReportFile> < /dev/ttyR0 > /dev/ttyR0
#              to have it standart inout and output redirected
#              to the  port connected to the modem or directly to receiver
#
#     $Id: remote33.pl,v 1.23 2000/01/05 21:28:16 carmen Exp $
#
#
#############################################################################
use strict
require "ash-lib.pl";  # This library contains some useful subroutines
                       # like Connect(), Query(), Command()
require "mdm-lib.pl";  # This library contains some useful subroutines
                       # for all modem related scripts.

require "z12dld.pl";   # Z-12 download support 
require "uzdld.pl" ;    # Micro Z download support 
require "rawdld.pl";   # Raw data logging support
require "em1000.pl";  # This library contains the support of ETHERNET modem

# ------------------------------------------------------------------
# Constants
# ------------------------------------------------------------------

$MNT_TIMEOUT=300; # Give maximum of 5 min for the data card to be mounted
$DownloadTries = 3;

$RCV_UNKN =  0;
$RCV_Z12 =  1;
$RCV_UZ =  2;

# ------------------------------------------------------------------
# Globals
# ------------------------------------------------------------------

$ReceiverType = $RCV_UNKN;

# ------------------------------------------------------------------
# Read configuration from script args  and config file
# ------------------------------------------------------------------

ReadConfiguration();

print STDERR "Open log file\n";
open(LOG,">$LogFile") or AbortScript ("Can't open log file");

#
# Print the report header 
#

print LOG "Test Name: $Title \n";
$CurrTime = localtime();
print LOG $CurrTime, "\n";
$CurrDir=qx{pwd};
PrintStatus("Data Directory: $CurrDir\n");

# ------------------------------------------------------------------
# Configure the modem connected to the receiver
# ------------------------------------------------------------------

if ( $Configure eq "yes") 
{
    # We are going to setup the receiver modem before running the test
    # using one more serial port directly connected to the receiver

    PrintStatus ("Do the modem configuration\n");
    $rc = ConfigureReceiverModem();

    if ( $rc != 0 )
    {
	print LOG "Modem configuration has been changed !!!\n";
    }
    else
    {
	print LOG "Modem succesfully configured !!!\n";
    }
} 

# ------------------------------------------------------------------
# Connect to the receiver
# ------------------------------------------------------------------

$Connected = ConnectToReceiver($Connection);

if ( $Connected ) 
{
    PrintStatus ("Connected to receiver\n");
}
else
{
    AbortScript ("Connection to receiver failed\n");
}

#----------------------------------------------------------------------
#  Configure receiver 
#---------------------------------------------------------------------

SetupReceiver($SessDuration);

#----------------------------------------------------------------------------
# Retrieve the list of files
#---------------------------------------------------------------------------

$TotalFiles = GetFilesList();
if ( $TotalFiles < 1 ) 
{
    AbortScript ("No files found in the receiver\n");
}  

#----------------------------------------------------------------------------
# Download files 
#----------------------------------------------------------------------------
 
DOWNLOAD_TRY: for($DldTry=0;$DldTry<$DownloadTries;$DldTry++)
{
    $DownLoadErr = DownloadFiles();
    last DOWNLOAD_TRY if not $DownLoadErr;
    $Connected = ConnectToReceiver($Connection);
    last DOWNLOAD_TRY if not $Connected; 
}

not $DownLoadErr or AbortScript( "Files Download failed after $DldTry attempts");

#----------------------------------------------------------------------------
# Process files 
#----------------------------------------------------------------------------

ConvertAndCheckData();

#----------------------------------------------------------------------------
# Generate the report
#----------------------------------------------------------------------------

GenerateFilesReport($SessDuration);

CloseConnection($Connection);

close(LOG);

1; 

############################################################################
#
# Subroutine: ConfigureReceiverModem
#
# Description: Runs the script to configure the receiver modem
#
# Parameters:  None
#
############################################################################

sub ConfigureReceiverModem
{
    $ScriptDir   = ReadVar($ConfigFile,"ScriptDir",".");
    $ScriptName  = $ScriptDir . "/setup_mdm.pl";
    $RcvrMdmPort = ReadVar($ConfigFile,"RcvrMdmPort","C","[A,B,C,D]"); 
    $ConfigPort  = ReadVar($ConfigFile,"ConfigPort"); 
    
    $Command = "perl -I $ScriptDir $ScriptName $RcvrMdmPort $ReceiverRate /dev/null < $ConfigPort > $ConfigPort";
    
    PrintStatus ("Use port $ConfigPort to configure receiver modem\n");
    PrintStatus ("Modem is connected to port $RcvrMdmPort of  receiver at rate $ReceiverRate\n") ;

    $rc = system $Command;
    return $rc;
}

############################################################################
#
# Subroutine: ConnectToReceiver
#
# Description: Runs the script to configure the receiver modem
#
# Parameters:  None
#
############################################################################

sub ConnectToReceiver
{
    if ( $DataSource eq "stream" )
    {
	$RawDataDir = GetRawDataDir();
	$Connected = 1 if defined ($RawDataDir);
	return $Connected;
    }

    if ( $Connection  eq  "direct" )
    {
	PrintStatus ("Direct serial connection\n");
	$Connected =  Connect( $ReceiverRate );
    }
    elsif ( $Connection  eq  "modem" )  
    {
	PrintStatus ("Dial phone $PhoneNumber to connect to receiver\n");
	HangUp ();
	$Connected =  not DialPhone ($BaudRate, $PhoneNumber, $InitString);
    }
    elsif ( $Connection  eq  "tcp" ) 
    {
	$Connected =  TcpConnect( $HostName );
    }
    else  # It's perfectly legal. User just wanted initialize the modem
    {
	PrintStatus ("No connection type was specfied\n");
	$Connected = 0;
    }

    if ( $Connection  eq  "direct" || $Connection  eq  "modem" || $Connection  eq  "tcp" )
    { 
	EstablishProtocol() or AbortScript ( "Can not establish protocol");
	GetReceiverId()     or AbortScript ( "Can not identify the reciver");
	PrepareReceiver()   or AbortScript ( "Can not prepare receiver");
    }

    return $Connected;
}


############################################################################
#
# Subroutine:  PrepareReceiver
#
# Description: 
#
# Parameters:  None
#
############################################################################

sub PrepareReceiver {

    if( $ReceiverType eq $RCV_UZ )
    {
	Command("BEEP,OFF");
    
	$endtime = time()+$MNT_TIMEOUT;
      MNT_WAIT: while (time() < $endtime) {
	  ($Diag,$Code1,$Code2,$CurrInd,$TotalFiles,$PercentMount) = Query("FSS");
	  last MNT_WAIT if ($PercentMount eq '100'); 
	  sleep 10;
      }
	
	sleep 1;
	
	return 1 if ($PercentMount eq '100'); 
	return 0;
    }

    return 1;
}


############################################################################
#
# Subroutine:  GetReceiverId
#
# Description: 
#
# Parameters:  None
#
############################################################################

sub GetReceiverId {
    
    EstablishProtocol() or return 0;

    @ver = Query("RID");

    if ($ver[0] eq "ZM" ) {$ReceiverType = $RCV_Z12;}
    if ($ver[0] eq "UZ" ) {$ReceiverType = $RCV_UZ;}
    
    print LOG "\nReceiver Firmware Ver: $ver[2]::$ver[4]\n";

    if( $ReceiverType eq $RCV_UZ )
    {
	print "\$PASHQ,SID\n\r";
	$timeout = time()+10;

	$timeout = 3;
	eval {
	    local $SIG{ALRM} = sub { die "alarm\n" };       # NB \n required
	    alarm $timeout;
	  GetSer: while (<STDIN>) 
	  {
	      ($serial) = $_;
	      if ($serial =~ /^SER/) {
		  last GetSer;
	      }
	      if (time() > $timeout) {
		  return 0;
	      }
	  }
	    alarm 0;
	};
	
	$serial = substr($serial,6,15);

	print LOG "Receiver Serial No: $serial";
    }

    return 1;
}
############################################################################
#
# Subroutine:  GetFilesList
#
# Description: This subroutine obtains the list of the recorded files using 
#              the query $PASHQ,FIF,<n> where n is the index of the first 
#              file.
#              Each $PASHR,FIF can contain information for up to 10 files.
#              Format of the $PASHR,FIF is the following
#              $PASHR,FIF,<SpaceLeft>,<TotalFiles>,<FilesInThisMessage>,\
#              [<FileIdx>,<FileName>,<FileDate>,<FileSize>] - this group
#              is repeated <FilesInThisMessage> times
#
# Parameters:  None
#
############################################################################
sub GetFilesList
{
    if ( $DataSource eq "stream" )
    {
	return GetFileListLogger($RawDataDir);
    }

    if( $ReceiverType eq $RCV_Z12 )
    {
	return GetFilesListZ12();
    }

    if( $ReceiverType eq $RCV_UZ )
    {
	return GetFilesListuZ();
    }
}

############################################################################
#
# Subroutine:  SetupReceiver
#
# Description: 
#
# Parameters:  None
#
############################################################################
sub SetupReceiver
{
    my( $SessDuration ) = @_;

    if ( $DataSource eq "stream" )
    {
	return 1; #Do nothing
    }

    if( $ReceiverType eq $RCV_UZ )
    {
	# Set Multipath mode
	$Line = "\$PASHG,MULM,$MultiPathMode\n\r";
	print STDERR "< " , $Line; 
	print $Line; 
	WaitFor(3,('PASHR\,MULM') ) eq 0  
	    or AbortScript("Can not set Multipath Mode");

       
	#Set GPS sync mode
	Command("UTS,$GPSSyncMode") eq "ACK" 
	    or AbortScript("Can not set GPS Sync Mode");

	if ( $DoSessionProg eq "yes" )
	{
	    SetupSessionsuZ($SessDuration);
	}
	else
	{
             # Set up recording parameters
	    Command("SES,DEL")
		or AbortScript("Can not delete sessions");
	    Command("MSV,$MinSvNum")
		or AbortScript("Can not set MSV");
	    Command("ELM,$ElevationMask")
		or AbortScript("Can not set ELM");
	    Command("RCI,$RecInt")
		or AbortScript("Can not set RCI");
	    
	    Command("FIL,C"); 
	    sleep 5; # Let data recording refresh internal structures
	    
	}
	return 1
    }

    if( $ReceiverType eq $RCV_Z12 )
    {
	if ( $DoSessionProg eq "yes" )
	{
	    SetupSessionsZ12($SessDuration);
	}
	else
	{
	    Command("FIL,C");
	}

	return 1;
    }

}

############################################################################
#
# Subroutine:  DownloadFiles
#
# Description: This subroutines uses the utility rz to download receiver U
#              files via ZMODEM.
#              The recieiver initiates the transfer upon receiving command:
#              $PASHS,FIL,DOWNLOAD,<file name>,ZMODEM   
#
# Parameters:  None
#
############################################################################
sub DownloadFiles()
{
    if ( $DataSource eq "stream" )
    {
	return 0; #Do nothing
    }

    if( $ReceiverType eq $RCV_Z12 )
    {
	return DownloadFilesFromZ12();
    }

    if( $ReceiverType eq $RCV_UZ )
    {
	return DownloadFilesuZ();
    }
}
############################################################################
#
# Subroutine:  ConvertAndCheckData
#
# Description: This subroutines converts U-files to the RINEX format 
#              and get the measurement quality information 
#              out of these RINEX files.
# 
#              The conversion to the  RINEX is done in two steps:
#              a) u-convert utility converts U-files to B and E files
#              b) teqc      utility converts B and E files to RINEX
#
# Parameters:  None
#
############################################################################
sub ConvertAndCheckData()
{
    $DataDirName = "./";

    @ArchiveList = ();
    ($sec,$min,$hour,$mday,$mon,$year) = localtime();
    $year = sprintf "%02d", $year % 100;

    for($i=0; $i<$FilesFound ;$i++)
    {
	# Build necessary file names
	$UfileName        = $FileName[$i];
	@FullPathName     = split /\//,$FileName[$i];
	$BaseFileName     =  substr(pop(@FullPathName),1); # Strip leading 'U'
	$B_FileName       =  $DataDirName . "B"  . $BaseFileName;
	$D_FileName       =  $DataDirName . "D"  . $BaseFileName;
	$RinexObsFileName =  $DataDirName . $BaseFileName . ".OBS";
	$RinexNavFileName =  $DataDirName . $BaseFileName . ".NAV";
	$ReportFileName   =  $DataDirName . $BaseFileName . "." . $year . "S";

	if ( $DataSource ne "stream" )
	{
	
	    # Convert [U,R]-file to B,E,S files
	    $ConvertCommand = $ConvertDir . "convert $UfileName  $DataDirName";
	    
	    print STDERR "Execute command: $ConvertCommand\n";
	    system $ConvertCommand;
	    @fileinfo = stat($B_FileName); 
	}
	else
	{
	    # Convert raw log file to B,E,S files
	    $ConvertCommand = $ConvertDir . "convert $UfileName ren $BaseFileName";
	    print STDERR "Execute command: $ConvertCommand\n";
	    system $ConvertCommand;
	    @fileinfo = stat($UfileName); 
	}


	if ( int($fileinfo[7]) != 0 ) 
	{    
	    # teqc doesn't like Ashtech D-files,  rename it or teqc will hang.
	    rename $D_FileName, "_no_teqc_" . $D_FileName;

	    # Convert  B,E,S files to the RINEX files
	    $ConvertCommand = $TeqcDir . "teqc -ash d +smooth  +nav $RinexNavFileName +obs $RinexObsFileName $B_FileName";
	    print STDERR "Execute command: $ConvertCommand\n";

	    system $ConvertCommand;

	    # Remove zero size RINEX NAV file, otherwise the teqc will complain later
	    @fileinfo = stat($RinexNavFileName); 
	    unlink $RinexNavFileName if ( int($fileinfo[7]) == 0 ) ;
	
	    # Run Quality check for the RINEX file
	    $QaSummary[$i] = RunQualityCheck(  $RinexObsFileName ,$ReportFileName);
	    
	    # Addd files to the  list for the archiving
	    push ( @ArchiveList, $RinexObsFileName);
	    push ( @ArchiveList, $RinexNavFileName) unless ( int($fileinfo[7]) == 0 ) ;
	}
	else
	{
	    $QaSummary[$i] = "R-file Conversion error     ";
	}

	# Addd files to the  list for the archiving
	push ( @ArchiveList, $UfileName);
	# Delete temporary B,E,S files
	# unlink $DataDirName . "B"  .$BaseFileName;
	# unlink $DataDirName . "E"  .$BaseFileName;
	# unlink $DataDirName . "S"  .$BaseFileName;
    }
}
#######################################################################################
#
# FUNCTION NAME: RunQualityCheck
# ARGUMENTS    : $Label,$Observables,$ReportFileName
# DESCRIPTION  : This function executes teqc utility to check the Quality of the 
#                data in the input RINEX file(s). It output the report like this:
#                U____B99.251 -> 01:26 01:30 1584/1582 (100%)
#                U____C99.251 -> 01:30 01:40 4186/4186 (100%)
#                U____D99.251 -> 01:40 01:50 4018/4186 (96%)
#                U____E99.251 -> 22:18 22:19 10/10 (100%)
#                U____F99.251 -> 22:56 22:58 28/30 (93%)
#                U____G99.251 -> 23:06 23:07 11/n/a (n/a%)
#                U____H99.251 -> 23:09 23:10 10/15 (67%)
#                U____I99.251 -> 23:11 23:13 26/30 (87%)
#                U____J99.251 -> 23:13 23:14 24/24 (100%)
#                Each line consists from the following fields:
#                1. Label
#                2. Time of the first epoch
#                3. Time of the last epoch
#                4. Number of observables in the file
#                5. Potential  number of observables in the file 
#                          calculated on base of the ephemeris 
#                          information
#                Ratio of the  last two fields.
#
#######################################################################################
sub RunQualityCheck
{
    my ($Observables,$ReportFileName) = @_ ;
    my ($Keyword, $StartYear, $StartMonth, $StartDay,  $StartTime,
        $EndYear, $EndMonth,  $EndDay,     $EndTime,
	$Length,  $Interval,  $ExpObs,     $HaveObs,    $PercObs) ;
    my ($Command) ;

# Execute teqc to generate the report file
    $Command = $TeqcDir . "teqc +qc -plot $Observables >/dev/null";
    print STDERR "Execute command: $Command\n";
    system $Command;

# Process Report File

    open(REPORT_FILE,"<$ReportFileName") or AbortScript ("Can not open $ReportFileName");

  PROCESS_REPORT: while( <REPORT_FILE> )
    {
	@LineItems = split( ' ',$_);
	last PROCESS_REPORT if ( defined $LineItems[0] && $LineItems[0] eq "SUM" ) ; # Find the summary line
    }
    close (REPORT_FILE);

    $Keyword = "";

     # Split the summary line to the items
    ($Keyword, $StartYear, $StartMonth, $StartDay,  $StartTime,
               $EndYear,   $EndMonth,   $EndDay,    $EndTime,
     $Length,  $Interval,  $ExpObs,     $HaveObs,   $PercObs) = @LineItems;
    
     if ( $Keyword eq "SUM")
     {
	 $Summ =  "$StartTime $EndTime $HaveObs/$ExpObs ($PercObs%)";
     }
     else
     {
	 $Summ = "No report Was generated";
     }
    return $Summ;
}

############################################################################
#
# Subroutine:  GenerateFilesReport
#
# Description: This subroutines generates the report on the files quality.
#
#
# Parameters:  None
#
############################################################################

sub GenerateFilesReport()
{
    my ($SessInfo) = @_;

    $last_hour=0;
    $last_minu=0;
    print LOG "Name         Week Day Time  Rcv/Dnld      First Last   Obs Ratio       DnSpd   Notes:\n";

    

    for($i=0; $i<$FilesFound;$i++)
    {
	$Notes = ' ';
	@fileinfo = stat($FileName[$i]);

	@FullPathName     = split /\//,$FileName[$i];
	$FileName[$i]     =  pop(@FullPathName);

	$dn_sz = int($fileinfo[7]/1024+0.5);

	$DnSpeed = 0;
	if ((defined $DownLoadTime) && ($DownLoadTime[$i] != 0) ) 
	{
	    $DnSpeed = int($fileinfo[7]*10/$DownLoadTime[$i]);
	}


	%wkdays = qw/1 Sun 2 Mon 3 Tue 4 Wed 5 Thu 6 Fri 7 Sat/;

	
	if ( $DataSource eq "stream" )
	{
	    $week = 9999;
	    ($sec,$minu,$hour,$mday,$mon,$year,$day,$yday) = gmtime( $FileDate[$i] );
	    $day = $wkdays{chop($day)};
	    $hour = sprintf("%02d",$hour);
	    $minu = sprintf("%02d",$minu);
	    $FileSize[$i] = sprintf("%06d",int($FileSize[$i]/1024+0.5));
	}
	
	if( $ReceiverType eq $RCV_UZ )
	{
	    $week = substr($FileDate[$i],0,5);
	    $day = $wkdays{chop($week)};
	    $hour = substr($FileDate[$i],5,8);
	    $hour = substr($hour,0,2);
	    $minu = substr($FileDate[$i],7,8);
	}
	
	if( $ReceiverType eq $RCV_Z12 )
	{
	    $week = $FileStartWn[$i];
	    ($sec,$minu,$hour,$mday,$mon,$year,$day,$yday) = GpsToNormalTime($FileStartWn[$i],$FileStartTime[$i]);
	    $day = $wkdays{chop($day)};
	    $hour = sprintf("%02d",$hour);
	    $minu = sprintf("%02d",$minu);
	    $FileSize[$i] = sprintf("%06d",int($FileSize[$i]/1024+0.5));
	}

	$sessdiff = ($minu-$last_minu) + 60*($hour-$last_hour);
	$last_hour=$hour;
	$last_minu=$minu;
	
	$Notes = "";

	if( $ReceiverType eq $RCV_UZ )
	{
	    if ($i>0 && ($sessdiff != $SessInfo)) {
		$Notes = '*** Time Err ';
	    }
	}

	if ($dn_sz !=  $FileSize[$i]) {
	    $Notes = $Notes .  ' *** Size Err';
	}
	
	$dn_sz = sprintf("%06d", $dn_sz);
	print LOG "$FileName[$i] $week $day $hour:$minu $FileSize[$i]/$dn_sz $QaSummary[$i]  $DnSpeed   $Notes\n";
    }

    # Compress data files 
    foreach $FileName (@ArchiveList) 
    {
	system "gzip $FileName";
    }

   $gmt = gmtime; 
   print LOG "\nCurr GMT time: $gmt\n";

    if( $ReceiverType eq $RCV_UZ )
    {
	($GPSwk) = Query("WKN");
	print LOG "Curr GPS Week: $GPSwk\n";
    }
    
    print LOG "\nFiles were downloaded to the following dir:\n";
    print LOG $CurrDir, "\n";
    

    $URLbase = ReadVar($ConfigFile,"URLbase","http://mzserver.ashtech.com/");
    @DirComponents =  split/\//,$CurrDir;
    $BaseDir = $DirComponents[$#DirComponents]; 
    $FullURL = $URLbase . $BaseDir;
    print LOG "\These files can be obtained via HTTP at\n";
    print LOG "$FullURL \n";
}

############################################################################
#
# Subroutine:  PrintStatus
#
# Description: 
#
# Parameters:  
#
############################################################################
sub PrintStatus
{
    print STDERR @_;
}

############################################################################
#
# Subroutine:  AbortScript
#
# Description: Aborts the script
#
# Parameters:  
#
############################################################################
sub AbortScript
{
    print LOG @_ , "\n";
    print STDERR @_, "\n";

    print LOG "Test aborted\n";

    close(LOG);
    die "Test aborted\n";
}

############################################################################
#
# Subroutine:  CloseConnection
#
# Description: Closes the connection to receiver
#
# Parameters:  
#
############################################################################

sub CloseConnection
{
    my($Connection) = @_;
    if ( $Connection  eq  "modem" )  
    {
	HangUp ();
    }
}

############################################################################
#
# Subroutine:  ReadConfiguration
#
# Description: Reads Configuration from arguments and config file
#
# Parameters:  
#
############################################################################

sub ReadConfiguration
{
    

    if ($#ARGV <1 ) {
	print STDERR "Correct usage is:\n";
	print STDERR "remote33.pl ConfigFile ReportFile\n";
	exit 1;
    }
    
    $ConfigFile   = $ARGV[0];
    $LogFile      = $ARGV[1];

    print STDERR "Script params: $ConfigFile $LogFile\n";

    $Title        = ReadVar($ConfigFile,"Title","No title specified");

    
    $DataSource   = ReadVar($ConfigFile,"DataSource");
    $Connection   = ReadVar($ConfigFile,"Connection");

    $ReceiverRate = ReadVar($ConfigFile,"ReceiverRate","7","[3,4,5,6,7,8,9]");
    $Configure    = ReadVar($ConfigFile,"Configure","no","[yes,no]");
    $SessDuration = ReadVar($ConfigFile,"SessDuration",10);

    
    if ( $Connection  eq  "modem" )  
    {
	$BaudRate     = ReadVar($ConfigFile,"ConnectionRate",38400,"[2400,4800,9600,19200,38400,57600,115200]");
	$PhoneNumber  = ReadVar($ConfigFile,"PhoneNumber");
	$InitString   = ReadVar($ConfigFile,"PCModemInitString","ATS0=0S12=25&H1X4E0M3&D0&W");
    }
    
    if ( $Connection  eq  "tcp" )  
    {
	$HostName     = ReadVar($ConfigFile,"HostName","www.yahoo.com");
    }

    $ScriptDir   = ReadVar($ConfigFile,"ScriptDir",".");
    $ZmodemDir = ReadVar($ConfigFile,"ZmodemDir","$ScriptDir/zmodem/");
    $ConvertDir = ReadVar($ConfigFile,"ConvertDir","$ScriptDir/convert/");
    $TeqcDir = ReadVar($ConfigFile,"TeqcDir","$ScriptDir/teqc/");

    $MinSvNum      = ReadVar($ConfigFile,"MinSvNum",3,0,12);
    $ElevationMask = ReadVar($ConfigFile,"ElevationMask",10,0,90);
    $RecInt        = ReadVar($ConfigFile,"RecInt",0.5,0.1,1000);

    $MultiPathMode = ReadVar($ConfigFile,"MultiPathMode",2, "[0,2]");
    $GPSSyncMode   = ReadVar($ConfigFile,"GPSSyncMode","N", "[Y,N]");

    $DoSessionProg = ReadVar($ConfigFile,"DoSessionProg","no", "[yes,no]");
}


