#!/usr/bin/perl
#
# FILE:        setup_mdm.pl
#
# DESCRIPTION: This scripts tests and sets the receiver modem 
#              settings
#
# USED  TOOLS:
#
#  $Log: setup_mdm.pl,v $
#  Revision 1.9  1999/10/04 23:51:43  phong
#  Moved VerifyModemSetup, SetupModem, and GetModemCfg sub-routines from setup_mdm.pl to mdm-lib.pl
#
#  Revision 1.8  1999/10/02 01:40:26  phong
#  Created two functions from this script so other scripts can call

#  VerifyModemSetup () and SetupModem () as neede.
#
#  Revision 1.7  1999/09/30 20:36:44  phong
#  Created new Modem related functions so everyone can use it to dial a phone number.
#  All modem related functions are now in mdm-lib.pl. The file dial.pl is only for
#  modem dialing testing.
#  Created new mail-lib.pl which contain email related functions. You can use these
#  functions to compressed a log file and send it via email.
#
#  Revision 1.6  1999/09/29 19:46:01  phong
#  Modified Files: dial.pl mdm-lib.pl modem.pl setup_mdm.pl.
#  Output all messages to STDERR and then convert STDERR to a log file.
#
#  Revision 1.5  1999/09/28 00:58:50  phong
#  Modified Files: dial.pl mdm-lib.pl modem.pl setup_mdm.pl.
#  Changed the "modem_log" to "modem_log.txt" for windows text displaying.
#  Added $EmailList as a parameter string to SendMail subroutine.
#
#  Revision 1.4  1999/09/24 01:12:52  phong
#  When we send a new modem configuration string we
#  have to cycle the receiver power. For some reason
#  $PASHS,MDM,INI cannot do the job even when the response
#  is good.
#
#  Revision 1.3  1999/09/24 01:06:52  phong
#  Added the check for "INITOK" and accept it as a valid response.
#
#  Revision 1.2  1999/09/23 23:15:09  phong
#  New modification for microz modem testing.
#
#  Revision 1.1  1999/09/10 18:58:44  serg
#  Initial revision
#
#
#############################################################################

require "ash-lib.pl";  # This library contains some useful subroutines
                       # like Connect(), Query(), Command()
require "mdm-lib.pl";  # This library contains some useful subroutines
                       # for all modem related scripts.
require "mail-lib.pl"; # This library contains some useful subroutines
                       # for email related scripts.



#$ModemPort          = "C";
#$BaudRate           = "7";


if ($#ARGV > 1)
{
    $MdmPrt          = $ARGV[0];
    $BdRate          = $ARGV[1];
    $EmailFile       = $ARGV[2];
}
else 
{
    $Line = "** Need the Modem Port, its Baud Rate, and The email log file." . localtime () . "\n";
    print STDERR $Line;
    exit (3);
}

#-----------------------------------------------------------------------------
# Connect to receiver at the baud rate 9600
#----------------------------------------------------------------------------

Connect("5") or die "Can not connect to receiver";


#-----------------------------------------------------------------------------
# Check the modem setup configuration
#----------------------------------------------------------------------------

if (VerifyModemSetup ($MdmPrt, $BdRate))
{
    exit (0);
}
else
{
    if (SetupModem ()) 
    {        
        PrintTimeHeader ($EmailFile, "** Modem configuration changed at ", "\n");
        exit (1);
    }
    else
    {
        PrintTimeHeader ($EmailFile, "** Modem Initialization Fails at ", "\n");
        exit (2);
    }   
}


