#!/usr/bin/perl
#    Ashtech remote receiver access tools
#    Copyright (C) 1999  Magellan Coropration/Ashtech Precision Products 
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
#     FILE:        uzdld.pl
#
#     DESCRIPTION: This PERL script downloads the data from Micro Z
#
#     $Id: uzdld.pl,v 1.4 1999/12/10 21:32:04 serg Exp $
#
#
#############################################################################

############################################################################
#
# Subroutine:  GetFilesListuZ
#
# Description: This subroutine obtains the list of the recorded files using 
#              the query $PASHQ,FIF,<n> where n is the index of the first 
#              file.
#              Each $PASHR,FIF can contain information for up to 10 files.
#              Format of the $PASHR,FIF is the following
#              $PASHR,FIF,<SpaceLeft>,<TotalFiles>,<FilesInThisMessage>,\
#              [<FileIdx>,<FileName>,<FileDate>,<FileSize>] - this group
#              is repeated <FilesInThisMessage> times
#
# Parameters:  None
#
############################################################################
sub GetFilesListuZ
{
    $FilesFound = 0;
    $TotalFiles = 10;
    
  COLLECTDIR: while ( $FilesFound < $TotalFiles)
  {
      @Response = Query("FIF," . $FilesFound);
      
      if ( $#Response > 0 )
      {
	  ($BytesLeft,$TotalFiles,$FilesInThisQuery,@Files) = @Response;
	  for($i=0; $i<$FilesInThisQuery;$i++)
	  {
	      $FileName[$FilesFound] = $Files[$i*5+1];
	      $FileDate[$FilesFound] = $Files[$i*5+2];
	      $FileSize[$FilesFound] = $Files[$i*5+3];
	      $FileOpen[$FilesFound] = $Files[$i*5+4];
	      $FileDeleted[$FilesFound] = 0;
	      last COLLECTDIR if $FileOpen[$FilesFound];
	      $FilesFound ++;
	  }
      }
      else
      {
	  last COLLECTDIR;
      }
  }

    return $FilesFound ;
}

############################################################################
#
# Subroutine:  DownloadFilesuZ
#
# Description: This subroutines uses the utility rz to download receiver U
#              files via ZMODEM.
#              The recieiver initiates the transfer upon receiving command:
#              $PASHS,FIL,DOWNLOAD,<file name>,ZMODEM   
#
# Parameters:  None
#
############################################################################
sub DownloadFilesuZ()
{
    $DownloadError = 0;
    DOWNLOAD: for($i=0; $i<$FilesFound;$i++)
    {
	next DOWNLOAD if ( $FileDeleted[$i] );

	if ( Command("FIL,DOWNLOAD," . $FileName[$i] . ",ZMODEM") ne "NAK" )
	{
	    $DnStart = time();

	    # start rz utility, crash recovery set to ON
	    $DownLoadCmd = "$ZmodemDir". "rz";
	    print STDERR "*>$DownLoadCmd\n";

	    $DownloadError = system $DownLoadCmd;
	    if ( $DownloadError  )
	    {
		AbortZmodem();
	    }

	    last DOWNLOAD if $DownloadError;
	    
            # delete downloaded file from receiver
	    if ( Command ("FIL,D,0") eq "ACK" ) 
	    {
		$FileDeleted[$i] = 1;
	    }
	    $DnEnd = time();
	    $DownLoadTime[$i] = $DnEnd - $DnStart;
	}
    }
    return $DownloadError;
}
############################################################################
#
# Subroutine:  SetUpSessionsuZ
#
# Description: 
#
# Parameters:  Session duration in minutes
#
############################################################################
sub SetupSessionsuZ
{
    my ($SessInfo) = @_;

    $iha = int($SessInfo/60);
    $ima = $SessInfo - $iha*60;

    $ses_nm='A';
    $ses_flg='Y';
    $init=0;
    $end=1;
    $ses_int  = $RecInt;
    $ses_mask = $ElevationMask;
    $ses_min  = $MinSvNum;
    $ses_type=0;
    $sw=0;
    $ih=0;
    $eh=0;
    $reprg=0;


    print LOG "\nSessions:\n";
    print LOG "    Start    End      RCI   Mask MIN TYPE\n";
    for $j(0..23) {
	@Rspn = Query("SSN," . chr($j+65));
	if ($#Rspn > 0) {
	    for $i(5..12) {
		if ($i==12) {
		    $Rspn[$i] =~ tr/\r/ /;
		    print LOG "$Rspn[$i]\n";
		}
		else {
		    print LOG "$Rspn[$i] ";
		}
	    }
	}
	($sh,$sm,$ss) = split /:/, $Rspn[7];
	($fh,$fm,$fs) = split /:/, $Rspn[8];
	$start_time = 3600*$sh+60*$sm+$ss;
	$end_time = 3600*$fh+60*$fm+$fs;
	if ($j==0) {
	    $first_starthr = $sh; 
	}
	if ($end_time == 0 && $end_time < $start_time) {
	    $end_time = 3600*24+60*$fm+$fs;
	}
	$duration = $end_time - $start_time ; # +1
	if ($reprg==0){
	    if (($duration != 60*$SessInfo) || (int($Rspn[9]) != $ses_int) || (int($Rspn[10]) != $ses_mask) || ($Rspn[11] != $ses_min)){
		$reprg=1;
	    }
	}
    }
    
    @GMtime = gmtime(time);
    $ih = sprintf("%02d",$GMtime[2]+1);
    $im = '00';
    $eh = $ih + $iha;
    $em = $im + $ima;
    print STDERR "ih = $ih, eh = $eh, sh = $first_starthr \n";
    if ($SessInfo != 60){
	if (($ih-$first_starthr)>3) {
	    $reprg=2;
	}
    }
 
#----------------------------------------------------------------------------
# If session pattern not match, then reprogram sessions
#---------------------------------------------------------------------------
    if ($reprg > 0) {
	print STDERR "reprg = $reprg \n";  
	Command("SES,DEL");
	Command("SES,PAR,Y,257,0");
	for $i(0..23){
#	    $sw = 1;
	    $sw = 0;
	    if ($ih >= 24) {
		$ih -= 24;
	    } 
	    if ($eh >= 24) {
		$eh -= 24;
	    } 
	    if ($im >= 60) {
		$im -= 60;
		$ih++; 
	    } 
	    if ($em >= 60) {
		$em -= 60;
		$eh++; 
     } 
	    $ih = sprintf("%02d",$ih);
	    $im = sprintf("%02d",$im);
	    $eh = sprintf("%02d",$eh);
	    $em = sprintf("%02d",$em);
	    Command("SES,SET," . chr($i+65) . "," . $ses_flg . "," . $ih . $im . "0" . $sw . "," . $eh . $em . "00," . $ses_int . "," . $ses_mask . "," . $ses_min . "," . $ses_type);
	    $ih = $ih + $iha;
	    $im = $im + $ima;
	    $eh += $iha;
	    $em += $ima;
	}
	print LOG "\nReprogramed Sessions:\n";
	print LOG "    Start    End      RCI   Mask MIN TYPE\n";
	@Response00 = QuerySes("SES");
    }  

    Command("SAV,Y");
}

# This line has to be the very last line of the module, for some 
# obscured for me reason 
1; #return true , to please the require satement in main module

