#!/usr/bin/perl
#    Ashtech remote receiver access tools
#    Copyright (C) 1999  Magellan Coropration/Ashtech Precision Products 
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
#    FILE:        z12dld.pl
#
#    DESCRIPTION: This PERL script downloads the data from  Z-12,
#
#    $Id: z12dld.pl,v 1.8 2000/01/05 21:28:16 carmen Exp $
#
#
#############################################################################


############################################################################
#
# Subroutine:   GetFilesListZ12
#
# Description: This subroutine obtains the list of the recorded files using 
#              the FAT information located on top of receiver memory.
#              First we download that block of memory and then parse it.
#
# Parameters:  None
#
############################################################################

# Some constants 

$MaxFileNumbers = 100;  # For some versions of Z-12 it's 10

# Binary structure of the FAT entry
$FileHeaderStruct = "N N A4 C C n N C A3 A1 A33 n N n n C C C C C C n n n";


sub GetFilesListZ12
{
    $FatInfoFile    = "fatinfo.dat";
    $MemInfoSize    = 10;
    $FileInfoSize   = 40;
    $FatInfoSize    = $MemInfoSize + $FileInfoSize * $MaxFileNumbers;

    # Download the headers memory region
 
    print STDERR  "Download File $FatInfoFile\n";
    $QueryString = sprintf("BLK,%X,%X",0,$FatInfoSize*2);
    Query($QueryString);
    $rc = system("$ZmodemDir" . "rx  $FatInfoFile");

    if ($rc) {
	print STDERR  "FAT Download error\n";
	return 0;
    }

    # Now read and process downloaded data

    print STDERR  "Process image header\n";
    open(FAT_INFO,$FatInfoFile) or die "Can not open fat finfo file\n";

    # Process the  image header
    sysread(FAT_INFO,$Buff,$MemInfoSize*2);
    @MemHeader = unpack("n10",$Buff);
    $FilesFound = $MemHeader[9] + 1;

    # Process the the file headers
    for($i = 0; $i < $FilesFound; $i++)
    { 
	sysread(FAT_INFO,$FileHeader[$i],$FileInfoSize*2);

	# Unpack file header
	@FileHdr = unpack($FileHeaderStruct,$FileHeader[$i]);
	print STDERR  join(',',@FileHdr), "\n";
	($SegBeg[$i],$WordCount[$i],$SegName[$i],$RangerMode,$Tmp,$FileStartWn[$i],$FileStartTime[$i],$RcvrType,$Tmp2,$SessName,$Project,$SegWn,$SegTime) = @FileHdr;
	
	# Store file information for later use in downloads
	$FileName[$i] = ComposeRFileName($SegName[$i],$i,$SegWn,$SegTime);
	$FileSize[$i] = (int($WordCount[$i]*2 / 1024)+1)*1024 + 8170;

	if( $WordCount[$i] == 0 )   # Skip zero sized fi 
	{ 
	    $FileDownloaded[$i] = -1;
	}

    }

    close(FAT_INFO);
    unlink($FatInfoFile);

    --$FilesFound; # Do not download the last file, it's current

    return $FilesFound;
}

############################################################################
#
# Subroutine:  ComposeFileName
#
# Description: This subroutines composes R file name
#
# Parameters:  None
#
############################################################################

sub ComposeRFileName{
    my($SiteName,$SessNum,$SegWn,$SegTime) = @_;
    my($sec,$min,$hour,$mday,$mon,$year,$wday,$yday) = GpsToNormalTime($SegWn,$SegTime);
    my($SessId,$FileName);

    $SiteName =~tr/\?/_/; # Replace ???? with ____
	
    if ( $SessNum < 26 ) 
    {
	$SessId = chr(65+$SessNum) . sprintf("%02d",$year % 100);
    }
    else
    {
	$SessId = chr(65+($SessNum / 10)) . chr(65+($SessNum % 10)) . $year % 10;
    }

    $FileName = "R" . $SiteName . $SessId  . sprintf(".%03d",$yday+1);

    return $FileName;
} 

############################################################################
#
# Subroutine:  DownloadFiles
#
# Description: This subroutines uses the utility rx to download receiver R
#              files via XMODEM.
#              The recieiver initiates the transfer upon receiving command:
#              $PASHQ,BLK,start addr,len in bytes
#
# Parameters:  None
#
############################################################################
sub DownloadFilesFromZ12()
{
    DOWNLOAD: for($i = 0; $i < $FilesFound; $i++)
    { 
	next DOWNLOAD if ( $FileDownloaded[$i] );

	# Download the memory image

	print STDERR  "Download File $FileName[$i].body\n";
	$QueryString = sprintf("BLK,%X,%X",$SegBeg[$i],$WordCount[$i]*2);
	Query($QueryString);

	$DnStart = time();
	$DownloadError = system("$ZmodemDir" .  "rx  $FileName[$i].body");
	last DOWNLOAD if $DownloadError;

	@RcvrId = Query("RID");
	
	# Compose the R-file header using the FAT information
	
	open(R_FILE,">$FileName[$i].head");
	
	$ImageHeader = BuildImageHeader(@RcvrId);
	syswrite(R_FILE,$ImageHeader,length($ImageHeader));

	$FatInfo  = "";
	BuildFat($FileHeader[$i],$i);

	syswrite(R_FILE,$FatInfo,length($FatInfo));

	close(R_FILE);
	
        # Put the head and body together
	system("cat $FileName[$i].head $FileName[$i].body > $FileName[$i]");

	# Remove temporary files
	unlink("$FileName[$i].head");
	unlink("$FileName[$i].body");

	$DnEnd = time();
	$DownLoadTime[$i] = $DnEnd - $DnStart;
	$FileDownloaded[$i] = 1;
    }
    
      if ( not $DownloadError )
      {
	  for($i=0; $i<$FilesFound;$i++)
	  {
	      Command("FIL,D,0","no","no"); # No checksum, no wait for $PASHR,ACK
	      print STDERR "File was deleted\n" if WaitFor(3*60,'C');  # Wai up to 3 min for 'C' character
	  }
      }
      else
      {
	  sleep 10; # Give XMODEM time to timeout
      }

      return $DownloadError;
}

############################################################################
#
# Subroutine: BuildImageHeader
#
# Description: 
#
############################################################################
sub BuildImageHeader
{
    @Tmp = @_;

    shift(@Tmp);pop(@Tmp); # Get rid of first and last fields in RID response
    $VersionInfo = join(",",@Tmp);
    
    $ImageHeader="";
	
    $ImageHeader .= pack('a20','Z-12');
    $ImageHeader .= pack('a20',$VersionInfo);
    $ImageHeader .= pack('cc',1,0);
    $ImageHeader .= pack('N',$TotalMemory);
    $ImageHeader .= pack('N',$FATStart);
    $ImageHeader .= pack('N',$FATEnd);
    $ImageHeader .= pack('N',$PhotoStart);
    $ImageHeader .= pack('N',$PhotoEnd);
    $ImageHeader .= pack('N',$AlmStart);
    $ImageHeader .= pack('N',$AlmEnd);
    $ImageHeader .= pack('N',$DataStart);
    $ImageHeader .= pack('N',$DataEnd);
    $ImageHeader .= pack('a72','SPARE');

    return $ImageHeader;
}

############################################################################
#
# Subroutine: BuildFat
#
# Description: 
#
############################################################################

sub BuildFat
{
    ($FileInfo,$SessNum) = @_;

    # We need to replace the original start address, with offset in our
    # file.

    # Split the header to start address and remainder
    ($SegBeg,@Remainder) = unpack($FileHeaderStruct,$FileInfo);
    
    $Remainder[8] = chr(65+$SessNum);

    # Calculate new value of the start address
    $SegBeg = 10 + $MaxFileNumbers*$FileInfoSize;
        
    # FAT header 
    $FatInfo  = pack(a19c,"",1);   

    # Our file always first
    $FatInfo .= pack($FileHeaderStruct,$SegBeg,@Remainder);

    # Then 0 for the remainding files
    $RemainderSize = $FileInfoSize*2*($MaxFileNumbers-1);
    $FatInfo .= pack("a$RemainderSize","");

    return $FatInfo;
}

sub SetupSessionsZ12()
{
    my($SessDuration) = @_;

    $RecordingInterval = 5; 
    $ElevationMask     = 0;	

    $rci   = $RecordingInterval * 10; # Recording interval
    $elm   = $ElevationMask;
    $msv   = 1;
    $rectype = 0;

    Command("SCRN,70,Y0010000","yes","no");
    print STDERR "Got Ack\n" if WaitFor(10,chr(6));

    $start = 1;
    for $i(0..23)
    {

	$rectype = 0 if ($i % 2) == 0;
	$rectype = 3 unless ($i % 2) == 0;

	$end   = $start + $SessDuration*60 - 1;
	$Command = sprintf("SCRN,%02d,%05d%05d%04d%02d%01d%01dY",71+$i,$start,$end,$rci,$elm,$msv,$rectype); 
	Command($Command,"yes","no");
	print STDERR "Got Ack\n" if WaitFor(10,chr(6));
	$start += $SessDuration*60;
    }

}

# This line has to be the very last line of the module, for some 
# obscured for me reason 
1; #return true , to please the require satement in main module


